@@ -5,7 +5,7 @@ minion modules.
 '''
 
 # Import python libs
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, unicode_literals
 
 import os
 import sys
@@ -20,18 +20,19 @@ import salt.minion
 import salt.output
 import salt.payload
 import salt.transport
+import salt.transport.client
 import salt.utils.args
+import salt.utils.files
 import salt.utils.jid
+import salt.utils.kinds as kinds
 import salt.utils.minion
+import salt.utils.profile
+import salt.utils.stringutils
 import salt.defaults.exitcodes
-from salt.log import LOG_LEVELS
-from salt.utils import is_windows
-from salt.utils import print_cli
-from salt.utils import kinds
-from salt.utils import activate_profile
-from salt.utils import output_profile
-from salt.utils.process import MultiprocessingProcess
 from salt.cli import daemons
+from salt.log import LOG_LEVELS
+from salt.utils.platform import is_windows
+from salt.utils.process import MultiprocessingProcess
 
 try:
     from raet import raeting, nacling
@@ -46,7 +47,7 @@ except ImportError:
     pass
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 
 # Custom exceptions
 from salt.exceptions import (
@@ -99,9 +100,12 @@ class BaseCaller(object):
         # be imported as part of the salt api doesn't do  a
         # nasty sys.exit() and tick off our developer users
         try:
-            self.minion = salt.minion.SMinion(opts)
+            if self.opts.get('proxyid'):
+                self.minion = salt.minion.SProxyMinion(opts)
+            else:
+                self.minion = salt.minion.SMinion(opts)
         except SaltClientError as exc:
-            raise SystemExit(str(exc))
+            raise SystemExit(six.text_type(exc))
 
     def print_docs(self):
         '''
@@ -114,13 +118,13 @@ class BaseCaller(object):
                     docs[name] = func.__doc__
         for name in sorted(docs):
             if name.startswith(self.opts.get('fun', '')):
-                print_cli('{0}:\n{1}\n'.format(name, docs[name]))
+                salt.utils.stringutils.print_cli('{0}:\n{1}\n'.format(name, docs[name]))
 
     def print_grains(self):
         '''
         Print out the grains
         '''
-        grains = salt.loader.grains(self.opts)
+        grains = self.minion.opts.get('grains') or salt.loader.grains(self.opts)
         salt.output.display_output({'local': grains}, 'grains', self.opts)
 
     def run(self):
@@ -129,14 +133,14 @@ class BaseCaller(object):
         '''
         profiling_enabled = self.opts.get('profiling_enabled', False)
         try:
-            pr = activate_profile(profiling_enabled)
+            pr = salt.utils.profile.activate_profile(profiling_enabled)
             try:
                 ret = self.call()
             finally:
-                output_profile(pr,
-                               stats_path=self.opts.get('profiling_path',
-                                                        '/tmp/stats'),
-                               stop=True)
+                salt.utils.profile.output_profile(
+                    pr,
+                    stats_path=self.opts.get('profiling_path', '/tmp/stats'),
+                    stop=True)
             out = ret.get('out', 'nested')
             if self.opts['print_metadata']:
                 print_ret = ret
@@ -145,10 +149,14 @@ class BaseCaller(object):
                 print_ret = ret.get('return', {})
             salt.output.display_output(
                     {'local': print_ret},
-                    out,
-                    self.opts)
+                    out=out,
+                    opts=self.opts,
+                    _retcode=ret.get('retcode', 0))
+            # _retcode will be available in the kwargs of the outputter function
             if self.opts.get('retcode_passthrough', False):
                 sys.exit(ret['retcode'])
+            elif ret['retcode'] != salt.defaults.exitcodes.EX_OK:
+                sys.exit(salt.defaults.exitcodes.EX_GENERIC)
         except SaltInvocationError as err:
             raise SystemExit(err)
 
@@ -158,12 +166,18 @@ class BaseCaller(object):
         '''
         ret = {}
         fun = self.opts['fun']
-        ret['jid'] = salt.utils.jid.gen_jid()
+        ret['jid'] = salt.utils.jid.gen_jid(self.opts)
         proc_fn = os.path.join(
             salt.minion.get_proc_dir(self.opts['cachedir']),
             ret['jid']
         )
         if fun not in self.minion.functions:
+            docs = self.minion.functions['sys.doc']('{0}*'.format(fun))
+            if docs:
+                docs[fun] = self.minion.functions.missing_fun_string(fun)
+                ret['out'] = 'nested'
+                ret['return'] = docs
+                return ret
             sys.stderr.write(self.minion.functions.missing_fun_string(fun))
             mod_name = fun.split('.')[0]
             if mod_name in self.minion.function_errors:
@@ -189,7 +203,7 @@ class BaseCaller(object):
                     no_parse=self.opts.get('no_parse', [])),
                 data=sdata)
             try:
-                with salt.utils.fopen(proc_fn, 'w+b') as fp_:
+                with salt.utils.files.fopen(proc_fn, 'w+b') as fp_:
                     fp_.write(self.serial.dumps(sdata))
             except NameError:
                 # Don't require msgpack with local
@@ -200,11 +214,32 @@ class BaseCaller(object):
                     'Do you have permissions to '
                     'write to {0} ?\n'.format(proc_fn))
             func = self.minion.functions[fun]
+            data = {
+              'arg': args,
+              'fun': fun
+            }
+            data.update(kwargs)
+            executors = getattr(self.minion, 'module_executors', []) or \
+                        salt.utils.args.yamlify_arg(
+                            self.opts.get('module_executors', '[direct_call]')
+                        )
+            if self.opts.get('executor_opts', None):
+                data['executor_opts'] = salt.utils.args.yamlify_arg(
+                    self.opts['executor_opts']
+                )
+            if isinstance(executors, six.string_types):
+                executors = [executors]
             try:
-                ret['return'] = func(*args, **kwargs)
+                for name in executors:
+                    fname = '{0}.execute'.format(name)
+                    if fname not in self.minion.executors:
+                        raise SaltInvocationError("Executor '{0}' is not available".format(name))
+                    ret['return'] = self.minion.executors[fname](self.opts, data, func, args, kwargs)
+                    if ret['return'] is not None:
+                        break
             except TypeError as exc:
                 sys.stderr.write('\nPassed invalid arguments: {0}.\n\nUsage:\n'.format(exc))
-                print_cli(func.__doc__)
+                salt.utils.stringutils.print_cli(func.__doc__)
                 active_level = LOG_LEVELS.get(
                     self.opts['log_level'].lower(), logging.ERROR)
                 if active_level <= logging.DEBUG:
@@ -212,21 +247,35 @@ class BaseCaller(object):
                     sys.stderr.write(trace)
                 sys.exit(salt.defaults.exitcodes.EX_GENERIC)
             try:
-                ret['retcode'] = sys.modules[
+                retcode = sys.modules[
                     func.__module__].__context__.get('retcode', 0)
             except AttributeError:
-                ret['retcode'] = 1
+                retcode = salt.defaults.exitcodes.EX_GENERIC
+
+            if retcode == 0:
+                # No nonzero retcode in __context__ dunder. Check if return
+                # is a dictionary with a "result" or "success" key.
+                try:
+                    func_result = all(ret['return'].get(x, True)
+                                      for x in ('result', 'success'))
+                except Exception:
+                    # return data is not a dict
+                    func_result = True
+                if not func_result:
+                    retcode = salt.defaults.exitcodes.EX_GENERIC
+
+            ret['retcode'] = retcode
         except (CommandExecutionError) as exc:
             msg = 'Error running \'{0}\': {1}\n'
             active_level = LOG_LEVELS.get(
                 self.opts['log_level'].lower(), logging.ERROR)
             if active_level <= logging.DEBUG:
                 sys.stderr.write(traceback.format_exc())
-            sys.stderr.write(msg.format(fun, str(exc)))
+            sys.stderr.write(msg.format(fun, exc))
             sys.exit(salt.defaults.exitcodes.EX_GENERIC)
         except CommandNotFoundError as exc:
             msg = 'Command required for \'{0}\' not found: {1}\n'
-            sys.stderr.write(msg.format(fun, str(exc)))
+            sys.stderr.write(msg.format(fun, exc))
             sys.exit(salt.defaults.exitcodes.EX_GENERIC)
         try:
             os.remove(proc_fn)
@@ -286,11 +335,14 @@ class ZeroMQCaller(BaseCaller):
         '''
         Return the data up to the master
         '''
-        channel = salt.transport.Channel.factory(self.opts, usage='salt_call')
+        channel = salt.transport.client.ReqChannel.factory(self.opts, usage='salt_call')
         load = {'cmd': '_return', 'id': self.opts['id']}
         for key, value in six.iteritems(ret):
             load[key] = value
-        channel.send(load)
+        try:
+            channel.send(load)
+        finally:
+            channel.close()
 
 
 def raet_minion_run(cleanup_protecteds):
@@ -366,10 +418,14 @@ class RAETCaller(BaseCaller):
                 self.process.terminate()
             salt.output.display_output(
                     {'local': print_ret},
-                    ret.get('out', 'nested'),
-                    self.opts)
+                    out=ret.get('out', 'nested'),
+                    opts=self.opts,
+                    _retcode=ret.get('retcode', salt.defaults.exitcodes.EX_OK))
+            # _retcode will be available in the kwargs of the outputter function
             if self.opts.get('retcode_passthrough', False):
                 sys.exit(ret['retcode'])
+            elif ret['retcode'] != salt.defaults.exitcodes.EX_OK:
+                sys.exit(salt.defaults.exitcodes.EX_GENERIC)
 
         except SaltInvocationError as err:
             raise SystemExit(err)
@@ -410,7 +466,7 @@ class RAETCaller(BaseCaller):
                                    name='manor',
                                    lanename=lanename,
                                    dirpath=sockdirpath))
-        log.debug("Created Caller Jobber Stack {0}\n".format(stack.name))
+        log.debug("Created Caller Jobber Stack %s\n", stack.name)
 
         return stack
 
