@@ -648,7 +648,7 @@ def _get_properties(path="", method="GET", forced_params=None):
     # Browse all path elements but last
     for elem in path_levels[:-1]:
         search_path += '/' + elem
-        # Lookup for a dictionnary with path = "requested path" in list" and return its children
+        # Lookup for a dictionary with path = "requested path" in list" and return its children
         sub = (item for item in sub if item["path"] == search_path).next()['children']
     # Get leaf element in path
     search_path += '/' + path_levels[-1]
@@ -658,8 +658,6 @@ def _get_properties(path="", method="GET", forced_params=None):
         props = sub['info'][method]['parameters']['properties'].keys()
     except KeyError as exc:
         log.error('method not found: "%s"', exc)
-    except:
-        raise
     for prop in props:
         numerical = re.match(r'(\w+)\[n\]', prop)
         # generate (arbitrarily) 10 properties for duplicatable properties identified by:
@@ -728,6 +726,9 @@ def create_node(vm_, newid):
             if prop in vm_:  # if the property is set, use it for the VM request
                 newnode[prop] = vm_[prop]
 
+        if 'pubkey' in vm_:
+            newnode['ssh-public-keys'] = vm_['pubkey']
+
         # inform user the "disk" option is not supported for LXC hosts
         if 'disk' in vm_:
             log.warning('The "disk" option is not supported for LXC hosts and was ignored')
@@ -893,7 +894,7 @@ def destroy(name, call=None):
 
         # required to wait a bit here, otherwise the VM is sometimes
         # still locked and destroy fails.
-        time.sleep(1)
+        time.sleep(3)
 
         query('delete', 'nodes/{0}/{1}'.format(
             vmobj['node'], vmobj['id']
