@@ -63,6 +63,7 @@ from salt.ext import six
 def __virtual__():
     return HAS_HYPCHAT
 
+
 log = logging.getLogger(__name__)
 
 _DEFAULT_API_URL = 'https://api.hipchat.com'
@@ -244,13 +245,13 @@ def start(token,
             - ``html``: send the output as HTML
             - ``code``: send the output as code
 
-        This can be overriden when executing a command, using the ``--out-type`` argument.
+        This can be overridden when executing a command, using the ``--out-type`` argument.
 
         .. versionadded:: 2017.7.0
 
     outputter: ``nested``
         The format to display the data, using the outputters available on the CLI.
-        This argument can also be overriden when executing a command, using the ``--out`` option.
+        This argument can also be overridden when executing a command, using the ``--out`` option.
 
         .. versionadded:: 2017.7.0
 
