@@ -103,6 +103,20 @@ class SysLogHandler(ExcInfoOnLogLevelFormatMixIn, logging.handlers.SysLogHandler
     '''
     Syslog handler which properly handles exc_info on a per handler basis
     '''
+    def handleError(self, record):
+        '''
+        Override the default error handling mechanism for py3
+        Deal with syslog os errors when the log file does not exist
+        '''
+        handled = False
+        if sys.stderr and sys.version_info >= (3, 5, 4):
+            t, v, tb = sys.exc_info()
+            if t.__name__ in 'FileNotFoundError':
+                sys.stderr.write('[WARNING ] The log_file does not exist. Logging not setup correctly or syslog service not started.\n')
+                handled = True
+
+        if not handled:
+            super(SysLogHandler, self).handleError(record)
 
 
 class RotatingFileHandler(ExcInfoOnLogLevelFormatMixIn, logging.handlers.RotatingFileHandler, NewStyleClassMixIn):
