@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
 
     salt.log.setup
@@ -48,6 +48,7 @@ from salt.log.handlers import (TemporaryLoggingHandler,
                                QueueHandler)
 from salt.log.mixins import LoggingMixInMeta, NewStyleClassMixIn
 
+from salt.utils.ctx import RequestContext
 
 LOG_LEVELS = {
     'all': logging.NOTSET,
@@ -117,6 +118,7 @@ __EXTERNAL_LOGGERS_CONFIGURED = False
 __MP_LOGGING_LISTENER_CONFIGURED = False
 __MP_LOGGING_CONFIGURED = False
 __MP_LOGGING_QUEUE = None
+__MP_LOGGING_LEVEL = GARBAGE
 __MP_LOGGING_QUEUE_PROCESS = None
 __MP_LOGGING_QUEUE_HANDLER = None
 __MP_IN_MAINPROCESS = multiprocessing.current_process().name == 'MainProcess'
@@ -164,6 +166,7 @@ def is_mp_logging_configured():
 def is_extended_logging_configured():
     return __EXTERNAL_LOGGERS_CONFIGURED
 
+
 # Store a reference to the temporary queue logging handler
 LOGGING_NULL_HANDLER = __NullLoggingHandler(logging.WARNING)
 
@@ -304,6 +307,18 @@ class SaltLoggingClass(six.with_metaclass(LoggingMixInMeta, LOGGING_LOGGER_CLASS
     def _log(self, level, msg, args, exc_info=None, extra=None,  # pylint: disable=arguments-differ
              exc_info_on_loglevel=None):
         # If both exc_info and exc_info_on_loglevel are both passed, let's fail
+        if extra is None:
+            extra = {}
+
+        current_jid = RequestContext.current.get('data', {}).get('jid', None)
+        log_fmt_jid = RequestContext.current.get('opts', {}).get('log_fmt_jid', None)
+
+        if current_jid is not None:
+            extra['jid'] = current_jid
+
+        if log_fmt_jid is not None:
+            extra['log_fmt_jid'] = log_fmt_jid
+
         if exc_info and exc_info_on_loglevel:
             raise RuntimeError(
                 'Only one of \'exc_info\' and \'exc_info_on_loglevel\' is '
@@ -334,6 +349,12 @@ class SaltLoggingClass(six.with_metaclass(LoggingMixInMeta, LOGGING_LOGGER_CLASS
                    func=None, extra=None, sinfo=None):
         # Let's remove exc_info_on_loglevel from extra
         exc_info_on_loglevel = extra.pop('exc_info_on_loglevel')
+
+        jid = extra.pop('jid', '')
+        if jid:
+            log_fmt_jid = extra.pop('log_fmt_jid')
+            jid = log_fmt_jid % {'jid': jid}
+
         if not extra:
             # If nothing else is in extra, make it None
             extra = None
@@ -392,6 +413,7 @@ class SaltLoggingClass(six.with_metaclass(LoggingMixInMeta, LOGGING_LOGGER_CLASS
             logrecord.exc_info_on_loglevel_formatted = None
 
         logrecord.exc_info_on_loglevel = exc_info_on_loglevel
+        logrecord.jid = jid
         return logrecord
 
     # pylint: enable=C0103
@@ -820,6 +842,34 @@ def set_multiprocessing_logging_queue(queue):
         __MP_LOGGING_QUEUE = queue
 
 
+def get_multiprocessing_logging_level():
+    return __MP_LOGGING_LEVEL
+
+
+def set_multiprocessing_logging_level(log_level):
+    global __MP_LOGGING_LEVEL
+    __MP_LOGGING_LEVEL = log_level
+
+
+def set_multiprocessing_logging_level_by_opts(opts):
+    '''
+    This will set the multiprocessing logging level to the lowest
+    logging level of all the types of logging that are configured.
+    '''
+    global __MP_LOGGING_LEVEL
+
+    log_levels = [
+        LOG_LEVELS.get(opts.get('log_level', '').lower(), logging.ERROR),
+        LOG_LEVELS.get(opts.get('log_level_logfile', '').lower(), logging.ERROR)
+    ]
+    for level in six.itervalues(opts.get('log_granular_levels', {})):
+        log_levels.append(
+            LOG_LEVELS.get(level.lower(), logging.ERROR)
+        )
+
+    __MP_LOGGING_LEVEL = min(log_levels)
+
+
 def setup_multiprocessing_logging_listener(opts, queue=None):
     global __MP_LOGGING_QUEUE_PROCESS
     global __MP_LOGGING_LISTENER_CONFIGURED
@@ -883,11 +933,13 @@ def setup_multiprocessing_logging(queue=None):
         # Let's add a queue handler to the logging root handlers
         __MP_LOGGING_QUEUE_HANDLER = SaltLogQueueHandler(queue or get_multiprocessing_logging_queue())
         logging.root.addHandler(__MP_LOGGING_QUEUE_HANDLER)
-        # Set the logging root level to the lowest to get all messages
-        logging.root.setLevel(logging.GARBAGE)
+        # Set the logging root level to the lowest needed level to get all
+        # desired messages.
+        log_level = get_multiprocessing_logging_level()
+        logging.root.setLevel(log_level)
         logging.getLogger(__name__).debug(
             'Multiprocessing queue logging configured for the process running '
-            'under PID: %s', os.getpid()
+            'under PID: %s at log level %s', os.getpid(), log_level
         )
         # The above logging call will create, in some situations, a futex wait
         # lock condition, probably due to the multiprocessing Queue's internal
