@@ -47,21 +47,20 @@ The dependencies listed above can be installed via package or pip.
 
 '''
 # keep lint from choking on _get_conn and _cache_id
-#pylint: disable=E0602
-# disable complaints about perfectly falid non-assignment code
-#pylint: disable=W0106
+# pylint: disable=E0602
+#  disable complaints about perfectly valid non-assignment code
+# pylint: disable=W0106
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
-from distutils.version import LooseVersion as _LooseVersion  # pylint: disable=import-error,no-name-in-module
-import json
 
 # Import Salt libs
 from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error
 import salt.utils.compat
-import salt.utils
+import salt.utils.json
+import salt.utils.versions
 from salt.exceptions import SaltInvocationError
 
 log = logging.getLogger(__name__)
@@ -70,10 +69,10 @@ log = logging.getLogger(__name__)
 
 # pylint: disable=import-error
 try:
-    #pylint: disable=unused-import
+    # pylint: disable=unused-import
     import boto
     import boto3
-    #pylint: enable=unused-import
+    # pylint: enable=unused-import
     from botocore.exceptions import ClientError
     logging.getLogger('boto3').setLevel(logging.CRITICAL)
     HAS_BOTO = True
@@ -87,18 +86,12 @@ def __virtual__():
     Only load if boto libraries exist and if boto libraries are greater than
     a given version.
     '''
-    required_boto3_version = '1.2.1'
     # the boto_lambda execution module relies on the connect_to_region() method
     # which was added in boto 2.8.0
     # https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
-    if not HAS_BOTO:
-        return (False, 'The boto_s3_bucket module could not be loaded: '
-                'boto libraries not found')
-    elif _LooseVersion(boto3.__version__) < _LooseVersion(required_boto3_version):
-        return (False, 'The boto_cognitoidentity module could not be loaded: '
-                'boto version {0} or later must be installed.'.format(required_boto3_version))
-    else:
-        return True
+    return salt.utils.versions.check_boto_reqs(
+        boto3_ver='1.2.1'
+    )
 
 
 def __init__(opts):
@@ -167,14 +160,14 @@ def create(Bucket,
                     'GrantRead', 'GrantReadACP',
                     'GrantWrite', 'GrantWriteACP'):
             if locals()[arg] is not None:
-                kwargs[arg] = str(locals()[arg])
+                kwargs[arg] = str(locals()[arg])  # future lint: disable=blacklisted-function
         if LocationConstraint:
             kwargs['CreateBucketConfiguration'] = {'LocationConstraint': LocationConstraint}
         location = conn.create_bucket(Bucket=Bucket,
                                   **kwargs)
         conn.get_waiter("bucket_exists").wait(Bucket=Bucket)
         if location:
-            log.info('The newly created bucket name is located at {0}'.format(location['Location']))
+            log.info('The newly created bucket name is located at %s', location['Location'])
 
             return {'created': True, 'name': Bucket, 'Location': location['Location']}
         else:
@@ -228,7 +221,7 @@ def delete_objects(Bucket, Delete, MFA=None, RequestPayer=None,
     '''
 
     if isinstance(Delete, six.string_types):
-        Delete = json.loads(Delete)
+        Delete = salt.utils.json.loads(Delete)
     if not isinstance(Delete, dict):
         raise SaltInvocationError("Malformed Delete request.")
     if 'Objects' not in Delete:
@@ -484,14 +477,14 @@ def put_acl(Bucket,
         kwargs = {}
         if AccessControlPolicy is not None:
             if isinstance(AccessControlPolicy, six.string_types):
-                AccessControlPolicy = json.loads(AccessControlPolicy)
+                AccessControlPolicy = salt.utils.json.loads(AccessControlPolicy)
             kwargs['AccessControlPolicy'] = AccessControlPolicy
         for arg in ('ACL',
                     'GrantFullControl',
                     'GrantRead', 'GrantReadACP',
                     'GrantWrite', 'GrantWriteACP'):
             if locals()[arg] is not None:
-                kwargs[arg] = str(locals()[arg])
+                kwargs[arg] = str(locals()[arg])  # future lint: disable=blacklisted-function
         conn.put_bucket_acl(Bucket=Bucket, **kwargs)
         return {'updated': True, 'name': Bucket}
     except ClientError as e:
@@ -524,7 +517,7 @@ def put_cors(Bucket,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         if CORSRules is not None and isinstance(CORSRules, six.string_types):
-            CORSRules = json.loads(CORSRules)
+            CORSRules = salt.utils.json.loads(CORSRules)
         conn.put_bucket_cors(Bucket=Bucket, CORSConfiguration={'CORSRules': CORSRules})
         return {'updated': True, 'name': Bucket}
     except ClientError as e:
@@ -559,7 +552,7 @@ def put_lifecycle_configuration(Bucket,
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         if Rules is not None and isinstance(Rules, six.string_types):
-            Rules = json.loads(Rules)
+            Rules = salt.utils.json.loads(Rules)
         conn.put_bucket_lifecycle_configuration(Bucket=Bucket, LifecycleConfiguration={'Rules': Rules})
         return {'updated': True, 'name': Bucket}
     except ClientError as e:
@@ -597,7 +590,7 @@ def put_logging(Bucket,
         else:
             logstatus = {}
         if TargetGrants is not None and isinstance(TargetGrants, six.string_types):
-            TargetGrants = json.loads(TargetGrants)
+            TargetGrants = salt.utils.json.loads(TargetGrants)
         conn.put_bucket_logging(Bucket=Bucket, BucketLoggingStatus=logstatus)
         return {'updated': True, 'name': Bucket}
     except ClientError as e:
@@ -630,15 +623,15 @@ def put_notification_configuration(Bucket,
         if TopicConfigurations is None:
             TopicConfigurations = []
         elif isinstance(TopicConfigurations, six.string_types):
-            TopicConfigurations = json.loads(TopicConfigurations)
+            TopicConfigurations = salt.utils.json.loads(TopicConfigurations)
         if QueueConfigurations is None:
             QueueConfigurations = []
         elif isinstance(QueueConfigurations, six.string_types):
-            QueueConfigurations = json.loads(QueueConfigurations)
+            QueueConfigurations = salt.utils.json.loads(QueueConfigurations)
         if LambdaFunctionConfigurations is None:
             LambdaFunctionConfigurations = []
         elif isinstance(LambdaFunctionConfigurations, six.string_types):
-            LambdaFunctionConfigurations = json.loads(LambdaFunctionConfigurations)
+            LambdaFunctionConfigurations = salt.utils.json.loads(LambdaFunctionConfigurations)
         # TODO allow the user to use simple names & substitute ARNs for those names
         conn.put_bucket_notification_configuration(Bucket=Bucket, NotificationConfiguration={
                 'TopicConfigurations': TopicConfigurations,
@@ -671,7 +664,7 @@ def put_policy(Bucket, Policy,
         if Policy is None:
             Policy = '{}'
         elif not isinstance(Policy, six.string_types):
-            Policy = json.dumps(Policy)
+            Policy = salt.utils.json.dumps(Policy)
         conn.put_bucket_policy(Bucket=Bucket, Policy=Policy)
         return {'updated': True, 'name': Bucket}
     except ClientError as e:
@@ -715,7 +708,7 @@ def put_replication(Bucket, Role, Rules,
         if Rules is None:
             Rules = []
         elif isinstance(Rules, six.string_types):
-            Rules = json.loads(Rules)
+            Rules = salt.utils.json.loads(Rules)
         conn.put_bucket_replication(Bucket=Bucket, ReplicationConfiguration={
                 'Role': Role,
                 'Rules': Rules
@@ -771,9 +764,9 @@ def put_tagging(Bucket,
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         tagslist = []
         for k, v in six.iteritems(kwargs):
-            if str(k).startswith('__'):
+            if six.text_type(k).startswith('__'):
                 continue
-            tagslist.append({'Key': str(k), 'Value': str(v)})
+            tagslist.append({'Key': six.text_type(k), 'Value': six.text_type(v)})
         conn.put_bucket_tagging(Bucket=Bucket, Tagging={
                 'TagSet': tagslist,
         })
@@ -839,7 +832,7 @@ def put_website(Bucket, ErrorDocument=None, IndexDocument=None,
             val = locals()[key]
             if val is not None:
                 if isinstance(val, six.string_types):
-                    WebsiteConfiguration[key] = json.loads(val)
+                    WebsiteConfiguration[key] = salt.utils.json.loads(val)
                 else:
                     WebsiteConfiguration[key] = val
         conn.put_bucket_website(Bucket=Bucket,
