@@ -16,9 +16,11 @@ i.e. ``'vim'`` will not work, ``'app-editors/vim'`` will.
 from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
+import os
 import copy
 import logging
 import re
+import datetime
 
 # Import salt libs
 import salt.utils.args
@@ -31,6 +33,10 @@ import salt.utils.versions
 from salt.exceptions import CommandExecutionError, MinionError
 from salt.ext import six
 
+# Workaround for 'reload' builtin of py2.7
+if six.PY3:
+    from importlib import reload  # pylint: disable=no-name-in-module
+
 # Import third party libs
 HAS_PORTAGE = False
 try:
@@ -274,6 +280,7 @@ def latest_version(*names, **kwargs):
         return ret[names[0]]
     return ret
 
+
 # available_version is being deprecated
 available_version = salt.utils.functools.alias_function(latest_version, 'available_version')
 
@@ -437,7 +444,21 @@ def list_pkgs(versions_as_list=False, **kwargs):
 
 def refresh_db():
     '''
-    Updates the portage tree (emerge --sync). Uses eix-sync if available.
+    Update the portage tree using the first available method from the following
+    list:
+
+    - emaint sync
+    - eix-sync
+    - emerge-webrsync
+    - emerge --sync
+
+    To prevent the portage tree from being synced within one day of the
+    previous sync, add the following pillar data for this minion:
+
+    .. code-block:: yaml
+
+        portage:
+          sync_wait_one_day: True
 
     CLI Example:
 
@@ -445,28 +466,37 @@ def refresh_db():
 
         salt '*' pkg.refresh_db
     '''
+    has_emaint = os.path.isdir('/etc/portage/repos.conf')
+    has_eix = True if 'eix.sync' in __salt__ else False
+    has_webrsync = True if __salt__['makeconf.features_contains']('webrsync-gpg') else False
+
     # Remove rtag file to keep multiple refreshes from happening in pkg states
     salt.utils.pkg.clear_rtag(__opts__)
-    if 'eix.sync' in __salt__:
-        return __salt__['eix.sync']()
+    # Option to prevent syncing package tree if done in the last 24 hours
+    if __salt__['pillar.get']('portage:sync_wait_one_day', False):
+        main_repo_root = __salt__['cmd.run']('portageq get_repo_path / gentoo')
+        day = datetime.timedelta(days=1)
+        now = datetime.datetime.now()
+        timestamp = datetime.datetime.fromtimestamp(os.path.getmtime(main_repo_root))
+        if now - timestamp < day:
+            log.info('Did not sync package tree since last sync was done at'
+                     ' {0}, less than 1 day ago'.format(timestamp))
+            return False
 
-    if 'makeconf.features_contains'in __salt__ and __salt__['makeconf.features_contains']('webrsync-gpg'):
-        # GPG sign verify is supported only for "webrsync"
+    if has_emaint:
+        return __salt__['cmd.retcode']('emaint sync -a') == 0
+    elif has_eix:
+        return __salt__['eix.sync']()
+    elif has_webrsync:
+        # GPG sign verify is supported only for 'webrsync'
         cmd = 'emerge-webrsync -q'
-        # We prefer 'delta-webrsync' to 'webrsync'
+        # Prefer 'delta-webrsync' to 'webrsync'
         if salt.utils.path.which('emerge-delta-webrsync'):
             cmd = 'emerge-delta-webrsync -q'
-        return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
+        return __salt__['cmd.retcode'](cmd) == 0
     else:
-        if __salt__['cmd.retcode']('emerge --ask n --quiet --sync',
-                                   python_shell=False) == 0:
-            return True
-        # We fall back to "webrsync" if "rsync" fails for some reason
-        cmd = 'emerge-webrsync -q'
-        # We prefer 'delta-webrsync' to 'webrsync'
-        if salt.utils.path.which('emerge-delta-webrsync'):
-            cmd = 'emerge-delta-webrsync -q'
-        return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
+        # Default to deprecated `emerge --sync` form
+        return __salt__['cmd.retcode']('emerge --ask n --quiet --sync') == 0
 
 
 def _flags_changed(inst_flags, conf_flags):
