@@ -3,8 +3,9 @@
 Use the :ref:`Salt Event System <events>` to fire events from the
 master to the minion and vice-versa.
 '''
-from __future__ import absolute_import
+
 # Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import collections
 import logging
 import os
@@ -14,9 +15,10 @@ import traceback
 # Import salt libs
 import salt.crypt
 import salt.utils.event
+import salt.utils.zeromq
 import salt.payload
-import salt.transport
-import salt.ext.six as six
+import salt.transport.client
+from salt.ext import six
 
 __proxyenabled__ = ['*']
 log = logging.getLogger(__name__)
@@ -41,10 +43,10 @@ def fire_master(data, tag, preload=None):
     '''
     if (__opts__.get('local', None) or __opts__.get('file_client', None) == 'local') and not __opts__.get('use_master_when_local', False):
         #  We can't send an event if we're in masterless mode
-        log.warning('Local mode detected. Event with tag {0} will NOT be sent.'.format(tag))
+        log.warning('Local mode detected. Event with tag %s will NOT be sent.', tag)
         return False
     if __opts__['transport'] == 'raet':
-        channel = salt.transport.Channel.factory(__opts__)
+        channel = salt.transport.client.ReqChannel.factory(__opts__)
         load = {'id': __opts__['id'],
                 'tag': tag,
                 'data': data,
@@ -53,6 +55,8 @@ def fire_master(data, tag, preload=None):
             channel.send(load)
         except Exception:
             pass
+        finally:
+            channel.close()
         return True
 
     if preload or __opts__.get('__cli') == 'salt-call':
@@ -60,7 +64,7 @@ def fire_master(data, tag, preload=None):
         # slower because it has to independently authenticate)
         if 'master_uri' not in __opts__:
             __opts__['master_uri'] = 'tcp://{ip}:{port}'.format(
-                    ip=salt.utils.ip_bracket(__opts__['interface']),
+                    ip=salt.utils.zeromq.ip_bracket(__opts__['interface']),
                     port=__opts__.get('ret_port', '4506')  # TODO, no fallback
                     )
         masters = list()
@@ -74,18 +78,23 @@ def fire_master(data, tag, preload=None):
         load = {'id': __opts__['id'],
                 'tag': tag,
                 'data': data,
-                'tok': auth.gen_token('salt'),
+                'tok': auth.gen_token(b'salt'),
                 'cmd': '_minion_event'}
 
         if isinstance(preload, dict):
             load.update(preload)
 
         for master in masters:
-            channel = salt.transport.Channel.factory(__opts__, master_uri=master)
+            channel = salt.transport.client.ReqChannel.factory(__opts__, master_uri=master)
             try:
                 channel.send(load)
+                # channel.send was successful.
+                # Ensure ret is True.
+                ret = True
             except Exception:
                 ret = False
+            finally:
+                channel.close()
         return ret
     else:
         # Usually, we can send the event via the minion, which is faster
@@ -224,7 +233,7 @@ def send(tag,
             data_dict['pillar'] = __pillar__
 
     if with_env_opts:
-        data_dict['saltenv'] = __opts__.get('environment', 'base')
+        data_dict['saltenv'] = __opts__.get('saltenv', 'base')
         data_dict['pillarenv'] = __opts__.get('pillarenv')
 
     if kwargs:
