@@ -39,9 +39,9 @@ def __virtual__():
             'only available on Gentoo/Open-RC systems.')
 
 
-def _ret_code(cmd):
+def _ret_code(cmd, ignore_retcode=False):
     log.debug('executing [{0}]'.format(cmd))
-    sts = __salt__['cmd.retcode'](cmd, python_shell=False)
+    sts = __salt__['cmd.retcode'](cmd, python_shell=False, ignore_retcode=ignore_retcode)
     return sts
 
 
@@ -270,7 +270,7 @@ def status(name, sig=None):
     results = {}
     for service in services:
         cmd = _service_cmd(service, 'status')
-        results[service] = not _ret_code(cmd)
+        results[service] = not _ret_code(cmd, ignore_retcode=True)
     if contains_globbing:
         return results
     return results[name]
