@@ -8,10 +8,11 @@ from __future__ import absolute_import, print_function, unicode_literals
 import inspect
 import logging
 import sys
+import copy
 
 # Import salt libs
 import salt.minion
-import salt.utils.versions
+import salt.loader
 from salt.defaults import DEFAULT_TARGET_DELIM
 from salt.ext import six
 
@@ -37,16 +38,16 @@ def compound(tgt, minion_id=None):
 
         salt '*' match.compound 'L@cheese,foo and *'
     '''
-    opts = {'grains': __grains__, 'pillar': __pillar__}
     if minion_id is not None:
+        opts = copy.copy(__opts__)
         if not isinstance(minion_id, six.string_types):
             minion_id = six.text_type(minion_id)
+        opts['id'] = minion_id
     else:
-        minion_id = __grains__['id']
-    opts['id'] = minion_id
-    matcher = salt.minion.Matcher(opts, __salt__)
+        opts = __opts__
+    matchers = salt.loader.matchers(opts)
     try:
-        return matcher.compound_match(tgt)
+        return matchers['compound_match.match'](tgt)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -72,9 +73,9 @@ def ipcidr(tgt):
          - nodeclass: internal
 
     '''
-    matcher = salt.minion.Matcher({'grains': __grains__}, __salt__)
+    matchers = salt.loader.matchers(__opts__)
     try:
-        return matcher.ipcidr_match(tgt)
+        return matchers['ipcidr_match.match'](tgt)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -103,9 +104,9 @@ def pillar_pcre(tgt, delimiter=DEFAULT_TARGET_DELIM):
         .. versionadded:: 0.16.4
         .. deprecated:: 2015.8.0
     '''
-    matcher = salt.minion.Matcher({'pillar': __pillar__}, __salt__)
+    matchers = salt.loader.matchers(__opts__)
     try:
-        return matcher.pillar_pcre_match(tgt, delimiter=delimiter)
+        return matchers['pillar_pcre_match.match'](tgt, delimiter=delimiter)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -134,9 +135,9 @@ def pillar(tgt, delimiter=DEFAULT_TARGET_DELIM):
         .. versionadded:: 0.16.4
         .. deprecated:: 2015.8.0
     '''
-    matcher = salt.minion.Matcher({'pillar': __pillar__}, __salt__)
+    matchers = salt.loader.matchers(__opts__)
     try:
-        return matcher.pillar_match(tgt, delimiter=delimiter)
+        return matchers['pillar_match.match'](tgt, delimiter=delimiter)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -152,9 +153,9 @@ def data(tgt):
 
         salt '*' match.data 'spam:eggs'
     '''
-    matcher = salt.minion.Matcher(__opts__, __salt__)
+    matchers = salt.loader.matchers(__opts__)
     try:
-        return matcher.data_match(tgt)
+        return matchers['data_match.match'](tgt)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -183,9 +184,9 @@ def grain_pcre(tgt, delimiter=DEFAULT_TARGET_DELIM):
         .. versionadded:: 0.16.4
         .. deprecated:: 2015.8.0
     '''
-    matcher = salt.minion.Matcher({'grains': __grains__}, __salt__)
+    matchers = salt.loader.matchers(__opts__)
     try:
-        return matcher.grain_pcre_match(tgt, delimiter=delimiter)
+        return matchers['grain_pcre_match.match'](tgt, delimiter=delimiter)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -214,9 +215,9 @@ def grain(tgt, delimiter=DEFAULT_TARGET_DELIM):
         .. versionadded:: 0.16.4
         .. deprecated:: 2015.8.0
     '''
-    matcher = salt.minion.Matcher({'grains': __grains__}, __salt__)
+    matchers = salt.loader.matchers(__opts__)
     try:
-        return matcher.grain_match(tgt, delimiter=delimiter)
+        return matchers['grain_match.match'](tgt, delimiter=delimiter)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -238,13 +239,15 @@ def list_(tgt, minion_id=None):
         salt '*' match.list 'server1,server2'
     '''
     if minion_id is not None:
+        opts = copy.copy(__opts__)
         if not isinstance(minion_id, six.string_types):
             minion_id = six.text_type(minion_id)
+        opts['id'] = minion_id
     else:
-        minion_id = __grains__['id']
-    matcher = salt.minion.Matcher({'id': minion_id}, __salt__)
+        opts = __opts__
+    matchers = salt.loader.matchers(opts)
     try:
-        return matcher.list_match(tgt)
+        return matchers['list_match.match'](tgt)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -266,13 +269,15 @@ def pcre(tgt, minion_id=None):
         salt '*' match.pcre '.*'
     '''
     if minion_id is not None:
+        opts = copy.copy(__opts__)
         if not isinstance(minion_id, six.string_types):
             minion_id = six.text_type(minion_id)
+        opts['id'] = minion_id
     else:
-        minion_id = __grains__['id']
-    matcher = salt.minion.Matcher({'id': minion_id}, __salt__)
+        opts = __opts__
+    matchers = salt.loader.matchers(opts)
     try:
-        return matcher.pcre_match(tgt)
+        return matchers['pcre_match.match'](tgt)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -294,13 +299,16 @@ def glob(tgt, minion_id=None):
         salt '*' match.glob '*'
     '''
     if minion_id is not None:
+        opts = copy.copy(__opts__)
         if not isinstance(minion_id, six.string_types):
             minion_id = six.text_type(minion_id)
+        opts['id'] = minion_id
     else:
-        minion_id = __grains__['id']
-    matcher = salt.minion.Matcher({'id': minion_id}, __salt__)
+        opts = __opts__
+    matchers = salt.loader.matchers(opts)
+
     try:
-        return matcher.glob_match(tgt)
+        return matchers['glob_match.match'](tgt)
     except Exception as exc:
         log.exception(exc)
         return False
@@ -309,7 +317,6 @@ def glob(tgt, minion_id=None):
 def filter_by(lookup,
               tgt_type='compound',
               minion_id=None,
-              expr_form=None,
               default='default'):
     '''
     Return the first match in a dictionary of target patterns
@@ -335,17 +342,6 @@ def filter_by(lookup,
         # Make the filtered data available to Pillar:
         roles: {{ roles | yaml() }}
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     expr_funcs = dict(inspect.getmembers(sys.modules[__name__],
         predicate=inspect.isfunction))
 
@@ -375,7 +371,7 @@ def search_by(lookup, tgt_type='compound', minion_id=None):
 
     Pillar Example:
 
-    .. code-block:: yaml
+    .. code-block:: jinja
 
         {% set roles = salt.match.search_by({
             'web': ['G@os_family:Debian not nodeX'],
