@@ -7,16 +7,18 @@ Execution module for Cisco NX OS Switches Proxy minions
 For documentation on setting up the nxos proxy minion look in the documentation
 for :mod:`salt.proxy.nxos <salt.proxy.nxos>`.
 '''
-from __future__ import absolute_import
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 
-import salt.utils
+# Import Salt libs
+import salt.utils.platform
 
 __proxyenabled__ = ['nxos']
 __virtualname__ = 'nxos'
 
 
 def __virtual__():
-    if salt.utils.is_proxy():
+    if salt.utils.platform.is_proxy():
         return __virtualname__
     return (False, 'The nxos execution module failed to load: '
             'only available on proxy minions.')
@@ -57,7 +59,7 @@ def cmd(command, *args, **kwargs):
     proxy_cmd = '.'.join([proxy_prefix, command])
     if proxy_cmd not in __proxy__:
         return False
-    for k in kwargs.keys():
+    for k in kwargs:
         if k.startswith('__pub_'):
             kwargs.pop(k)
     return __proxy__[proxy_cmd](*args, **kwargs)
