@@ -4,15 +4,16 @@ Support for OSQuery - https://osquery.io.
 
 .. versionadded:: 2015.8.0
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
-import json
+import logging
 
 # Import Salt libs
-import salt.utils
+import salt.utils.json
+import salt.utils.path
+import salt.utils.platform
 
-import logging
 log = logging.getLogger(__name__)
 
 
@@ -26,7 +27,7 @@ __virtualname__ = 'osquery'
 
 
 def __virtual__():
-    if salt.utils.which('osqueryi'):
+    if salt.utils.path.which('osqueryi'):
         return __virtualname__
     return (False, 'The osquery execution module cannot be loaded: '
                    'osqueryi binary is not in the path.')
@@ -36,11 +37,11 @@ def _table_attrs(table):
     '''
     Helper function to find valid table attributes
     '''
-    cmd = ['osqueryi'] + ['--json'] + ['pragma table_info{0}'.format(table)]
+    cmd = ['osqueryi'] + ['--json'] + ['pragma table_info({0})'.format(table)]
     res = __salt__['cmd.run_all'](cmd)
     if res['retcode'] == 0:
         attrs = []
-        text = json.loads(res['stdout'])
+        text = salt.utils.json.loads(res['stdout'])
         for item in text:
             attrs.append(item['name'])
         return attrs
@@ -57,11 +58,12 @@ def _osquery(sql, format='json'):
 
     cmd = ['osqueryi'] + ['--json'] + [sql]
     res = __salt__['cmd.run_all'](cmd)
-    if res['retcode'] == 0:
-        ret['data'] = json.loads(res['stdout'])
-    else:
+    if res['stderr']:
         ret['result'] = False
         ret['error'] = res['stderr']
+    else:
+        ret['data'] = salt.utils.json.loads(res['stdout'])
+    log.debug('== %s ==', ret)
     return ret
 
 
@@ -119,9 +121,13 @@ def version():
 
         salt '*' osquery.version
     '''
+    _false_return = {'result': False,
+                     'comment': 'OSQuery version unavailable.'}
     res = _osquery_cmd(table='osquery_info', attrs=['version'])
-    if res and isinstance(res, list):
-        return res[0].get('version', '') or False
+    if 'result' in res and res['result']:
+        if 'data' in res and isinstance(res['data'], list):
+            return res['data'][0].get('version', '') or _false_return
+    return _false_return
 
 
 def rpm_packages(attrs=None, where=None):
@@ -136,7 +142,8 @@ def rpm_packages(attrs=None, where=None):
     '''
     if __grains__['os_family'] == 'RedHat':
         return _osquery_cmd(table='rpm_packages', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on Red Hat based systems.'}
+    return {'result': False,
+            'comment': 'Only available on Red Hat based systems.'}
 
 
 def kernel_integrity(attrs=None, where=None):
@@ -657,7 +664,7 @@ def alf(attrs=None, where=None):
 
         salt '*' osquery.alf
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='alf', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -672,7 +679,7 @@ def alf_exceptions(attrs=None, where=None):
 
         salt '*' osquery.alf_exceptions
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='alf_exceptions', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -687,7 +694,7 @@ def alf_explicit_auths(attrs=None, where=None):
 
         salt '*' osquery.alf_explicit_auths
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='alf_explicit_auths', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -702,7 +709,7 @@ def alf_services(attrs=None, where=None):
 
         salt '*' osquery.alf_services
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='alf_services', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -717,7 +724,7 @@ def apps(attrs=None, where=None):
 
         salt '*' osquery.apps
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='apps', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -732,7 +739,7 @@ def certificates(attrs=None, where=None):
 
         salt '*' osquery.certificates
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='certificates', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -747,7 +754,7 @@ def chrome_extensions(attrs=None, where=None):
 
         salt '*' osquery.chrome_extensions
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='chrome_extensions', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -762,7 +769,7 @@ def firefox_addons(attrs=None, where=None):
 
         salt '*' osquery.firefox_addons
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='firefox_addons', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -777,7 +784,7 @@ def homebrew_packages(attrs=None, where=None):
 
         salt '*' osquery.homebrew_packages
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='homebrew_packages', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -792,7 +799,7 @@ def iokit_devicetree(attrs=None, where=None):
 
         salt '*' osquery.iokit_devicetree
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='iokit_devicetree', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -807,7 +814,7 @@ def iokit_registry(attrs=None, where=None):
 
         salt '*' osquery.iokit_registry
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='iokit_registry', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -822,7 +829,7 @@ def kernel_extensions(attrs=None, where=None):
 
         salt '*' osquery.kernel_extensions
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='kernel_extensions', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -837,7 +844,7 @@ def keychain_items(attrs=None, where=None):
 
         salt '*' osquery.keychain_items
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='keychain_items', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -852,7 +859,7 @@ def launchd(attrs=None, where=None):
 
         salt '*' osquery.launchd
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='launchd', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -867,7 +874,7 @@ def nfs_shares(attrs=None, where=None):
 
         salt '*' osquery.nfs_shares
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='nfs_shares', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -882,7 +889,7 @@ def nvram(attrs=None, where=None):
 
         salt '*' osquery.nvram
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='nvram', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -897,7 +904,7 @@ def preferences(attrs=None, where=None):
 
         salt '*' osquery.preferences
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='preferences', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -912,7 +919,7 @@ def quarantine(attrs=None, where=None):
 
         salt '*' osquery.quarantine
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='quarantine', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -927,7 +934,7 @@ def safari_extensions(attrs=None, where=None):
 
         salt '*' osquery.safari_extensions
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='safari_extensions', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -942,7 +949,7 @@ def startup_items(attrs=None, where=None):
 
         salt '*' osquery.startup_items
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='startup_items', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -957,7 +964,7 @@ def xattr_where_from(attrs=None, where=None):
 
         salt '*' osquery.xattr_where_from
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='xattr_where_from', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -972,7 +979,7 @@ def xprotect_entries(attrs=None, where=None):
 
         salt '*' osquery.xprotect_entries
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='xprotect_entries', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
@@ -987,7 +994,7 @@ def xprotect_reports(attrs=None, where=None):
 
         salt '*' osquery.xprotect_reports
     '''
-    if salt.utils.is_darwin():
+    if salt.utils.platform.is_darwin():
         return _osquery_cmd(table='xprotect_reports', attrs=attrs, where=where)
     return {'result': False, 'comment': 'Only available on macOS systems.'}
 
