@@ -32,7 +32,7 @@ Module to provide Postgres compatibility to salt.
 # pylint: disable=E8203
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 import datetime
 import logging
 import hashlib
@@ -47,14 +47,16 @@ except ImportError:
     HAS_CSV = False
 
 # Import salt libs
-import salt.utils
 import salt.utils.files
 import salt.utils.itertools
+import salt.utils.odict
+import salt.utils.path
+import salt.utils.stringutils
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import zip  # pylint: disable=import-error,redefined-builtin
 from salt.ext.six.moves import StringIO
 
@@ -120,7 +122,7 @@ def __virtual__():
     if not HAS_CSV:
         return False
     for util in utils:
-        if not salt.utils.which(util):
+        if not salt.utils.path.which(util):
             if not _find_pg_binary(util):
                 return (False, '{0} was not found'.format(util))
     return True
@@ -133,10 +135,10 @@ def _find_pg_binary(util):
     Helper function to locate various psql related binaries
     '''
     pg_bin_dir = __salt__['config.option']('postgres.bins_dir')
-    util_bin = salt.utils.which(util)
+    util_bin = salt.utils.path.which(util)
     if not util_bin:
         if pg_bin_dir:
-            return salt.utils.which(os.path.join(pg_bin_dir, util))
+            return salt.utils.path.which(os.path.join(pg_bin_dir, util))
     else:
         return util_bin
 
@@ -171,13 +173,13 @@ def _run_psql(cmd, runas=None, password=None, host=None, port=None, user=None):
         password = __salt__['config.option']('postgres.pass')
     if password is not None:
         pgpassfile = salt.utils.files.mkstemp(text=True)
-        with salt.utils.fopen(pgpassfile, 'w') as fp_:
-            fp_.write('{0}:{1}:*:{2}:{3}'.format(
+        with salt.utils.files.fopen(pgpassfile, 'w') as fp_:
+            fp_.write(salt.utils.stringutils.to_str('{0}:{1}:*:{2}:{3}'.format(
                 'localhost' if not host or host.startswith('/') else host,
                 port if port else '*',
                 user if user else '*',
                 password,
-            ))
+            )))
             __salt__['file.chown'](pgpassfile, runas, '')
             kwargs['env'] = {'PGPASSFILE': pgpassfile}
 
@@ -197,7 +199,9 @@ def _run_initdb(name,
         password=None,
         encoding='UTF8',
         locale=None,
-        runas=None):
+        runas=None,
+        waldir=None,
+        checksums=False):
     '''
     Helper function to call initdb
     '''
@@ -225,10 +229,19 @@ def _run_initdb(name,
     if locale is not None:
         cmd.append('--locale={0}'.format(locale))
 
+    # intentionally use short option, as the long option name has been
+    # renamed from "xlogdir" to "waldir" in PostgreSQL 10
+    if waldir is not None:
+        cmd.append('-X')
+        cmd.append(waldir)
+
+    if checksums:
+        cmd.append('--data-checksums')
+
     if password is not None:
         pgpassfile = salt.utils.files.mkstemp(text=True)
-        with salt.utils.fopen(pgpassfile, 'w') as fp_:
-            fp_.write('{0}'.format(password))
+        with salt.utils.files.fopen(pgpassfile, 'w') as fp_:
+            fp_.write(salt.utils.stringutils.to_str('{0}'.format(password)))
             __salt__['file.chown'](pgpassfile, runas, '')
         cmd.extend([
             '--pwfile={0}'.format(pgpassfile),
@@ -300,8 +313,7 @@ def _parsed_version(user=None, host=None, port=None, maintenance_db=None,
         return None
 
 
-def _connection_defaults(user=None, host=None, port=None, maintenance_db=None,
-                         password=None):
+def _connection_defaults(user=None, host=None, port=None, maintenance_db=None):
     '''
     Returns a tuple of (user, host, port, db) with config, pillar, or default
     values assigned to missing values.
@@ -314,37 +326,35 @@ def _connection_defaults(user=None, host=None, port=None, maintenance_db=None,
         port = __salt__['config.option']('postgres.port')
     if not maintenance_db:
         maintenance_db = __salt__['config.option']('postgres.maintenance_db')
-    if password is None:
-        password = __salt__['config.option']('postgres.pass')
 
-    return (user, host, port, maintenance_db, password)
+    return (user, host, port, maintenance_db)
 
 
 def _psql_cmd(*args, **kwargs):
     '''
     Return string with fully composed psql command.
 
-    Accept optional keyword arguments: user, host and port as well as any
-    number or positional arguments to be added to the end of command.
+    Accepts optional keyword arguments: user, host, port and maintenance_db,
+    as well as any number of positional arguments to be added to the end of
+    the command.
     '''
-    (user, host, port, maintenance_db, password) = _connection_defaults(
+    (user, host, port, maintenance_db) = _connection_defaults(
         kwargs.get('user'),
         kwargs.get('host'),
         kwargs.get('port'),
-        kwargs.get('maintenance_db'),
-        kwargs.get('password'))
+        kwargs.get('maintenance_db'))
     _PSQL_BIN = _find_pg_binary('psql')
     cmd = [_PSQL_BIN,
            '--no-align',
            '--no-readline',
            '--no-psqlrc',
-           '--no-password']  # It is never acceptable to issue a password prompt.
+           '--no-password']  # Never prompt, handled in _run_psql.
     if user:
         cmd += ['--username', user]
     if host:
         cmd += ['--host', host]
     if port:
-        cmd += ['--port', str(port)]
+        cmd += ['--port', six.text_type(port)]
     if not maintenance_db:
         maintenance_db = 'postgres'
     cmd.extend(['--dbname', maintenance_db])
@@ -361,7 +371,7 @@ def _psql_prepare_and_run(cmd,
                           user=None):
     rcmd = _psql_cmd(
         host=host, user=user, port=port,
-        maintenance_db=maintenance_db, password=password,
+        maintenance_db=maintenance_db,
         *cmd)
     cmdret = _run_psql(
         rcmd, runas=runas, password=password, host=host, port=port, user=user)
@@ -431,7 +441,9 @@ def psql_query(query, user=None, host=None, port=None, maintenance_db=None,
 
     csv_file = StringIO(cmdret['stdout'])
     header = {}
-    for row in csv.reader(csv_file, delimiter=',', quotechar='"'):
+    for row in csv.reader(csv_file,
+                          delimiter=salt.utils.stringutils.to_str(','),
+                          quotechar=salt.utils.stringutils.to_str('"')):
         if not row:
             continue
         if not header:
@@ -848,6 +860,10 @@ def user_list(user=None, host=None, port=None, maintenance_db=None,
         'pg_roles.rolcanlogin as "can login", '
         '{1} as "replication", '
         'pg_roles.rolconnlimit as "connections", '
+        '(SELECT array_agg(pg_roles2.rolname)'
+        '    FROM pg_catalog.pg_auth_members'
+        '    JOIN pg_catalog.pg_roles pg_roles2 ON (pg_auth_members.roleid = pg_roles2.oid)'
+        '    WHERE pg_auth_members.member = pg_roles.oid) as "groups",'
         'pg_roles.rolvaliduntil::timestamp(0) as "expiry time", '
         'pg_roles.rolconfig  as "defaults variables" '
         , _x(', COALESCE(pg_shadow.passwd, pg_authid.rolpassword) as "password" '),
@@ -884,43 +900,16 @@ def user_list(user=None, host=None, port=None, maintenance_db=None,
         for date_key in ('expiry time',):
             try:
                 retrow[date_key] = datetime.datetime.strptime(
-                    row['date_key'], '%Y-%m-%d %H:%M:%S')
-            except (ValueError, KeyError):
+                    row[date_key], '%Y-%m-%d %H:%M:%S')
+            except ValueError:
                 retrow[date_key] = None
         retrow['defaults variables'] = row['defaults variables']
         if return_password:
             retrow['password'] = row['password']
+        # use csv reader to handle quoted roles correctly
+        retrow['groups'] = list(csv.reader([row['groups'].strip('{}')]))[0]
         ret[row['name']] = retrow
 
-    # for each role, determine the inherited roles
-    for role in six.iterkeys(ret):
-        rdata = ret[role]
-        groups = rdata.setdefault('groups', [])
-        query = (
-            'select rolname'
-            ' from pg_user'
-            ' join pg_auth_members'
-            '      on (pg_user.usesysid=pg_auth_members.member)'
-            ' join pg_roles '
-            '      on (pg_roles.oid=pg_auth_members.roleid)'
-            ' where pg_user.usename=\'{0}\''
-        ).format(role)
-        try:
-            rows = psql_query(query,
-                              runas=runas,
-                              host=host,
-                              user=user,
-                              port=port,
-                              maintenance_db=maintenance_db,
-                              password=password)
-            for row in rows:
-                if row['rolname'] not in groups:
-                    groups.append(row['rolname'])
-        except Exception:
-            # do not fail here, it is just a bonus
-            # to try to determine groups, but the query
-            # is not portable amongst all pg versions
-            continue
     return ret
 
 
@@ -1002,10 +991,10 @@ def _maybe_encrypt_password(role,
     pgsql passwords are md5 hashes of the string: 'md5{password}{rolename}'
     '''
     if password is not None:
-        password = str(password)
+        password = six.text_type(password)
     if encrypted and password and not password.startswith('md5'):
         password = "md5{0}".format(
-            hashlib.md5(salt.utils.to_bytes('{0}{1}'.format(password, role))).hexdigest())
+            hashlib.md5(salt.utils.stringutils.to_bytes('{0}{1}'.format(password, role))).hexdigest())
     return password
 
 
@@ -1017,15 +1006,13 @@ def _role_cmd_args(name,
                    connlimit=None,
                    inherit=None,
                    createdb=None,
-                   createuser=None,
                    createroles=None,
                    superuser=None,
                    groups=None,
                    replication=None,
                    rolepassword=None,
+                   valid_until=None,
                    db_role=None):
-    if createuser is not None and superuser is None:
-        superuser = createuser
     if inherit is None:
         if typ_ in ['user', 'group']:
             inherit = True
@@ -1039,6 +1026,7 @@ def _role_cmd_args(name,
         encrypted = _DEFAULT_PASSWORDS_ENCRYPTION
     skip_passwd = False
     escaped_password = ''
+    escaped_valid_until = ''
     if not (
         rolepassword is not None
         # first is passwd set
@@ -1056,6 +1044,10 @@ def _role_cmd_args(name,
             _maybe_encrypt_password(name,
                                     rolepassword.replace('\'', '\'\''),
                                     encrypted=encrypted))
+    if isinstance(valid_until, six.string_types) and bool(valid_until):
+        escaped_valid_until = '\'{0}\''.format(
+            valid_until.replace('\'', '\'\''),
+        )
     skip_superuser = False
     if bool(db_role) and bool(superuser) == bool(db_role['superuser']):
         skip_superuser = True
@@ -1069,7 +1061,7 @@ def _role_cmd_args(name,
         {'flag': 'LOGIN', 'test': login},
         {'flag': 'CONNECTION LIMIT',
          'test': bool(connlimit),
-         'addtxt': str(connlimit),
+         'addtxt': six.text_type(connlimit),
          'skip': connlimit is None},
         {'flag': 'ENCRYPTED',
          'test': (encrypted is not None and bool(rolepassword)),
@@ -1079,6 +1071,10 @@ def _role_cmd_args(name,
         {'flag': 'PASSWORD', 'test': bool(rolepassword),
          'skip': skip_passwd,
          'addtxt': escaped_password},
+        {'flag': 'VALID UNTIL',
+         'test': bool(valid_until),
+         'skip': valid_until is None,
+         'addtxt': escaped_valid_until},
     )
     for data in flags:
         sub_cmd = _add_role_flag(sub_cmd, **data)
@@ -1100,7 +1096,6 @@ def _role_create(name,
                  password=None,
                  createdb=None,
                  createroles=None,
-                 createuser=None,
                  encrypted=None,
                  superuser=None,
                  login=None,
@@ -1108,6 +1103,7 @@ def _role_create(name,
                  inherit=None,
                  replication=None,
                  rolepassword=None,
+                 valid_until=None,
                  typ_='role',
                  groups=None,
                  runas=None):
@@ -1119,7 +1115,7 @@ def _role_create(name,
     # check if role exists
     if user_exists(name, user, host, port, maintenance_db,
                    password=password, runas=runas):
-        log.info('{0} \'{1}\' already exists'.format(typ_.capitalize(), name))
+        log.info('%s \'%s\' already exists', typ_.capitalize(), name)
         return False
 
     sub_cmd = 'CREATE ROLE "{0}" WITH'.format(name)
@@ -1132,11 +1128,11 @@ def _role_create(name,
         inherit=inherit,
         createdb=createdb,
         createroles=createroles,
-        createuser=createuser,
         superuser=superuser,
         groups=groups,
         replication=replication,
-        rolepassword=rolepassword
+        rolepassword=rolepassword,
+        valid_until=valid_until
     ))
     ret = _psql_prepare_and_run(['-c', sub_cmd],
                                 runas=runas, host=host, user=user, port=port,
@@ -1153,7 +1149,6 @@ def user_create(username,
                 maintenance_db=None,
                 password=None,
                 createdb=None,
-                createuser=None,
                 createroles=None,
                 inherit=None,
                 login=None,
@@ -1162,6 +1157,7 @@ def user_create(username,
                 superuser=None,
                 replication=None,
                 rolepassword=None,
+                valid_until=None,
                 groups=None,
                 runas=None):
     '''
@@ -1173,7 +1169,7 @@ def user_create(username,
 
         salt '*' postgres.user_create 'username' user='user' \\
                 host='hostname' port='port' password='password' \\
-                rolepassword='rolepassword'
+                rolepassword='rolepassword' valid_until='valid_until'
     '''
     return _role_create(username,
                         typ_='user',
@@ -1183,7 +1179,6 @@ def user_create(username,
                         maintenance_db=maintenance_db,
                         password=password,
                         createdb=createdb,
-                        createuser=createuser,
                         createroles=createroles,
                         inherit=inherit,
                         login=login,
@@ -1192,6 +1187,7 @@ def user_create(username,
                         superuser=superuser,
                         replication=replication,
                         rolepassword=rolepassword,
+                        valid_until=valid_until,
                         groups=groups,
                         runas=runas)
 
@@ -1203,7 +1199,6 @@ def _role_update(name,
                  maintenance_db=None,
                  password=None,
                  createdb=None,
-                 createuser=None,
                  typ_='role',
                  createroles=None,
                  inherit=None,
@@ -1213,6 +1208,7 @@ def _role_update(name,
                  superuser=None,
                  replication=None,
                  rolepassword=None,
+                 valid_until=None,
                  groups=None,
                  runas=None):
     '''
@@ -1230,7 +1226,7 @@ def _role_update(name,
     # check if user exists
     if not bool(role):
         log.info(
-            '{0} \'{1}\' could not be found'.format(typ_.capitalize(), name)
+            '%s \'%s\' could not be found', typ_.capitalize(), name
         )
         return False
 
@@ -1242,12 +1238,12 @@ def _role_update(name,
         connlimit=connlimit,
         inherit=inherit,
         createdb=createdb,
-        createuser=createuser,
         createroles=createroles,
         superuser=superuser,
         groups=groups,
         replication=replication,
         rolepassword=rolepassword,
+        valid_until=valid_until,
         db_role=role
     ))
     ret = _psql_prepare_and_run(['-c', sub_cmd],
@@ -1265,7 +1261,6 @@ def user_update(username,
                 maintenance_db=None,
                 password=None,
                 createdb=None,
-                createuser=None,
                 createroles=None,
                 encrypted=None,
                 superuser=None,
@@ -1274,6 +1269,7 @@ def user_update(username,
                 connlimit=None,
                 replication=None,
                 rolepassword=None,
+                valid_until=None,
                 groups=None,
                 runas=None):
     '''
@@ -1285,7 +1281,7 @@ def user_update(username,
 
         salt '*' postgres.user_update 'username' user='user' \\
                 host='hostname' port='port' password='password' \\
-                rolepassword='rolepassword'
+                rolepassword='rolepassword' valid_until='valid_until'
     '''
     return _role_update(username,
                         user=user,
@@ -1298,12 +1294,12 @@ def user_update(username,
                         login=login,
                         connlimit=connlimit,
                         createdb=createdb,
-                        createuser=createuser,
                         createroles=createroles,
                         encrypted=encrypted,
                         superuser=superuser,
                         replication=replication,
                         rolepassword=rolepassword,
+                        valid_until=valid_until,
                         groups=groups,
                         runas=runas)
 
@@ -1317,7 +1313,7 @@ def _role_remove(name, user=None, host=None, port=None, maintenance_db=None,
     # check if user exists
     if not user_exists(name, user, host, port, maintenance_db,
                        password=password, runas=runas):
-        log.info('User \'{0}\' does not exist'.format(name))
+        log.info('User \'%s\' does not exist', name)
         return False
 
     # user exists, proceed
@@ -1331,7 +1327,7 @@ def _role_remove(name, user=None, host=None, port=None, maintenance_db=None,
                        password=password, runas=runas):
         return True
     else:
-        log.info('Failed to delete user \'{0}\'.'.format(name))
+        log.info('Failed to delete user \'%s\'.', name)
         return False
 
 
@@ -1479,11 +1475,12 @@ def is_available_extension(name,
 
 
 def _pg_is_older_ext_ver(a, b):
-    '''Return true if version a is lesser than b
-    TODO: be more intelligent to test versions
-
     '''
-    return a < b
+    Compare versions of extensions using salt.utils.versions.LooseVersion.
+
+    Returns ``True`` if version a is lesser than b.
+    '''
+    return _LooseVersion(a) < _LooseVersion(b)
 
 
 def is_installed_extension(name,
@@ -1616,7 +1613,7 @@ def drop_extension(name,
                                      password=password,
                                      runas=runas)
     if not ret:
-        log.info('Failed to drop ext: {0}'.format(name))
+        log.info('Failed to drop ext: %s', name)
     return ret
 
 
@@ -1706,7 +1703,7 @@ def create_extension(name,
         if (i in mtdata) and (i != _EXTENSION_INSTALLED):
             ret = False
     if not ret:
-        log.info('Failed to create ext: {0}'.format(name))
+        log.info('Failed to create ext: %s', name)
     return ret
 
 
@@ -1744,7 +1741,6 @@ def group_create(groupname,
                  maintenance_db=None,
                  password=None,
                  createdb=None,
-                 createuser=None,
                  createroles=None,
                  encrypted=None,
                  login=None,
@@ -1775,7 +1771,6 @@ def group_create(groupname,
                         password=password,
                         createdb=createdb,
                         createroles=createroles,
-                        createuser=createuser,
                         encrypted=encrypted,
                         login=login,
                         inherit=inherit,
@@ -1794,7 +1789,6 @@ def group_update(groupname,
                  password=None,
                  createdb=None,
                  createroles=None,
-                 createuser=None,
                  encrypted=None,
                  inherit=None,
                  login=None,
@@ -1823,7 +1817,6 @@ def group_update(groupname,
                         createdb=createdb,
                         typ_='group',
                         createroles=createroles,
-                        createuser=createuser,
                         encrypted=encrypted,
                         login=login,
                         inherit=inherit,
@@ -1954,10 +1947,10 @@ def schema_create(dbname, name, owner=None,
     '''
 
     # check if schema exists
-    if schema_exists(dbname, name,
+    if schema_exists(dbname, name, user=user,
                      db_user=db_user, db_password=db_password,
                      db_host=db_host, db_port=db_port):
-        log.info('\'{0}\' already exists in \'{1}\''.format(name, dbname))
+        log.info('\'%s\' already exists in \'%s\'', name, dbname)
         return False
 
     sub_cmd = 'CREATE SCHEMA "{0}"'.format(name)
@@ -2009,10 +2002,10 @@ def schema_remove(dbname, name,
     '''
 
     # check if schema exists
-    if not schema_exists(dbname, name,
+    if not schema_exists(dbname, name, user=None,
                          db_user=db_user, db_password=db_password,
                          db_host=db_host, db_port=db_port):
-        log.info('Schema \'{0}\' does not exist in \'{1}\''.format(name, dbname))
+        log.info('Schema \'%s\' does not exist in \'%s\'', name, dbname)
         return False
 
     # schema exists, proceed
@@ -2023,16 +2016,16 @@ def schema_remove(dbname, name,
         maintenance_db=dbname,
         host=db_host, user=db_user, port=db_port, password=db_password)
 
-    if not schema_exists(dbname, name,
+    if not schema_exists(dbname, name, user,
                          db_user=db_user, db_password=db_password,
                          db_host=db_host, db_port=db_port):
         return True
     else:
-        log.info('Failed to delete schema \'{0}\'.'.format(name))
+        log.info('Failed to delete schema \'%s\'.', name)
         return False
 
 
-def schema_exists(dbname, name,
+def schema_exists(dbname, name, user=None,
                   db_user=None, db_password=None,
                   db_host=None, db_port=None):
     '''
@@ -2050,6 +2043,9 @@ def schema_exists(dbname, name,
     name
        Schema name we look for
 
+    user
+        The system user the operation should be performed on behalf of
+
     db_user
         database username if different from config or default
 
@@ -2064,14 +2060,14 @@ def schema_exists(dbname, name,
 
     '''
     return bool(
-        schema_get(dbname, name,
+        schema_get(dbname, name, user=user,
                    db_user=db_user,
                    db_host=db_host,
                    db_port=db_port,
                    db_password=db_password))
 
 
-def schema_get(dbname, name,
+def schema_get(dbname, name, user=None,
                db_user=None, db_password=None,
                db_host=None, db_port=None):
     '''
@@ -2089,6 +2085,9 @@ def schema_get(dbname, name,
     name
        Schema name we look for
 
+    user
+        The system user the operation should be performed on behalf of
+
     db_user
         database username if different from config or default
 
@@ -2101,7 +2100,7 @@ def schema_get(dbname, name,
     db_port
         Database port if different from config or default
     '''
-    all_schemas = schema_list(dbname,
+    all_schemas = schema_list(dbname, user=user,
                               db_user=db_user,
                               db_host=db_host,
                               db_port=db_port,
@@ -2113,7 +2112,7 @@ def schema_get(dbname, name,
         return False
 
 
-def schema_list(dbname,
+def schema_list(dbname, user=None,
                 db_user=None, db_password=None,
                 db_host=None, db_port=None):
     '''
@@ -2128,6 +2127,9 @@ def schema_list(dbname,
     dbname
         Database name we query on
 
+    user
+        The system user the operation should be performed on behalf of
+
     db_user
         database username if different from config or default
 
@@ -2152,7 +2154,7 @@ def schema_list(dbname,
         'LEFT JOIN pg_roles ON pg_roles.oid = pg_namespace.nspowner '
     ]))
 
-    rows = psql_query(query,
+    rows = psql_query(query, runas=user,
                       host=db_host,
                       user=db_user,
                       port=db_port,
@@ -3089,6 +3091,8 @@ def datadir_init(name,
         password=None,
         encoding='UTF8',
         locale=None,
+        waldir=None,
+        checksums=False,
         runas=None):
     '''
     .. versionadded:: 2016.3.0
@@ -3119,8 +3123,22 @@ def datadir_init(name,
     locale
         The default locale for new databases
 
+    waldir
+        The transaction log (WAL) directory (default is to keep WAL
+        inside the data directory)
+
+        .. versionadded:: 2019.2.0
+
+    checksums
+        If True, the cluster will be created with data page checksums.
+
+        .. note::  Data page checksums are supported since PostgreSQL 9.3.
+
+        .. versionadded:: 2019.2.0
+
     runas
         The system user the operation should be performed on behalf of
+
     '''
     if datadir_exists(name):
         log.info('%s already exists', name)
