@@ -27,7 +27,7 @@ def __virtual__():
         return __virtualname__
     return (
         False,
-        '{0} module can only be loaded on SmartOS computed nodes'.format(
+        '{0} module can only be loaded on SmartOS compute nodes'.format(
             __virtualname__
         )
     )
@@ -57,8 +57,7 @@ def list_domains():
         salt '*' virt.list_domains
     '''
     data = __salt__['vmadm.list'](keyed=True)
-    vms = []
-    vms.append("UUID                                  TYPE  RAM      STATE             ALIAS")
+    vms = ["UUID                                  TYPE  RAM      STATE             ALIAS"]
     for vm in data:
         vms.append("{vmuuid}{vmtype}{vmram}{vmstate}{vmalias}".format(
             vmuuid=vm.ljust(38),
