@@ -19,9 +19,9 @@ import logging
 import copy
 
 # Import salt libs
+import salt.utils.data
 import salt.utils.files
 import salt.utils.decorators.path
-import salt.utils.locales
 import salt.utils.stringutils
 import salt.utils.user
 from salt.exceptions import CommandExecutionError
@@ -60,17 +60,18 @@ def _get_gecos(name):
     Retrieve GECOS field info and return it in dictionary form
     '''
     gecos_field = salt.utils.stringutils.to_unicode(
-        pwd.getpwnam(_quote_username(name)).pw_gecos).split(',', 3)
+        pwd.getpwnam(_quote_username(name)).pw_gecos).split(',', 4)
     if not gecos_field:
         return {}
     else:
         # Assign empty strings for any unspecified trailing GECOS fields
-        while len(gecos_field) < 4:
+        while len(gecos_field) < 5:
             gecos_field.append('')
-        return {'fullname': salt.utils.locales.sdecode(gecos_field[0]),
-                'roomnumber': salt.utils.locales.sdecode(gecos_field[1]),
-                'workphone': salt.utils.locales.sdecode(gecos_field[2]),
-                'homephone': salt.utils.locales.sdecode(gecos_field[3])}
+        return {'fullname': salt.utils.data.decode(gecos_field[0]),
+                'roomnumber': salt.utils.data.decode(gecos_field[1]),
+                'workphone': salt.utils.data.decode(gecos_field[2]),
+                'homephone': salt.utils.data.decode(gecos_field[3]),
+                'other': salt.utils.data.decode(gecos_field[4])}
 
 
 def _build_gecos(gecos_dict):
@@ -78,10 +79,11 @@ def _build_gecos(gecos_dict):
     Accepts a dictionary entry containing GECOS field names and their values,
     and returns a full GECOS comment string, to be used with usermod.
     '''
-    return '{0},{1},{2},{3}'.format(gecos_dict.get('fullname', ''),
-                                    gecos_dict.get('roomnumber', ''),
-                                    gecos_dict.get('workphone', ''),
-                                    gecos_dict.get('homephone', ''))
+    return '{0},{1},{2},{3},{4}'.format(gecos_dict.get('fullname', ''),
+                                        gecos_dict.get('roomnumber', ''),
+                                        gecos_dict.get('workphone', ''),
+                                        gecos_dict.get('homephone', ''),
+                                        gecos_dict.get('other', ''),).rstrip(',')
 
 
 def _update_gecos(name, key, value, root=None):
@@ -124,6 +126,7 @@ def add(name,
         roomnumber='',
         workphone='',
         homephone='',
+        other='',
         createhome=True,
         loginclass=None,
         root=None,
@@ -237,6 +240,8 @@ def add(name,
         chworkphone(name, workphone)
     if homephone:
         chhomephone(name, homephone)
+    if other:
+        chother(name, other)
     return True
 
 
@@ -507,6 +512,19 @@ def chhomephone(name, homephone):
     return _update_gecos(name, 'homephone', homephone)
 
 
+def chother(name, other):
+    '''
+    Change the user's other GECOS attribute
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.chother foobar
+    '''
+    return _update_gecos(name, 'other', other)
+
+
 def chloginclass(name, loginclass, root=None):
     '''
     Change the default login class of the user
@@ -588,9 +606,9 @@ def _format_info(data):
     Return user information in a pretty way
     '''
     # Put GECOS info into a list
-    gecos_field = salt.utils.stringutils.to_unicode(data.pw_gecos).split(',', 3)
-    # Make sure our list has at least four elements
-    while len(gecos_field) < 4:
+    gecos_field = salt.utils.stringutils.to_unicode(data.pw_gecos).split(',', 4)
+    # Make sure our list has at least five elements
+    while len(gecos_field) < 5:
         gecos_field.append('')
 
     return {'gid': data.pw_gid,
@@ -603,7 +621,8 @@ def _format_info(data):
             'fullname': gecos_field[0],
             'roomnumber': gecos_field[1],
             'workphone': gecos_field[2],
-            'homephone': gecos_field[3]}
+            'homephone': gecos_field[3],
+            'other': gecos_field[4]}
 
 
 @salt.utils.decorators.path.which('id')
