@@ -29,11 +29,11 @@ suppress refreshes when the metadata is less than a given number of seconds
 old.
 
 .. note::
-    Version numbers can be `version number string`, `latest` and `Not Found`.
-    Where `Not Found` means this module was not able to determine the version of
-    the software installed, it can also be used as the version number in sls
-    definitions file in these cases. Versions numbers are sorted in order of
-    0,`Not Found`,`order version numbers`,...,`latest`.
+    Version numbers can be ``version number string``, ``latest`` and ``Not
+    Found``, where ``Not Found`` means this module was not able to determine
+    the version of the software installed, it can also be used as the version
+    number in sls definitions file in these cases. Versions numbers are sorted
+    in order of 0, ``Not Found``, ``order version numbers``, ..., ``latest``.
 
 '''
 
@@ -66,6 +66,7 @@ import salt.utils.path
 import salt.utils.pkg
 import salt.utils.platform
 import salt.utils.versions
+import salt.utils.win_functions
 import salt.syspaths
 import salt.payload
 from salt.exceptions import MinionError
@@ -331,6 +332,7 @@ def version(*names, **kwargs):
         dict: The package name(s) with the installed versions.
 
     .. code-block:: cfg
+
         {['<version>', '<version>', ]} OR
         {'<package name>': ['<version>', '<version>', ]}
 
@@ -362,16 +364,35 @@ def version(*names, **kwargs):
     return ret
 
 
-def list_pkgs(versions_as_list=False, **kwargs):
+def list_pkgs(versions_as_list=False,
+              include_components=True,
+              include_updates=True,
+              **kwargs):
     '''
-    List the packages currently installed
+    List the packages currently installed.
+
+    .. note::
+        To view installed software as displayed in the Add/Remove Programs, set
+        ``include_components`` and ``include_updates`` to False.
 
     Args:
-        version_as_list (bool): Returns the versions as a list
+
+        versions_as_list (bool):
+            Returns the versions as a list
+
+        include_components (bool):
+            Include sub components of installed software. Default is ``True``
+
+        include_updates (bool):
+            Include software updates and Windows updates. Default is ``True``
 
     Kwargs:
-        saltenv (str): The salt environment to use. Default ``base``.
-        refresh (bool): Refresh package metadata. Default ``False`.
+
+        saltenv (str):
+            The salt environment to use. Default ``base``
+
+        refresh (bool):
+            Refresh package metadata. Default ``False``
 
     Returns:
         dict: A dictionary of installed software with versions installed
@@ -398,7 +419,9 @@ def list_pkgs(versions_as_list=False, **kwargs):
 
     ret = {}
     name_map = _get_name_map(saltenv)
-    for pkg_name, val_list in six.iteritems(_get_reg_software()):
+    for pkg_name, val_list in six.iteritems(
+            _get_reg_software(include_components=include_components,
+                              include_updates=include_updates)):
         if pkg_name in name_map:
             key = name_map[pkg_name]
             for val in val_list:
@@ -422,91 +445,320 @@ def list_pkgs(versions_as_list=False, **kwargs):
     return ret
 
 
-def _get_reg_software():
+def _get_reg_software(include_components=True,
+                      include_updates=True):
     '''
     This searches the uninstall keys in the registry to find a match in the sub
     keys, it will return a dict with the display name as the key and the
     version as the value
+
+    Args:
+
+        include_components (bool):
+            Include sub components of installed software. Default is ``True``
+
+        include_updates (bool):
+            Include software updates and Windows updates. Default is ``True``
+
+    Returns:
+        dict: A dictionary of installed software with versions installed
+
+    .. code-block:: cfg
+
+        {'<package_name>': '<version>'}
     '''
-    ignore_list = ['AddressBook',
-                   'Connection Manager',
-                   'DirectDrawEx',
-                   'Fontcore',
-                   'IE40',
-                   'IE4Data',
-                   'IE5BAKEX',
-                   'IEData',
-                   'MobileOptionPack',
-                   'SchedulingAgent',
-                   'WIC',
-                   'Not Found',
-                   '(value not set)',
-                   '',
-                   None]
-
+    # Logic for this can be found in this question:
+    # https://social.technet.microsoft.com/Forums/windows/en-US/d913471a-d7fb-448d-869b-da9025dcc943/where-does-addremove-programs-get-its-information-from-in-the-registry
+    # and also in the collectPlatformDependentApplicationData function in
+    # https://github.com/aws/amazon-ssm-agent/blob/master/agent/plugins/inventory/gatherers/application/dataProvider_windows.go
     reg_software = {}
-    hive = 'HKLM'
-    key = "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall"
 
-    def update(hive, key, reg_key, use_32bit):
-        # 2018 this code has been update to reflect some of utils/pkg/win.py logic
-        # i.e. check_reg, and checking of DisplayName and DisplayVersion.
-        d_name = ''
-        d_vers = ''
+    def skip_component(hive, key, sub_key, use_32bit):
+        '''
+        'SystemComponent' must be either absent or present with a value of 0,
+        because this value is usually set on programs that have been installed
+        via a Windows Installer Package (MSI).
+
+        Returns:
+            bool: True if the package needs to be skipped, otherwise False
+        '''
+        if include_components:
+            return False
+        if __utils__['reg.value_exists'](
+                hive=hive,
+                key='{0}\\{1}'.format(key, sub_key),
+                vname='SystemComponent',
+                use_32bit_registry=use_32bit):
+            if __utils__['reg.read_value'](
+                    hive=hive,
+                    key='{0}\\{1}'.format(key, sub_key),
+                    vname='SystemComponent',
+                    use_32bit_registry=use_32bit)['vdata'] > 0:
+                return True
+        return False
+
+    def skip_win_installer(hive, key, sub_key, use_32bit):
+        '''
+        'WindowsInstaller' must be either absent or present with a value of 0.
+        If the value is set to 1, then the application is included in the list
+        if and only if the corresponding compressed guid is also present in
+        HKLM:\\Software\\Classes\\Installer\\Products
+
+        Returns:
+            bool: True if the package needs to be skipped, otherwise False
+        '''
+        products_key = 'Software\\Classes\\Installer\\Products\\{0}'
+        if __utils__['reg.value_exists'](
+                hive=hive,
+                key='{0}\\{1}'.format(key, sub_key),
+                vname='WindowsInstaller',
+                use_32bit_registry=use_32bit):
+            if __utils__['reg.read_value'](
+                    hive=hive,
+                    key='{0}\\{1}'.format(key, sub_key),
+                    vname='WindowsInstaller',
+                    use_32bit_registry=use_32bit)['vdata'] > 0:
+                squid = salt.utils.win_functions.guid_to_squid(sub_key)
+                if not __utils__['reg.key_exists'](
+                        hive='HKLM',
+                        key=products_key.format(squid),
+                        use_32bit_registry=use_32bit):
+                    return True
+        return False
+
+    def skip_uninstall_string(hive, key, sub_key, use_32bit):
+        '''
+        'UninstallString' must be present, because it stores the command line
+        that gets executed by Add/Remove programs, when the user tries to
+        uninstall a program.
+
+        Returns:
+            bool: True if the package needs to be skipped, otherwise False
+        '''
+        if not __utils__['reg.value_exists'](
+                hive=hive,
+                key='{0}\\{1}'.format(key, sub_key),
+                vname='UninstallString',
+                use_32bit_registry=use_32bit):
+            return True
+        return False
+
+    def skip_release_type(hive, key, sub_key, use_32bit):
+        '''
+        'ReleaseType' must either be absent or if present must not have a
+        value set to 'Security Update', 'Update Rollup', or 'Hotfix', because
+        that indicates it's an update to an existing program.
+
+        Returns:
+            bool: True if the package needs to be skipped, otherwise False
+        '''
+        if include_updates:
+            return False
+        skip_types = ['Hotfix',
+                      'Security Update',
+                      'Update Rollup']
+        if __utils__['reg.value_exists'](
+                hive=hive,
+                key='{0}\\{1}'.format(key, sub_key),
+                vname='ReleaseType',
+                use_32bit_registry=use_32bit):
+            if __utils__['reg.read_value'](
+                    hive=hive,
+                    key='{0}\\{1}'.format(key, sub_key),
+                    vname='ReleaseType',
+                    use_32bit_registry=use_32bit)['vdata'] in skip_types:
+                return True
+        return False
+
+    def skip_parent_key(hive, key, sub_key, use_32bit):
+        '''
+        'ParentKeyName' must NOT be present, because that indicates it's an
+        update to the parent program.
+
+        Returns:
+            bool: True if the package needs to be skipped, otherwise False
+        '''
+        if __utils__['reg.value_exists'](
+                hive=hive,
+                key='{0}\\{1}'.format(key, sub_key),
+                vname='ParentKeyName',
+                use_32bit_registry=use_32bit):
+            return True
+
+        return False
+
+    def add_software(hive, key, sub_key, use_32bit):
+        '''
+        'DisplayName' must be present with a valid value, as this is reflected
+        as the software name returned by pkg.list_pkgs. Also, its value must
+        not start with 'KB' followed by 6 numbers - as that indicates a
+        Windows update.
+        '''
+        d_name_regdata = __utils__['reg.read_value'](
+            hive=hive,
+            key='{0}\\{1}'.format(key, sub_key),
+            vname='DisplayName',
+            use_32bit_registry=use_32bit)
 
-        d_name_regdata = __salt__['reg.read_value'](hive,
-                                                    '{0}\\{1}'.format(key, reg_key),
-                                                    'DisplayName',
-                                                    use_32bit)
         if (not d_name_regdata['success'] or
-            d_name_regdata['vtype'] not in ['REG_SZ', 'REG_EXPAND_SZ'] or
+                d_name_regdata['vtype'] not in ['REG_SZ', 'REG_EXPAND_SZ'] or
                 d_name_regdata['vdata'] in ['(value not set)', None, False]):
             return
         d_name = d_name_regdata['vdata']
 
-        d_vers_regdata = __salt__['reg.read_value'](hive,
-                                                    '{0}\\{1}'.format(key, reg_key),
-                                                    'DisplayVersion',
-                                                    use_32bit)
-        if (not d_vers_regdata['success'] or
-            d_vers_regdata['vtype'] not in ['REG_SZ', 'REG_EXPAND_SZ', 'REG_DWORD'] or
-                d_vers_regdata['vdata'] in [None, False]):
-            return
+        if not include_updates:
+            if re.match(r'^KB[0-9]{6}', d_name):
+                return
 
-        if isinstance(d_vers_regdata['vdata'], int):
-            d_vers = six.text_type(d_vers_regdata['vdata'])
-        else:
-            d_vers = d_vers_regdata['vdata']
+        d_vers_regdata = __utils__['reg.read_value'](
+            hive=hive,
+            key='{0}\\{1}'.format(key, sub_key),
+            vname='DisplayVersion',
+            use_32bit_registry=use_32bit)
 
-        if not d_vers or d_vers == '(value not set)':
-            d_vers = 'Not Found'
+        d_vers = 'Not Found'
+        if (d_vers_regdata['success'] and
+                d_vers_regdata['vtype'] in ['REG_SZ', 'REG_EXPAND_SZ', 'REG_DWORD']):
+            if isinstance(d_vers_regdata['vdata'], int):
+                d_vers = six.text_type(d_vers_regdata['vdata'])
+            elif d_vers_regdata['vdata'] and d_vers_regdata['vdata'] != '(value not set)':  # Check for blank values
+                d_vers = d_vers_regdata['vdata']
 
-        check_ok = False
-        for check_reg in ['UninstallString', 'QuietUninstallString', 'ModifyPath']:
-            check_regdata = __salt__['reg.read_value'](hive,
-                                                       '{0}\\{1}'.format(key, reg_key),
-                                                       check_reg,
-                                                       use_32bit)
-            if (not check_regdata['success'] or
-                check_regdata['vtype'] not in ['REG_SZ', 'REG_EXPAND_SZ'] or
-                    check_regdata['vdata'] in ['(value not set)', None, False]):
+        reg_software.setdefault(d_name, []).append(d_vers)
+
+    # Start gathering information from the registry
+    # HKLM Uninstall 64 bit
+    kwargs = {'hive': 'HKLM',
+              'key': 'Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall',
+              'use_32bit': False}
+    for sub_key in __utils__['reg.list_keys'](hive=kwargs['hive'],
+                                              key=kwargs['key']):
+        kwargs['sub_key'] = sub_key
+        if skip_component(**kwargs):
+            continue
+        if skip_win_installer(**kwargs):
+            continue
+        if skip_uninstall_string(**kwargs):
+            continue
+        if skip_release_type(**kwargs):
+            continue
+        if skip_parent_key(**kwargs):
+            continue
+        add_software(**kwargs)
+
+    # HKLM Uninstall 32 bit
+    kwargs['use_32bit'] = True
+    for sub_key in __utils__['reg.list_keys'](hive=kwargs['hive'],
+                                              key=kwargs['key'],
+                                              use_32bit_registry=kwargs['use_32bit']):
+        kwargs['sub_key'] = sub_key
+        if skip_component(**kwargs):
+            continue
+        if skip_win_installer(**kwargs):
+            continue
+        if skip_uninstall_string(**kwargs):
+            continue
+        if skip_release_type(**kwargs):
+            continue
+        if skip_parent_key(**kwargs):
+            continue
+        add_software(**kwargs)
+
+    # HKLM Uninstall 64 bit
+    kwargs = {'hive': 'HKLM',
+              'key': 'Software\\Classes\\Installer\\Products',
+              'use_32bit': False}
+    userdata_key = 'Software\\Microsoft\\Windows\\CurrentVersion\\Installer\\' \
+                   'UserData\\S-1-5-18\\Products'
+    for sub_key in __utils__['reg.list_keys'](hive=kwargs['hive'], key=kwargs['key']):
+        # If the key does not exist in userdata, skip it
+        if not __utils__['reg.key_exists'](
+                hive=kwargs['hive'],
+                key='{0}\\{1}'.format(userdata_key, sub_key)):
+            continue
+        kwargs['sub_key'] = sub_key
+        if skip_component(**kwargs):
+            continue
+        if skip_win_installer(**kwargs):
+            continue
+        add_software(**kwargs)
+
+    # Uninstall for each user on the system (HKU), 64 bit
+    # This has a propensity to take a while on a machine where many users have
+    # logged in. Untested in such a scenario
+    hive_hku = 'HKU'
+    uninstall_key = '{0}\\Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall'
+    product_key = '{0}\\Software\\Microsoft\\Installer\\Products'
+    user_data_key = 'Software\\Microsoft\\Windows\\CurrentVersion\\Installer\\' \
+                    'UserData\\{0}\\Products\\{1}'
+    for user_guid in __utils__['reg.list_keys'](hive=hive_hku):
+        kwargs = {'hive': hive_hku,
+                  'key': uninstall_key.format(user_guid),
+                  'use_32bit': False}
+        for sub_key in __utils__['reg.list_keys'](hive=kwargs['hive'],
+                                                  key=kwargs['key']):
+            kwargs['sub_key'] = sub_key
+            if skip_component(**kwargs):
                 continue
-            else:
-                check_ok = True
+            if skip_win_installer(**kwargs):
+                continue
+            if skip_uninstall_string(**kwargs):
+                continue
+            if skip_release_type(**kwargs):
+                continue
+            if skip_parent_key(**kwargs):
+                continue
+            add_software(**kwargs)
 
-        if not check_ok:
-            return
+        # While we have the user guid, we're gong to check userdata in HKLM
+        for sub_key in __utils__['reg.list_keys'](hive=hive_hku,
+                                                  key=product_key.format(user_guid)):
+            kwargs = {'hive': 'HKLM',
+                      'key': user_data_key.format(user_guid, sub_key),
+                      'sub_key': 'InstallProperties',
+                      'use_32bit': False}
+            if __utils__['reg.key_exists'](hive=kwargs['hive'],
+                                           key=kwargs['key']):
+                if skip_component(**kwargs):
+                    continue
+                add_software(**kwargs)
 
-        if d_name not in ignore_list:
-            # some MS Office updates don't register a product name which means
-            # their information is useless
-            reg_software.setdefault(d_name, []).append(d_vers)
+    # Uninstall for each user on the system (HKU), 32 bit
+    for user_guid in __utils__['reg.list_keys'](hive=hive_hku,
+                                                use_32bit_registry=True):
+        kwargs = {'hive': hive_hku,
+                  'key': uninstall_key.format(user_guid),
+                  'use_32bit': True}
+        for sub_key in __utils__['reg.list_keys'](hive=kwargs['hive'],
+                                                  key=kwargs['key'],
+                                                  use_32bit_registry=kwargs['use_32bit']):
+            kwargs['sub_key'] = sub_key
+            if skip_component(**kwargs):
+                continue
+            if skip_win_installer(**kwargs):
+                continue
+            if skip_uninstall_string(**kwargs):
+                continue
+            if skip_release_type(**kwargs):
+                continue
+            if skip_parent_key(**kwargs):
+                continue
+            add_software(**kwargs)
 
-    for reg_key in __salt__['reg.list_keys'](hive, key):
-        update(hive, key, reg_key, False)
-
-    for reg_key in __salt__['reg.list_keys'](hive, key, True):
-        update(hive, key, reg_key, True)
+        # While we have the user guid, we're gong to check userdata in HKLM
+        for sub_key_2 in __utils__['reg.list_keys'](hive=hive_hku,
+                                                    key=product_key.format(user_guid),
+                                                    use_32bit_registry=True):
+            kwargs = {'hive': 'HKLM',
+                      'key': user_data_key.format(user_guid, sub_key_2),
+                      'sub_key': 'InstallProperties',
+                      'use_32bit': True}
+            if __utils__['reg.key_exists'](hive=kwargs['hive'],
+                                           key=kwargs['key'],
+                                           use_32bit_registry=kwargs['use_32bit']):
+                if skip_component(**kwargs):
+                    continue
+                add_software(**kwargs)
 
     return reg_software
 
@@ -581,28 +833,79 @@ def _refresh_db_conditional(saltenv, **kwargs):
 
 
 def refresh_db(**kwargs):
-    '''
-    Fetches metadata files and calls :py:func:`pkg.genrepo
-    <salt.modules.win_pkg.genrepo>` to compile updated repository metadata.
+    r'''
+    Generates the local software metadata database (`winrepo.p`) on the minion.
+    The database is stored in a serialized format located by default at the
+    following location:
 
-    Kwargs:
+    `C:\salt\var\cache\salt\minion\files\base\win\repo-ng\winrepo.p`
 
-        saltenv (str): Salt environment. Default: ``base``
+    This module performs the following steps to generate the software metadata
+    database:
 
-        verbose (bool):
-            Return verbose data structure which includes 'success_list', a list
-            of all sls files and the package names contained within. Default
-            'False'
+    - Fetch the package definition files (.sls) from `winrepo_source_dir`
+      (default `salt://win/repo-ng`) and cache them in
+      `<cachedir>\files\<saltenv>\<winrepo_source_dir>`
+      (default: `C:\salt\var\cache\salt\minion\files\base\win\repo-ng`)
+    - Call :py:func:`pkg.genrepo <salt.modules.win_pkg.genrepo>` to parse the
+      package definition files and generate the repository metadata database
+      file (`winrepo.p`)
+    - Return the report received from
+      :py:func:`pkg.genrepo <salt.modules.win_pkg.genrepo>`
 
-        failhard (bool):
-            If ``True``, an error will be raised if any repo SLS files failed to
-            process. If ``False``, no error will be raised, and a dictionary
-            containing the full results will be returned.
+    The default winrepo directory on the master is `/srv/salt/win/repo-ng`. All
+    files that end with `.sls` in this and all subdirectories will be used to
+    generate the repository metadata database (`winrepo.p`).
+
+    .. note::
+        - Hidden directories (directories beginning with '`.`', such as
+          '`.git`') will be ignored.
+
+    .. note::
+        There is no need to call `pkg.refresh_db` every time you work with the
+        pkg module. Automatic refresh will occur based on the following minion
+        configuration settings:
+
+        - `winrepo_cache_expire_min`
+        - `winrepo_cache_expire_max`
+
+        However, if the package definition files have changed, as would be the
+        case if you are developing a new package definition, this function
+        should be called to ensure the minion has the latest information about
+        packages available to it.
+
+    .. warning::
+        Directories and files fetched from <winrepo_source_dir>
+        (`/srv/salt/win/repo-ng`) will be processed in alphabetical order. If
+        two or more software definition files contain the same name, the last
+        one processed replaces all data from the files processed before it.
+
+    For more information see
+    :ref:`Windows Software Repository <windows-package-manager>`
+
+    Arguments:
+
+    saltenv (str): Salt environment. Default: ``base``
+
+    verbose (bool):
+        Return a verbose data structure which includes 'success_list', a
+        list of all sls files and the package names contained within.
+        Default is 'False'
+
+    failhard (bool):
+        If ``True``, an error will be raised if any repo SLS files fails to
+        process. If ``False``, no error will be raised, and a dictionary
+        containing the full results will be returned.
 
     Returns:
         dict: A dictionary containing the results of the database refresh.
 
-    .. Warning::
+    .. note::
+        A result with a `total: 0` generally means that the files are in the
+        wrong location on the master. Try running the following command on the
+        minion: `salt-call -l debug pkg.refresh saltenv=base`
+
+    .. warning::
         When calling this command from a state using `module.run` be sure to
         pass `failhard: False`. Otherwise the state will report failure if it
         encounters a bad software definition file.
@@ -648,12 +951,13 @@ def refresh_db(**kwargs):
         )
 
     # Cache repo-ng locally
+    log.info('Fetching *.sls files from {0}'.format(repo_details.winrepo_source_dir))
     __salt__['cp.cache_dir'](
-        repo_details.winrepo_source_dir,
-        saltenv,
-        include_pat='*.sls'
+        path=repo_details.winrepo_source_dir,
+        saltenv=saltenv,
+        include_pat='*.sls',
+        exclude_pat=r'E@\/\..*?\/'  # Exclude all hidden directories (.git)
     )
-
     return genrepo(saltenv=saltenv, verbose=verbose, failhard=failhard)
 
 
@@ -754,6 +1058,10 @@ def genrepo(**kwargs):
             to process. If ``False``, no error will be raised, and a dictionary
             containing the full results will be returned.
 
+    .. note::
+        - Hidden directories (directories beginning with '`.`', such as
+          '`.git`') will be ignored.
+
     Returns:
         dict: A dictionary of the results of the command
 
@@ -777,9 +1085,16 @@ def genrepo(**kwargs):
     repo_details = _get_repo_details(saltenv)
 
     for root, _, files in salt.utils.path.os_walk(repo_details.local_dest, followlinks=False):
+
+        # Skip hidden directories (.git)
+        if re.search(r'[\\/]\..*', root):
+            log.debug('Skipping files in directory: {0}'.format(root))
+            continue
+
         short_path = os.path.relpath(root, repo_details.local_dest)
         if short_path == '.':
             short_path = ''
+
         for name in files:
             if name.endswith('.sls'):
                 total_files_processed += 1
@@ -891,7 +1206,7 @@ def _repo_process_pkg_sls(filename, short_path_name, ret, successful_verbose):
         else:
             ret.setdefault('repo', {}).update(config)
             ret.setdefault('name_map', {}).update(revmap)
-            successful_verbose[short_path_name] = config.keys()
+            successful_verbose[short_path_name] = list(config.keys())
     elif config:
         return _failed_compile('Compiled contents', 'not a dictionary/hash')
     else:
@@ -1035,7 +1350,8 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
             .. versionadded:: 2016.11.0
 
     Returns:
-        dict: Return a dict containing the new package names and versions
+        dict: Return a dict containing the new package names and versions. If
+        the package is already installed, an empty dict is returned.
 
         If the package is installed by ``pkg.install``:
 
@@ -1044,12 +1360,6 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
             {'<package>': {'old': '<old-version>',
                            'new': '<new-version>'}}
 
-        If the package is already installed:
-
-        .. code-block:: cfg
-
-            {'<package>': {'current': '<current-version>'}}
-
     The following example will refresh the winrepo and install a single
     package, 7zip.
 
@@ -1143,7 +1453,6 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
 
     # Loop through each package
     changed = []
-    latest = []
     for pkg_name, options in six.iteritems(pkg_params):
 
         # Load package information for the package
@@ -1155,28 +1464,33 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
             ret[pkg_name] = 'Unable to locate package {0}'.format(pkg_name)
             continue
 
-        version_num = options.get('version', '')
+        version_num = options.get('version')
         #  Using the salt cmdline with version=5.3 might be interpreted
         #  as a float it must be converted to a string in order for
         #  string matching to work.
         if not isinstance(version_num, six.string_types) and version_num is not None:
             version_num = six.text_type(version_num)
 
+        # If the version was not passed, version_num will be None
         if not version_num:
             if pkg_name in old:
-                log.debug('A version (%s) already installed for package '
-                          '%s', version_num, pkg_name)
+                log.debug('pkg.install: \'%s\' version \'%s\' is already installed',
+                          pkg_name, old[pkg_name][0])
                 continue
-            # following can be version number or latest or Not Found
+            # Get the most recent version number available from winrepo.p
+            # May also return `latest` or an empty string
             version_num = _get_latest_pkg_version(pkginfo)
 
         if version_num == 'latest' and 'latest' not in pkginfo:
+            # Get the most recent version number available from winrepo.p
+            # May also return `latest` or an empty string
             version_num = _get_latest_pkg_version(pkginfo)
 
         # Check if the version is already installed
         if version_num in old.get(pkg_name, []):
             # Desired version number already installed
-            ret[pkg_name] = {'current': version_num}
+            log.debug('pkg.install: \'%s\' version \'%s\' is already installed',
+                      pkg_name, version_num)
             continue
         # If version number not installed, is the version available?
         elif version_num != 'latest' and version_num not in pkginfo:
@@ -1185,9 +1499,6 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
             ret[pkg_name] = {'not found': version_num}
             continue
 
-        if 'latest' in pkginfo:
-            latest.append(pkg_name)
-
         # Get the installer settings from winrepo.p
         installer = pkginfo[version_num].get('installer', '')
         cache_dir = pkginfo[version_num].get('cache_dir', False)
@@ -1209,11 +1520,11 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
             # single files
             if cache_dir and installer.startswith('salt:'):
                 path, _ = os.path.split(installer)
-                __salt__['cp.cache_dir'](path,
-                                         saltenv,
-                                         False,
-                                         None,
-                                         'E@init.sls$')
+                __salt__['cp.cache_dir'](path=path,
+                                         saltenv=saltenv,
+                                         include_empty=False,
+                                         include_pat=None,
+                                         exclude_pat='E@init.sls$')
 
             # Check to see if the cache_file is cached... if passed
             if cache_file and cache_file.startswith('salt:'):
@@ -1272,18 +1583,18 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
 
         # Fix non-windows slashes
         cached_pkg = cached_pkg.replace('/', '\\')
-        cache_path, _ = os.path.split(cached_pkg)
+        cache_path = os.path.dirname(cached_pkg)
 
         # Compare the hash sums
         source_hash = pkginfo[version_num].get('source_hash', False)
         if source_hash:
             source_sum = _get_source_sum(source_hash, cached_pkg, saltenv)
-            log.debug('Source %s hash: %s',
+            log.debug('pkg.install: Source %s hash: %s',
                       source_sum['hash_type'], source_sum['hsum'])
 
             cached_pkg_sum = salt.utils.hashutils.get_hash(cached_pkg,
                                                            source_sum['hash_type'])
-            log.debug('Package %s hash: %s',
+            log.debug('pkg.install: Package %s hash: %s',
                       source_sum['hash_type'], cached_pkg_sum)
 
             if source_sum['hsum'] != cached_pkg_sum:
@@ -1291,7 +1602,7 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
                     ("Source hash '{0}' does not match package hash"
                      " '{1}'").format(source_sum['hsum'], cached_pkg_sum)
                 )
-            log.debug('Source hash matches package hash.')
+            log.debug('pkg.install: Source hash matches package hash.')
 
         # Get install flags
 
@@ -1402,15 +1713,6 @@ def install(name=None, refresh=False, pkgs=None, **kwargs):
     # preparation for the comparison below.
     __salt__['pkg_resource.stringify'](old)
 
-    # For installers that have no specific version (ie: chrome)
-    # The software definition file will have a version of 'latest'
-    # In that case there's no way to know which version has been installed
-    # Just return the current installed version
-    if latest:
-        for pkg_name in latest:
-            if old.get(pkg_name, 'old') == new.get(pkg_name, 'new'):
-                ret[pkg_name] = {'current': new[pkg_name]}
-
     # Check for changes in the registry
     difference = salt.utils.data.compare_dicts(old, new)
 
@@ -1820,7 +2122,7 @@ def get_repo_data(saltenv='base'):
         serial = salt.payload.Serial(__opts__)
         with salt.utils.files.fopen(repo_details.winrepo_file, 'rb') as repofile:
             try:
-                repodata = salt.utils.data.decode(serial.loads(repofile.read(), encoding='utf-8') or {})
+                repodata = salt.utils.data.decode(serial.loads(repofile.read()) or {})
                 __context__['winrepo.data'] = repodata
                 return repodata
             except Exception as exc:
