@@ -18,6 +18,7 @@ from __future__ import absolute_import, unicode_literals, print_function
 import ctypes
 import logging
 import time
+import platform
 from datetime import datetime
 
 # Import salt libs
@@ -304,7 +305,7 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
                                         force_close, reboot)
         return True
     except pywintypes.error as exc:
-        (number, context, message) = exc
+        (number, context, message) = exc.args
         log.error('Failed to shutdown the system')
         log.error('nbr: %s', number)
         log.error('ctx: %s', context)
@@ -347,7 +348,7 @@ def shutdown_abort():
         win32api.AbortSystemShutdown('127.0.0.1')
         return True
     except pywintypes.error as exc:
-        (number, context, message) = exc
+        (number, context, message) = exc.args
         log.error('Failed to abort system shutdown')
         log.error('nbr: %s', number)
         log.error('ctx: %s', context)
@@ -486,7 +487,7 @@ def set_computer_desc(desc=None):
     try:
         win32net.NetServerSetInfo(None, 101, system_info)
     except win32net.error as exc:
-        (number, context, message) = exc
+        (number, context, message) = exc.args
         log.error('Failed to update system')
         log.error('nbr: %s', number)
         log.error('ctx: %s', context)
@@ -511,13 +512,30 @@ def get_system_info():
 
     .. code-block:: bash
 
-        salt 'minion-id' system.get_info
+        salt 'minion-id' system.get_system_info
     '''
+    def byte_calc(val):
+        val = float(val)
+        if val < 2**10:
+            return '{0:.3f}B'.format(val)
+        elif val < 2**20:
+            return '{0:.3f}KB'.format(val / 2**10)
+        elif val < 2**30:
+            return '{0:.3f}MB'.format(val / 2**20)
+        elif val < 2**40:
+            return '{0:.3f}GB'.format(val / 2**30)
+        else:
+            return '{0:.3f}TB'.format(val / 2**40)
+
+    # Connect to WMI
+    pythoncom.CoInitialize()
+    conn = wmi.WMI()
+
+    # Lookup dicts for Win32_OperatingSystem
     os_type = {1: 'Work Station',
                2: 'Domain Controller',
                3: 'Server'}
-    pythoncom.CoInitialize()
-    conn = wmi.WMI()
+
     system = conn.Win32_OperatingSystem()[0]
     ret = {'name': get_computer_name(),
            'description': system.Description,
@@ -535,12 +553,73 @@ def get_system_info():
            'system_drive': system.SystemDrive,
            'os_version': system.Version,
            'windows_directory': system.WindowsDirectory}
+
+    # lookup dicts for Win32_ComputerSystem
+    domain_role = {0: 'Standalone Workstation',
+                   1: 'Member Workstation',
+                   2: 'Standalone Server',
+                   3: 'Member Server',
+                   4: 'Backup Domain Controller',
+                   5: 'Primary Domain Controller'}
+    warning_states = {1: 'Other',
+                      2: 'Unknown',
+                      3: 'Safe',
+                      4: 'Warning',
+                      5: 'Critical',
+                      6: 'Non-recoverable'}
+    pc_system_types = {0: 'Unspecified',
+                       1: 'Desktop',
+                       2: 'Mobile',
+                       3: 'Workstation',
+                       4: 'Enterprise Server',
+                       5: 'SOHO Server',
+                       6: 'Appliance PC',
+                       7: 'Performance Server',
+                       8: 'Maximum'}
     system = conn.Win32_ComputerSystem()[0]
-    ret.update({'hardware_manufacturer': system.Manufacturer,
-                'hardware_model': system.Model,
-                'processors': system.NumberOfProcessors,
-                'processors_logical': system.NumberOfLogicalProcessors,
-                'system_type': system.SystemType})
+    # Get pc_system_type depending on Windows version
+    if platform.release() in ['Vista', '7', '8']:
+        # Types for Vista, 7, and 8
+        pc_system_type = pc_system_types[system.PCSystemType]
+    else:
+        # New types were added with 8.1 and newer
+        pc_system_types.update({8: 'Slate', 9: 'Maximum'})
+        pc_system_type = pc_system_types[system.PCSystemType]
+    ret.update({
+        'bootup_state': system.BootupState,
+        'caption': system.Caption,
+        'chassis_bootup_state': warning_states[system.ChassisBootupState],
+        'chassis_sku_number': system.ChassisSKUNumber,
+        'dns_hostname': system.DNSHostname,
+        'domain': system.Domain,
+        'domain_role': domain_role[system.DomainRole],
+        'hardware_manufacturer': system.Manufacturer,
+        'hardware_model': system.Model,
+        'network_server_mode_enabled': system.NetworkServerModeEnabled,
+        'part_of_domain': system.PartOfDomain,
+        'pc_system_type': pc_system_type,
+        'power_state': system.PowerState,
+        'status': system.Status,
+        'system_type': system.SystemType,
+        'total_physical_memory': byte_calc(system.TotalPhysicalMemory),
+        'total_physical_memory_raw': system.TotalPhysicalMemory,
+        'thermal_state': warning_states[system.ThermalState],
+        'workgroup': system.Workgroup
+    })
+    # Get processor information
+    processors = conn.Win32_Processor()
+    ret['processors'] = 0
+    ret['processors_logical'] = 0
+    ret['processor_cores'] = 0
+    ret['processor_cores_enabled'] = 0
+    ret['processor_manufacturer'] = processors[0].Manufacturer
+    ret['processor_max_clock_speed'] = six.text_type(processors[0].MaxClockSpeed) + 'MHz'
+    for system in processors:
+        ret['processors'] += 1
+        ret['processors_logical'] += system.NumberOfLogicalProcessors
+        ret['processor_cores'] += system.NumberOfCores
+        ret['processor_cores_enabled'] += system.NumberOfEnabledCore
+
     system = conn.Win32_BIOS()[0]
     ret.update({'hardware_serial': system.SerialNumber,
                 'bios_manufacturer': system.Manufacturer,
@@ -1019,7 +1098,7 @@ def set_system_date_time(years=None,
     try:
         date_time = win32api.GetLocalTime()
     except win32api.error as exc:
-        (number, context, message) = exc
+        (number, context, message) = exc.args
         log.error('Failed to get local time')
         log.error('nbr: %s', number)
         log.error('ctx: %s', context)
@@ -1174,17 +1253,15 @@ def get_pending_component_servicing():
 
         salt '*' system.get_pending_component_servicing
     '''
-    vname = '(Default)'
     key = r'SOFTWARE\Microsoft\Windows\CurrentVersion\Component Based Servicing\RebootPending'
 
-    reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
-
     # So long as the registry key exists, a reboot is pending.
-    if reg_ret['success']:
-        log.debug('Found key: %s', key)
+    if __utils__['reg.key_exists']('HKLM', key):
+        log.debug('Key exists: %s', key)
         return True
     else:
-        log.debug('Unable to access key: %s', key)
+        log.debug('Key does not exist: %s', key)
+
     return False
 
 
@@ -1205,29 +1282,24 @@ def get_pending_domain_join():
 
         salt '*' system.get_pending_domain_join
     '''
-    vname = '(Default)'
     base_key = r'SYSTEM\CurrentControlSet\Services\Netlogon'
     avoid_key = r'{0}\AvoidSpnSet'.format(base_key)
     join_key = r'{0}\JoinDomain'.format(base_key)
 
     # If either the avoid_key or join_key is present,
     # then there is a reboot pending.
-
-    avoid_reg_ret = __salt__['reg.read_value']('HKLM', avoid_key, vname)
-
-    if avoid_reg_ret['success']:
-        log.debug('Found key: %s', avoid_key)
+    if __utils__['reg.key_exists']('HKLM', avoid_key):
+        log.debug('Key exists: %s', avoid_key)
         return True
     else:
-        log.debug('Unable to access key: %s', avoid_key)
+        log.debug('Key does not exist: %s', avoid_key)
 
-    join_reg_ret = __salt__['reg.read_value']('HKLM', join_key, vname)
-
-    if join_reg_ret['success']:
-        log.debug('Found key: %s', join_key)
+    if __utils__['reg.key_exists']('HKLM', join_key):
+        log.debug('Key exists: %s', join_key)
         return True
     else:
-        log.debug('Unable to access key: %s', join_key)
+        log.debug('Key does not exist: %s', join_key)
+
     return False
 
 
@@ -1288,7 +1360,7 @@ def get_pending_servermanager():
     key = r'SOFTWARE\Microsoft\ServerManager'
 
     # There are situations where it's possible to have '(value not set)' as
-    # the value data, and since an actual reboot wont be pending in that
+    # the value data, and since an actual reboot won't be pending in that
     # instance, just catch instances where we try unsuccessfully to cast as int.
 
     reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
@@ -1321,17 +1393,15 @@ def get_pending_update():
 
         salt '*' system.get_pending_update
     '''
-    vname = '(Default)'
     key = r'SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update\RebootRequired'
 
-    reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
-
     # So long as the registry key exists, a reboot is pending.
-    if reg_ret['success']:
-        log.debug('Found key: %s', key)
+    if __utils__['reg.key_exists']('HKLM', key):
+        log.debug('Key exists: %s', key)
         return True
     else:
-        log.debug('Unable to access key: %s', key)
+        log.debug('Key does not exist: %s', key)
+
     return False
 
 
@@ -1421,7 +1491,9 @@ def get_pending_reboot():
     '''
 
     # Order the checks for reboot pending in most to least likely.
-    checks = (get_pending_update, get_pending_file_rename, get_pending_servermanager,
+    checks = (get_pending_update,
+              get_pending_file_rename,
+              get_pending_servermanager,
               get_pending_component_servicing,
               get_reboot_required_witnessed,
               get_pending_computer_name,
