@@ -185,9 +185,8 @@ a return like::
 .. |406| replace:: requested Content-Type not available
 .. |500| replace:: internal server error
 '''
-from __future__ import absolute_import, print_function, unicode_literals
-
 # Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import time
 import fnmatch
 import logging
@@ -209,8 +208,8 @@ import salt.ext.six as six
 import salt.netapi
 import salt.utils.args
 import salt.utils.event
-import salt.utils.jid
 import salt.utils.json
+import salt.utils.minions
 import salt.utils.yaml
 import salt.utils.zeromq
 from salt.utils.event import tagify
@@ -503,6 +502,8 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler):  # pylint: disable=W0223
         '''
         # timeout all the futures
         self.timeout_futures()
+        # clear local_client objects to disconnect event publisher's IOStream connections
+        del self.saltclients
 
     def on_connection_close(self):
         '''
@@ -616,7 +617,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
     '''
     def get(self):
         '''
-        All logins are done over post, this is a parked enpoint
+        All logins are done over post, this is a parked endpoint
 
         .. http:get:: /login
 
@@ -629,7 +630,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
             curl -i localhost:8000/login
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /login HTTP/1.1
             Host: localhost:8000
@@ -637,7 +638,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 401 Unauthorized
             Content-Type: application/json
@@ -656,7 +657,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
     # TODO: make asynchronous? Underlying library isn't... and we ARE making disk calls :(
     def post(self):
         '''
-        :ref:`Authenticate  <rest_tornado-auth>` against Salt's eauth system
+        :ref:`Authenticate <rest_tornado-auth>` against Salt's eauth system
 
         .. http:post:: /login
 
@@ -684,7 +685,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                     -d password='saltpass' \\
                     -d eauth='pam'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST / HTTP/1.1
             Host: localhost:8000
@@ -696,7 +697,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Type: application/json
@@ -742,8 +743,19 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
         # Grab eauth config for the current backend for the current user
         try:
-            perms = self.application.opts['external_auth'][token['eauth']][token['name']]
+            eauth = self.application.opts['external_auth'][token['eauth']]
+            # Get sum of '*' perms, user-specific perms, and group-specific perms
+            perms = eauth.get(token['name'], [])
+            perms.extend(eauth.get('*', []))
 
+            if 'groups' in token and token['groups']:
+                user_groups = set(token['groups'])
+                eauth_groups = set([i.rstrip('%') for i in eauth.keys() if i.endswith('%')])
+
+                for group in user_groups & eauth_groups:
+                    perms.extend(eauth['{0}%'.format(group)])
+
+            perms = sorted(list(set(perms)))
         # If we can't find the creds, then they aren't authorized
         except KeyError:
             self.send_error(401)
@@ -777,7 +789,7 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
     '''
     def get(self):
         '''
-        An enpoint to determine salt-api capabilities
+        An endpoint to determine salt-api capabilities
 
         .. http:get:: /
 
@@ -793,7 +805,7 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
             curl -i localhost:8000
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET / HTTP/1.1
             Host: localhost:8000
@@ -801,7 +813,7 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Type: application/json
@@ -845,7 +857,7 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                     -d fun='test.ping' \\
                     -d arg
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST / HTTP/1.1
             Host: localhost:8000
@@ -857,11 +869,12 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
             fun=test.ping&arg&client=local&tgt=*
 
         **Example response:**
+
         Responses are an in-order list of the lowstate's return data. In the
         event of an exception running a command the return will be a string
         instead of a mapping.
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 200
@@ -876,10 +889,11 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                 ms-4: true
 
         .. admonition:: multiple commands
+
             Note that if multiple :term:`lowstate` structures are sent, the Salt
             API will execute them in serial, and will not stop execution upon failure
             of a previous job. If you need to have commands executed in order and
-            stop on failure please use compount-command-execution.
+            stop on failure please use compound-command-execution.
 
         '''
         # if you aren't authenticated, redirect to login
@@ -930,14 +944,27 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
         '''
         Dispatch local client commands
         '''
-        # Generate jid before triggering a job to subscribe all returns from minions
-        chunk['jid'] = salt.utils.jid.gen_jid(self.application.opts)
+        # Generate jid and find all minions before triggering a job to subscribe all returns from minions
+        chunk['jid'] = salt.utils.jid.gen_jid(self.application.opts) if not chunk.get('jid', None) else chunk['jid']
+        minions = set(self.ckminions.check_minions(chunk['tgt'], chunk.get('tgt_type', 'glob')))
+
+        def subscribe_minion(minion):
+            salt_evt = self.application.event_listener.get_event(
+                     self,
+                     tag='salt/job/{}/ret/{}'.format(chunk['jid'], minion),
+                     matcher=EventListener.exact_matcher)
+            syndic_evt = self.application.event_listener.get_event(
+                    self,
+                    tag='syndic/job/{}/ret/{}'.format(chunk['jid'], minion),
+                    matcher=EventListener.exact_matcher)
+            return salt_evt, syndic_evt
 
         # start listening for the event before we fire the job to avoid races
-        events = [
-            self.application.event_listener.get_event(self, tag='salt/job/'+chunk['jid']),
-            self.application.event_listener.get_event(self, tag='syndic/job/'+chunk['jid']),
-        ]
+        events = []
+        for minion in minions:
+            salt_evt, syndic_evt = subscribe_minion(minion)
+            events.append(salt_evt)
+            events.append(syndic_evt)
 
         f_call = self._format_call_run_job_async(chunk)
         # fire a job off
@@ -953,6 +980,12 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                     pass
             raise tornado.gen.Return('No minions matched the target. No command was sent, no jid was assigned.')
 
+        # get_event for missing minion
+        for minion in list(set(pub_data['minions']) - set(minions)):
+            salt_evt, syndic_evt = subscribe_minion(minion)
+            events.append(salt_evt)
+            events.append(syndic_evt)
+
         # Map of minion_id -> returned for all minions we think we need to wait on
         minions = {m: False for m in pub_data['minions']}
 
@@ -977,7 +1010,8 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                                                       is_finished)
 
         def more_todo():
-            '''Check if there are any more minions we are waiting on returns from
+            '''
+            Check if there are any more minions we are waiting on returns from
             '''
             return any(x is False for x in six.itervalues(minions))
 
@@ -1006,7 +1040,10 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                         cancel_inflight_futures()
                         raise tornado.gen.Return(chunk_ret)
                     continue
+
                 f_result = f.result()
+                if f in events:
+                    events.remove(f)
                 # if this is a start, then we need to add it to the pile
                 if f_result['tag'].endswith('/new'):
                     for minion_id in f_result['data']['minions']:
@@ -1016,7 +1053,6 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                     chunk_ret[f_result['data']['id']] = f_result['data']['return']
                     # clear finished event future
                     minions[f_result['data']['id']] = True
-
                     # if there are no more minions to wait for, then we are done
                     if not more_todo() and min_wait_time.done():
                         cancel_inflight_futures()
@@ -1025,11 +1061,6 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
             except TimeoutException:
                 pass
 
-            if f == events[0]:
-                events[0] = self.application.event_listener.get_event(self, tag='salt/job/'+chunk['jid'])
-            else:
-                events[1] = self.application.event_listener.get_event(self, tag='syndic/job/'+chunk['jid'])
-
     @tornado.gen.coroutine
     def job_not_running(self, jid, tgt, tgt_type, minions, is_finished):
         '''
@@ -1090,13 +1121,15 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
         '''
         Disbatch runner client commands
         '''
+        full_return = chunk.pop('full_return', False)
         pub_data = self.saltclients['runner'](chunk)
         tag = pub_data['tag'] + '/ret'
         try:
             event = yield self.application.event_listener.get_event(self, tag=tag)
 
             # only return the return data
-            raise tornado.gen.Return(event['data']['return'])
+            ret = event if full_return else event['data']['return']
+            raise tornado.gen.Return(ret)
         except TimeoutException:
             raise tornado.gen.Return('Timeout waiting for runner to execute')
 
@@ -1144,7 +1177,7 @@ class MinionSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
             curl -i localhost:8000/minions/ms-3
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /minions/ms-3 HTTP/1.1
             Host: localhost:8000
@@ -1152,7 +1185,7 @@ class MinionSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 129005
@@ -1205,7 +1238,7 @@ class MinionSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
                 -d tgt='*' \\
                 -d fun='status.diskusage'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /minions HTTP/1.1
             Host: localhost:8000
@@ -1217,7 +1250,7 @@ class MinionSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 202 Accepted
             Content-Length: 86
@@ -1272,7 +1305,7 @@ class JobsSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
             curl -i localhost:8000/jobs
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /jobs HTTP/1.1
             Host: localhost:8000
@@ -1280,7 +1313,7 @@ class JobsSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 165
@@ -1301,7 +1334,7 @@ class JobsSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
             curl -i localhost:8000/jobs/20121130104633606931
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /jobs/20121130104633606931 HTTP/1.1
             Host: localhost:8000
@@ -1309,7 +1342,7 @@ class JobsSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 73
@@ -1390,7 +1423,7 @@ class RunSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
                 -d password='saltdev' \\
                 -d eauth='pam'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /run HTTP/1.1
             Host: localhost:8000
@@ -1402,7 +1435,7 @@ class RunSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 73
@@ -1449,14 +1482,14 @@ class EventsSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
             curl -NsS localhost:8000/events
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /events HTTP/1.1
             Host: localhost:8000
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Connection: keep-alive
@@ -1604,7 +1637,7 @@ class WebhookSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
             curl -sS localhost:8000/hook -d foo='Foo!' -d bar='Bar!'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /hook HTTP/1.1
             Host: localhost:8000
@@ -1615,7 +1648,7 @@ class WebhookSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         **Example response**:
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 14
@@ -1628,7 +1661,9 @@ class WebhookSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
         ``http://localhost:8000/hook/mycompany/build/success`` which contains
         the result of a build and the SHA of the version that was built as
         JSON. That would then produce the following event in Salt that could be
-        used to kick off a deployment via Salt's Reactor::
+        used to kick off a deployment via Salt's Reactor:
+
+        .. code-block:: text
 
             Event fired at Fri Feb 14 17:40:11 2014
             *************************
@@ -1653,7 +1688,7 @@ class WebhookSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
         And finally deploy the new build:
 
-        .. code-block:: yaml
+        .. code-block:: jinja
 
             {% set secret_key = data.get('headers', {}).get('X-My-Secret-Key') %}
             {% set build = data.get('post', {}) %}
@@ -1708,9 +1743,9 @@ class WebhookSaltAPIHandler(SaltAPIHandler):  # pylint: disable=W0223
 
 
 def _check_cors_origin(origin, allowed_origins):
-    """
+    '''
     Check if an origin match cors allowed origins
-    """
+    '''
     if isinstance(allowed_origins, list):
         if origin in allowed_origins:
             return origin
