@@ -1,15 +1,12 @@
 # -*- coding: utf-8 -*-
 '''
-Execute a command and read the output as YAMLEX. The YAMLEX data is then
-directly overlaid onto the minion's Pillar data
+Execute a command and read the output as YAMLEX.
+
+The YAMLEX data is then directly overlaid onto the minion's Pillar data
 '''
 
-# Don't "fix" the above docstring to put it on two lines, as the sphinx
-# autosummary pulls only the first line for its description.
-
-from __future__ import absolute_import
-
 # Import python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 
 # Import salt libs
@@ -29,7 +26,5 @@ def ext_pillar(minion_id,  # pylint: disable=W0613
         command = command.replace('%s', minion_id)
         return deserialize(__salt__['cmd.run']('{0}'.format(command)))
     except Exception:
-        log.critical(
-                'YAML data from {0} failed to parse'.format(command)
-                )
+        log.critical('YAML data from %s failed to parse', command)
         return {}
