@@ -12,7 +12,7 @@ This renderer requires `Dogeon`__ (installable via pip)
 .. __: https://github.com/soasme/dogeon
 '''
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import logging
