@@ -10,27 +10,46 @@ import logging
 import salt.loader
 import salt.utils.event
 import salt.utils.functools
+import salt.utils.jid
 from salt.exceptions import SaltInvocationError
 
 LOGGER = logging.getLogger(__name__)
 
 
-def set_pause(jid, state_id, duration=None):
+def pause(jid, state_id=None, duration=None):
     '''
     Set up a state id pause, this instructs a running state to pause at a given
     state id. This needs to pass in the jid of the running state and can
     optionally pass in a duration in seconds.
     '''
     minion = salt.minion.MasterMinion(__opts__)
-    minion['state.set_pause'](jid, state_id, duration)
+    minion.functions['state.pause'](jid, state_id, duration)
 
 
-def rm_pause(jid, state_id, duration=None):
+set_pause = salt.utils.functools.alias_function(pause, 'set_pause')
+
+
+def resume(jid, state_id=None):
     '''
     Remove a pause from a jid, allowing it to continue
     '''
     minion = salt.minion.MasterMinion(__opts__)
-    minion['state.rm_pause'](jid, state_id)
+    minion.functions['state.resume'](jid, state_id)
+
+
+rm_pause = salt.utils.functools.alias_function(resume, 'rm_pause')
+
+
+def soft_kill(jid, state_id=None):
+    '''
+    Set up a state run to die before executing the given state id,
+    this instructs a running state to safely exit at a given
+    state id. This needs to pass in the jid of the running state.
+    If a state_id is not passed then the jid referenced will be safely exited
+    at the beginning of the next state run.
+    '''
+    minion = salt.minion.MasterMinion(__opts__)
+    minion.functions['state.soft_kill'](jid, state_id)
 
 
 def orchestrate(mods,
@@ -94,6 +113,8 @@ def orchestrate(mods,
         pillarenv = __opts__['pillarenv']
     if saltenv is None and 'saltenv' in __opts__:
         saltenv = __opts__['saltenv']
+    if orchestration_jid is None:
+        orchestration_jid = salt.utils.jid.gen_jid(__opts__)
 
     running = minion.functions['state.sls'](
             mods,
@@ -113,6 +134,7 @@ def orchestrate(mods,
         ret['retcode'] = 1
     return ret
 
+
 # Aliases for orchestrate runner
 orch = salt.utils.functools.alias_function(orchestrate, 'orch')
 sls = salt.utils.functools.alias_function(orchestrate, 'sls')
@@ -222,19 +244,22 @@ def orchestrate_show_sls(mods,
     ret = {minion.opts['id']: running}
     return ret
 
+
 orch_show_sls = salt.utils.functools.alias_function(orchestrate_show_sls, 'orch_show_sls')
 
 
 def event(tagmatch='*',
-        count=-1,
-        quiet=False,
-        sock_dir=None,
-        pretty=False,
-        node='master'):
+          count=-1,
+          quiet=False,
+          sock_dir=None,
+          pretty=False,
+          node='master'):
     r'''
     Watch Salt's event bus and block until the given tag is matched
 
     .. versionadded:: 2014.7.0
+    .. versionchanged:: 2019.2.0
+        ``tagmatch`` can now be either a glob or regular expression.
 
     This is useful for utilizing Salt's event bus from shell scripts or for
     taking simple actions directly from the CLI.
@@ -242,7 +267,7 @@ def event(tagmatch='*',
     Enable debug logging to see ignored events.
 
     :param tagmatch: the event is written to stdout for each tag that matches
-        this pattern; uses the same matching semantics as Salt's Reactor.
+        this glob or regular expression.
     :param count: this number is decremented for each event that matches the
         ``tagmatch`` parameter; pass ``-1`` to listen forever.
     :param quiet: do not print to stdout; just block
@@ -280,9 +305,9 @@ def event(tagmatch='*',
     statemod = salt.loader.raw_mod(__opts__, 'state', None)
 
     return statemod['state.event'](
-            tagmatch=tagmatch,
-            count=count,
-            quiet=quiet,
-            sock_dir=sock_dir,
-            pretty=pretty,
-            node=node)
+               tagmatch=tagmatch,
+               count=count,
+               quiet=quiet,
+               sock_dir=sock_dir,
+               pretty=pretty,
+               node=node)
