@@ -384,11 +384,15 @@ def running(name,
     **NETWORK MANAGEMENT**
 
     .. versionadded:: 2018.3.0
+    .. versionchanged:: 2019.2.0
+        If the ``networks`` option is used, any networks (including the default
+        ``bridge`` network) which are not specified will be disconnected.
 
     The ``networks`` argument can be used to ensure that a container is
     attached to one or more networks. Optionally, arguments can be passed to
     the networks. In the example below, ``net1`` is being configured with
-    arguments, while ``net2`` is being configured *without* arguments:
+    arguments, while ``net2`` and ``bridge`` are being configured *without*
+    arguments:
 
     .. code-block:: yaml
 
@@ -402,6 +406,7 @@ def running(name,
                   - baz
                 - ipv4_address: 10.0.20.50
               - net2
+              - bridge
             - require:
               - docker_network: net1
               - docker_network: net2
@@ -418,6 +423,17 @@ def running(name,
 
     .. _`connect_container_to_network`: https://docker-py.readthedocs.io/en/stable/api.html#docker.api.network.NetworkApiMixin.connect_container_to_network
 
+    To start a container with no network connectivity (only possible in
+    2019.2.0 and later) pass this option as an empty list. For example:
+
+    .. code-block:: yaml
+
+        foo:
+          docker_container.running:
+            - image: myuser/myimage:foo
+            - networks: []
+
+
     **CONTAINER CONFIGURATION PARAMETERS**
 
     auto_remove (or *rm*) : False
@@ -1017,7 +1033,7 @@ def running(name,
     labels
         Add metadata to the container. Labels can be set both with and without
         values, and labels with values can be passed either as ``key=value`` or
-        ``key: value `` pairs. For example, while the below would be very
+        ``key: value`` pairs. For example, while the below would be very
         confusing to read, it is technically valid, and demonstrates the
         different ways in which labels can be passed:
 
@@ -1356,7 +1372,7 @@ def running(name,
 
             foo:
               docker_container.running:
-                - image: bar/baz:lates
+                - image: bar/baz:latest
                 - privileged: True
 
     publish_all_ports (or *publish_all*) : False
@@ -1598,7 +1614,7 @@ def running(name,
                 - image: bar/baz:latest
                 - userns_mode: host
 
-    volumes (or *volume)
+    volumes (or *volume*)
         List of directories to expose as volumes. Can be expressed as a
         comma-separated list or a YAML list. The below two examples are
         equivalent:
@@ -1672,6 +1688,9 @@ def running(name,
         image = six.text_type(image)
 
     try:
+        # Since we're rewriting the "networks" value below, save the original
+        # value here.
+        configured_networks = networks
         networks = _parse_networks(networks)
         image_id = _resolve_image(ret, image, client_timeout)
     except CommandExecutionError as exc:
@@ -1802,9 +1821,32 @@ def running(name,
                 ret['result'] = False
                 comments.append(exc.__str__())
                 return _format_comments(ret, comments)
+
         post_net_connect = __salt__['docker.inspect_container'](
             temp_container_name)
 
+        if configured_networks is not None:
+            # Use set arithmetic to determine the networks which are connected
+            # but not explicitly defined. They will be disconnected below. Note
+            # that we check configured_networks because it represents the
+            # original (unparsed) network configuration. When no networks
+            # argument is used, the parsed networks will be an empty list, so
+            # it's not sufficient to do a boolean check on the "networks"
+            # variable.
+            extra_nets = set(
+                post_net_connect.get('NetworkSettings', {}).get('Networks', {})
+            ) - set(networks)
+
+            if extra_nets:
+                for extra_net in extra_nets:
+                    __salt__['docker.disconnect_container_from_network'](
+                        temp_container_name,
+                        extra_net)
+
+                # We've made changes, so we need to inspect the container again
+                post_net_connect = __salt__['docker.inspect_container'](
+                    temp_container_name)
+
         net_changes = __salt__['docker.compare_container_networks'](
             pre_net_connect, post_net_connect)
 
@@ -1989,6 +2031,7 @@ def running(name,
         if __opts__['test']:
             ret['result'] = None
             comments.append('Container would be started')
+            return _format_comments(ret, comments)
         else:
             try:
                 post_state = __salt__['docker.start'](name)['state']['new']
@@ -2137,11 +2180,12 @@ def run(name,
     CLI Examples:
 
     .. code-block:: bash
+
         salt myminion docker.run_container myuser/myimage command=/usr/local/bin/myscript.sh
 
     **USAGE EXAMPLE**
 
-    .. code-block:: yaml
+    .. code-block:: jinja
 
         {% set pkg_version = salt.pillar.get('pkg_version', '1.0-1') %}
         build_package:
@@ -2183,7 +2227,7 @@ def run(name,
         return ret
 
     try:
-        if 'networks' in kwargs:
+        if 'networks' in kwargs and kwargs['networks'] is not None:
             kwargs['networks'] = _parse_networks(kwargs['networks'])
         _resolve_image(ret, image, client_timeout)
     except CommandExecutionError as exc:
@@ -2570,6 +2614,15 @@ def mod_run_check(onlyif, unless, creates):
 
 
 def mod_watch(name, sfun=None, **kwargs):
+    '''
+    The docker_container watcher, called to invoke the watch command.
+
+    .. note::
+        This state exists to support special handling of the ``watch``
+        :ref:`requisite <requisites>`. It should not be called directly.
+
+        Parameters for this function should be set by the state being triggered.
+    '''
     if sfun == 'running':
         watch_kwargs = copy.deepcopy(kwargs)
         if watch_kwargs.get('watch_action', 'force') == 'force':
