@@ -275,8 +275,15 @@ def index_template_present(name, definition, check_definition=False):
                     ret['comment'] = 'Cannot create index template {0}, {1}'.format(name, output)
         else:
             if check_definition:
-                definition_parsed = salt.utils.json.loads(definition)
+                if isinstance(definition, str):
+                    definition_parsed = salt.utils.json.loads(definition)
+                else:
+                    definition_parsed = definition
                 current_template = __salt__['elasticsearch.index_template_get'](name=name)[name]
+                # Prune empty keys (avoid false positive diff)
+                for key in ("mappings", "aliases", "settings"):
+                    if current_template[key] == {}:
+                        del current_template[key]
                 diff = __utils__['dictdiffer.deep_diff'](current_template, definition_parsed)
                 if len(diff) != 0:
                     if __opts__['test']:
@@ -349,7 +356,7 @@ def pipeline_present(name, definition):
     .. code-block:: yaml
 
         test_pipeline:
-          elasticsearch_pipeline.present:
+          elasticsearch.pipeline_present:
             - definition:
                 description: example pipeline
                 processors:
