@@ -24,6 +24,11 @@ booleans can be set.
     Use of these states require that the :mod:`selinux <salt.modules.selinux>`
     execution module is available.
 '''
+# Import Python libs
+from __future__ import absolute_import, unicode_literals, print_function
+
+# Import 3rd party libs
+from salt.ext import six
 
 
 def __virtual__():
@@ -37,7 +42,7 @@ def _refine_mode(mode):
     '''
     Return a mode value that is predictable
     '''
-    mode = str(mode).lower()
+    mode = six.text_type(mode).lower()
     if any([mode.startswith('e'),
             mode == '1',
             mode == 'on']):
@@ -55,7 +60,7 @@ def _refine_value(value):
     '''
     Return a yes/no value, or None if the input is invalid
     '''
-    value = str(value).lower()
+    value = six.text_type(value).lower()
     if value in ('1', 'on', 'yes', 'true'):
         return 'on'
     if value in ('0', 'off', 'no', 'false'):
@@ -68,7 +73,7 @@ def _refine_module_state(module_state):
     Return a predictable value, or allow us to error out
     .. versionadded:: 2016.3.0
     '''
-    module_state = str(module_state).lower()
+    module_state = six.text_type(module_state).lower()
     if module_state in ('1', 'on', 'yes', 'true', 'enabled'):
         return 'enabled'
     if module_state in ('0', 'off', 'no', 'false', 'disabled'):
@@ -80,8 +85,10 @@ def mode(name):
     '''
     Verifies the mode SELinux is running in, can be set to enforcing,
     permissive, or disabled
-        Note: A change to or from disabled mode requires a system reboot.
-            You will need to perform this yourself.
+
+    .. note::
+        A change to or from disabled mode requires a system reboot. You will
+        need to perform this yourself.
 
     name
         The mode to run SELinux in, permissive, enforcing, or disabled.
@@ -310,17 +317,27 @@ def module_remove(name):
 
 def fcontext_policy_present(name, sel_type, filetype='a', sel_user=None, sel_level=None):
     '''
-    Makes sure a SELinux policy for a given filespec (name),
-    filetype and SELinux context type is present.
+    .. versionadded:: 2017.7.0
 
-    name: filespec of the file or directory. Regex syntax is allowed.
-    sel_type: SELinux context type. There are many.
-    filetype: The SELinux filetype specification.
-              Use one of [a, f, d, c, b, s, l, p].
-              See also `man semanage-fcontext`.
-              Defaults to 'a' (all files)
-    sel_user: The SELinux user.
-    sel_level: The SELinux MLS range
+    Makes sure a SELinux policy for a given filespec (name), filetype
+    and SELinux context type is present.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    sel_type
+        SELinux context type. There are many.
+
+    filetype
+        The SELinux filetype specification. Use one of [a, f, d, c, b,
+        s, l, p]. See also `man semanage-fcontext`. Defaults to 'a'
+        (all files).
+
+    sel_user
+        The SELinux user.
+
+    sel_level
+        The SELinux MLS range.
     '''
     ret = {'name': name, 'result': False, 'changes': {}, 'comment': ''}
     new_state = {}
@@ -337,8 +354,7 @@ def fcontext_policy_present(name, sel_type, filetype='a', sel_user=None, sel_lev
         if __opts__['test']:
             ret.update({'result': None})
         else:
-            add_ret = __salt__['selinux.fcontext_add_or_delete_policy'](
-                    action='add',
+            add_ret = __salt__['selinux.fcontext_add_policy'](
                     name=name,
                     filetype=filetype,
                     sel_type=sel_type,
@@ -365,8 +381,7 @@ def fcontext_policy_present(name, sel_type, filetype='a', sel_user=None, sel_lev
         if __opts__['test']:
             ret.update({'result': None})
         else:
-            change_ret = __salt__['selinux.fcontext_add_or_delete_policy'](
-                    action='add',
+            change_ret = __salt__['selinux.fcontext_add_policy'](
                     name=name,
                     filetype=filetype,
                     sel_type=sel_type,
@@ -383,17 +398,27 @@ def fcontext_policy_present(name, sel_type, filetype='a', sel_user=None, sel_lev
 
 def fcontext_policy_absent(name, filetype='a', sel_type=None, sel_user=None, sel_level=None):
     '''
-    Makes sure an SELinux file context policy for a given filespec (name),
-    filetype and SELinux context type is absent.
+    .. versionadded:: 2017.7.0
 
-    name: filespec of the file or directory. Regex syntax is allowed.
-    filetype: The SELinux filetype specification.
-              Use one of [a, f, d, c, b, s, l, p].
-              See also `man semanage-fcontext`.
-              Defaults to 'a' (all files).
-    sel_type: The SELinux context type. There are many.
-    sel_user: The SELinux user.
-    sel_level: The SELinux MLS range
+    Makes sure an SELinux file context policy for a given filespec
+    (name), filetype and SELinux context type is absent.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    filetype
+        The SELinux filetype specification. Use one of [a, f, d, c, b,
+        s, l, p]. See also `man semanage-fcontext`. Defaults to 'a'
+        (all files).
+
+    sel_type
+        The SELinux context type. There are many.
+
+    sel_user
+        The SELinux user.
+
+    sel_level
+        The SELinux MLS range.
     '''
     ret = {'name': name, 'result': False, 'changes': {}, 'comment': ''}
     new_state = {}
@@ -417,8 +442,7 @@ def fcontext_policy_absent(name, filetype='a', sel_type=None, sel_user=None, sel
     if __opts__['test']:
         ret.update({'result': None})
     else:
-        remove_ret = __salt__['selinux.fcontext_add_or_delete_policy'](
-                action='delete',
+        remove_ret = __salt__['selinux.fcontext_delete_policy'](
                 name=name,
                 filetype=filetype,
                 sel_type=sel_type or current_state['sel_type'],
@@ -433,7 +457,10 @@ def fcontext_policy_absent(name, filetype='a', sel_type=None, sel_user=None, sel
 
 def fcontext_policy_applied(name, recursive=False):
     '''
-    Checks and makes sure the SELinux policies for a given filespec are applied.
+    .. versionadded:: 2017.7.0
+
+    Checks and makes sure the SELinux policies for a given filespec are
+    applied.
     '''
     ret = {'name': name, 'result': False, 'changes': {}, 'comment': ''}
 
@@ -452,3 +479,109 @@ def fcontext_policy_applied(name, recursive=False):
             ret.update({'result': True})
             ret.update({'changes': apply_ret.get('changes')})
     return ret
+
+
+def port_policy_present(name, sel_type, protocol=None, port=None, sel_range=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Makes sure an SELinux port policy for a given port, protocol and SELinux context type is present.
+
+    name
+        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.
+
+    sel_type
+        The SELinux Type.
+
+    protocol
+        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.
+
+    port
+        The port or port range. Required if name is not formatted.
+
+    sel_range
+        The SELinux MLS/MCS Security Range.
+    '''
+    ret = {'name': name, 'result': False, 'changes': {}, 'comment': ''}
+    old_state = __salt__['selinux.port_get_policy'](
+        name=name,
+        sel_type=sel_type,
+        protocol=protocol,
+        port=port, )
+    if old_state:
+        ret.update({'result': True,
+                    'comment': 'SELinux policy for "{0}" already present '.format(name) +
+                               'with specified sel_type "{0}", protocol "{1}" and port "{2}".'.format(
+                                   sel_type, protocol, port)})
+        return ret
+    if __opts__['test']:
+        ret.update({'result': None})
+    else:
+        add_ret = __salt__['selinux.port_add_policy'](
+            name=name,
+            sel_type=sel_type,
+            protocol=protocol,
+            port=port,
+            sel_range=sel_range, )
+        if add_ret['retcode'] != 0:
+            ret.update({'comment': 'Error adding new policy: {0}'.format(add_ret)})
+        else:
+            ret.update({'result': True})
+            new_state = __salt__['selinux.port_get_policy'](
+                name=name,
+                sel_type=sel_type,
+                protocol=protocol,
+                port=port, )
+            ret['changes'].update({'old': old_state, 'new': new_state})
+    return ret
+
+
+def port_policy_absent(name, sel_type=None, protocol=None, port=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Makes sure an SELinux port policy for a given port, protocol and SELinux context type is absent.
+
+    name
+        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.
+
+    sel_type
+        The SELinux Type. Optional; can be used in determining if policy is present,
+        ignored by ``semanage port --delete``.
+
+    protocol
+        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.
+
+    port
+        The port or port range. Required if name is not formatted.
+    '''
+    ret = {'name': name, 'result': False, 'changes': {}, 'comment': ''}
+    old_state = __salt__['selinux.port_get_policy'](
+        name=name,
+        sel_type=sel_type,
+        protocol=protocol,
+        port=port, )
+    if not old_state:
+        ret.update({'result': True,
+                    'comment': 'SELinux policy for "{0}" already absent '.format(name) +
+                               'with specified sel_type "{0}", protocol "{1}" and port "{2}".'.format(
+                                   sel_type, protocol, port)})
+        return ret
+    if __opts__['test']:
+        ret.update({'result': None})
+    else:
+        delete_ret = __salt__['selinux.port_delete_policy'](
+            name=name,
+            protocol=protocol,
+            port=port, )
+        if delete_ret['retcode'] != 0:
+            ret.update({'comment': 'Error deleting policy: {0}'.format(delete_ret)})
+        else:
+            ret.update({'result': True})
+            new_state = __salt__['selinux.port_get_policy'](
+                name=name,
+                sel_type=sel_type,
+                protocol=protocol,
+                port=port, )
+            ret['changes'].update({'old': old_state, 'new': new_state})
+    return ret
