@@ -25,14 +25,14 @@ Management of SmartOS Standalone Compute Nodes
               label: 'test vm'
               owner: 'sjorge'
             nics:
-              "82:1b:8e:49:e9:12"
+              "82:1b:8e:49:e9:12":
                 nic_tag: trunk
                 mtu: 1500
                 ips:
                   - 172.16.1.123/16
                   - 192.168.2.123/24
                 vlan_id: 10
-              "82:1b:8e:49:e9:13"
+              "82:1b:8e:49:e9:13":
                 nic_tag: trunk
                 mtu: 1500
                 ips:
@@ -64,13 +64,43 @@ Management of SmartOS Standalone Compute Nodes
                 compression: lz4
                 boot: true
             nics:
-              "82:1b:8e:49:e9:15"
+              "82:1b:8e:49:e9:15":
                 nic_tag: trunk
                 mtu: 1500
                 ips:
                   - dhcp
                 vlan_id: 30
 
+    docker.example.org:
+      smartos.vm_present:
+        - config:
+            auto_import: true
+            reprovision: true
+        - vmconfig:
+            image_uuid: emby/embyserver:latest
+            brand: lx
+            alias: mydockervm
+            quota: 5
+            max_physical_memory: 1024
+            tags:
+              label: 'my emby docker'
+              owner: 'sjorge'
+            resolvers:
+              - 172.16.1.1
+            nics:
+              "82:1b:8e:49:e9:18":
+                nic_tag: trunk
+                mtu: 1500
+                ips:
+                  - 172.16.1.118/24
+                vlan_id: 10
+            filesystems:
+              "/config:
+                source: "/vmdata/emby_config"
+                type: lofs
+                options:
+                  - nodevices
+
     cleanup_images:
       smartos.image_vacuum
 
@@ -86,6 +116,7 @@ from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Python libs
 import logging
+import json
 import os
 
 # Import Salt libs
@@ -111,12 +142,47 @@ def __virtual__():
     else:
         return (
             False,
-            '{0} state module can only be loaded on SmartOS computed nodes'.format(
+            '{0} state module can only be loaded on SmartOS compute nodes'.format(
                 __virtualname__
             )
         )
 
 
+def _split_docker_uuid(uuid):
+    '''
+    Split a smartos docker uuid into repo and tag
+    '''
+    if uuid:
+        uuid = uuid.split(':')
+        if len(uuid) == 2:
+            tag = uuid[1]
+            repo = uuid[0]
+            if len(repo.split('/')) == 2:
+                return repo, tag
+    return None, None
+
+
+def _is_uuid(uuid):
+    '''
+    Check if uuid is a valid smartos uuid
+
+    Example: e69a0918-055d-11e5-8912-e3ceb6df4cf8
+    '''
+    if uuid and list((len(x) for x in uuid.split('-'))) == [8, 4, 4, 4, 12]:
+        return True
+    return False
+
+
+def _is_docker_uuid(uuid):
+    '''
+    Check if uuid is a valid smartos docker uuid
+
+    Example plexinc/pms-docker:plexpass
+    '''
+    repo, tag = _split_docker_uuid(uuid)
+    return not (not repo and not tag)
+
+
 def _load_config():
     '''
     Loads and parses /usbkey/config
@@ -207,6 +273,60 @@ def _get_instance_changes(current, state):
     return changed
 
 
+def _copy_lx_vars(vmconfig):
+    # NOTE: documentation on dockerinit: https://github.com/joyent/smartos-live/blob/master/src/dockerinit/README.md
+    if 'image_uuid' in vmconfig:
+        # NOTE: retrieve tags and type from image
+        imgconfig = __salt__['imgadm.get'](vmconfig['image_uuid']).get('manifest', {})
+        imgtype = imgconfig.get('type', 'zone-dataset')
+        imgtags = imgconfig.get('tags', {})
+
+        # NOTE: copy kernel_version (if not specified in vmconfig)
+        if 'kernel_version' not in vmconfig and 'kernel_version' in imgtags:
+            vmconfig['kernel_version'] = imgtags['kernel_version']
+
+        # NOTE: copy docker vars
+        if imgtype == 'docker':
+            vmconfig['docker'] = True
+            vmconfig['kernel_version'] = vmconfig.get('kernel_version', '4.3.0')
+            if 'internal_metadata' not in vmconfig:
+                vmconfig['internal_metadata'] = {}
+
+            for var in imgtags.get('docker:config', {}):
+                val = imgtags['docker:config'][var]
+                var = 'docker:{0}'.format(var.lower())
+
+                # NOTE: skip empty values
+                if not val:
+                    continue
+
+                # NOTE: skip or merge user values
+                if var == 'docker:env':
+                    try:
+                        val_config = json.loads(
+                            vmconfig['internal_metadata'].get(var, "")
+                        )
+                    except ValueError as e:
+                        val_config = []
+
+                    for config_env_var in val_config if isinstance(val_config, list) else json.loads(val_config):
+                        config_env_var = config_env_var.split('=')
+                        for img_env_var in val:
+                            if img_env_var.startswith('{0}='.format(config_env_var[0])):
+                                val.remove(img_env_var)
+                        val.append('='.join(config_env_var))
+                elif var in vmconfig['internal_metadata']:
+                    continue
+
+                if isinstance(val, list):
+                    # NOTE: string-encoded JSON arrays
+                    vmconfig['internal_metadata'][var] = json.dumps(val)
+                else:
+                    vmconfig['internal_metadata'][var] = val
+
+    return vmconfig
+
+
 def config_present(name, value):
     '''
     Ensure configuration property is set to value in /usbkey/config
@@ -251,7 +371,7 @@ def config_present(name, value):
         config[name] = value
 
     # apply change if needed
-    if not __opts__['test'] and len(ret['changes']) > 0:
+    if not __opts__['test'] and ret['changes']:
         ret['result'] = _write_config(config)
 
     return ret
@@ -286,12 +406,86 @@ def config_absent(name):
         ret['comment'] = 'property {0} is absent'.format(name)
 
     # apply change if needed
-    if not __opts__['test'] and len(ret['changes']) > 0:
+    if not __opts__['test'] and ret['changes']:
         ret['result'] = _write_config(config)
 
     return ret
 
 
+def source_present(name, source_type='imgapi'):
+    '''
+    Ensure an image source is present on the computenode
+
+    name : string
+        source url
+    source_type : string
+        source type (imgapi or docker)
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': None,
+           'comment': ''}
+
+    if name in __salt__['imgadm.sources']():
+        # source is present
+        ret['result'] = True
+        ret['comment'] = 'image source {0} is present'.format(name)
+    else:
+        # add new source
+        if __opts__['test']:
+            res = {}
+            ret['result'] = True
+        else:
+            res = __salt__['imgadm.source_add'](name, source_type)
+            ret['result'] = (name in res)
+
+        if ret['result']:
+            ret['comment'] = 'image source {0} added'.format(name)
+            ret['changes'][name] = 'added'
+        else:
+            ret['comment'] = 'image source {0} not added'.format(name)
+            if 'Error' in res:
+                ret['comment'] = '{0}: {1}'.format(ret['comment'], res['Error'])
+
+    return ret
+
+
+def source_absent(name):
+    '''
+    Ensure an image source is absent on the computenode
+
+    name : string
+        source url
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': None,
+           'comment': ''}
+
+    if name not in __salt__['imgadm.sources']():
+        # source is absent
+        ret['result'] = True
+        ret['comment'] = 'image source {0} is absent'.format(name)
+    else:
+        # remove source
+        if __opts__['test']:
+            res = {}
+            ret['result'] = True
+        else:
+            res = __salt__['imgadm.source_delete'](name)
+            ret['result'] = (name not in res)
+
+        if ret['result']:
+            ret['comment'] = 'image source {0} deleted'.format(name)
+            ret['changes'][name] = 'deleted'
+        else:
+            ret['comment'] = 'image source {0} not deleted'.format(name)
+            if 'Error' in res:
+                ret['comment'] = '{0}: {1}'.format(ret['comment'], res['Error'])
+
+    return ret
+
+
 def image_present(name):
     '''
     Ensure image is present on the computenode
@@ -304,21 +498,44 @@ def image_present(name):
            'result': None,
            'comment': ''}
 
-    if name in __salt__['imgadm.list']():
-        # we're good
+    if _is_docker_uuid(name) and __salt__['imgadm.docker_to_uuid'](name):
+        # docker image was imported
+        ret['result'] = True
+        ret['comment'] = 'image {0} ({1}) is present'.format(
+            name,
+            __salt__['imgadm.docker_to_uuid'](name),
+        )
+    elif name in __salt__['imgadm.list']():
+        # image was already imported
         ret['result'] = True
         ret['comment'] = 'image {0} is present'.format(name)
     else:
         # add image
-        available_images = __salt__['imgadm.avail']()
+        if _is_docker_uuid(name):
+            # NOTE: we cannot query available docker images
+            available_images = [name]
+        else:
+            available_images = __salt__['imgadm.avail']()
+
         if name in available_images:
             if __opts__['test']:
                 ret['result'] = True
+                res = {}
+                if _is_docker_uuid(name):
+                    res['00000000-0000-0000-0000-000000000000'] = name
+                else:
+                    res[name] = available_images[name]
             else:
-                __salt__['imgadm.import'](name)
-                ret['result'] = (name in __salt__['imgadm.list']())
-            ret['comment'] = 'image {0} installed'.format(name)
-            ret['changes'][name] = available_images[name]
+                res = __salt__['imgadm.import'](name)
+                if _is_uuid(name):
+                    ret['result'] = (name in res)
+                elif _is_docker_uuid(name):
+                    ret['result'] = __salt__['imgadm.docker_to_uuid'](name) is not None
+            if ret['result']:
+                ret['comment'] = 'image {0} imported'.format(name)
+                ret['changes'] = res
+            else:
+                ret['comment'] = 'image {0} was unable to be imported'.format(name)
         else:
             ret['result'] = False
             ret['comment'] = 'image {0} does not exists'.format(name)
@@ -343,13 +560,19 @@ def image_absent(name):
            'result': None,
            'comment': ''}
 
-    if name not in __salt__['imgadm.list']():
-        # we're good
+    uuid = None
+    if _is_uuid(name):
+        uuid = name
+    if _is_docker_uuid(name):
+        uuid = __salt__['imgadm.docker_to_uuid'](name)
+
+    if not uuid or uuid not in __salt__['imgadm.list']():
+        # image not imported
         ret['result'] = True
         ret['comment'] = 'image {0} is absent'.format(name)
     else:
         # check if image in use by vm
-        if name in __salt__['vmadm.list'](order='image_uuid'):
+        if uuid in __salt__['vmadm.list'](order='image_uuid'):
             ret['result'] = False
             ret['comment'] = 'image {0} currently in use by a vm'.format(name)
         else:
@@ -357,9 +580,26 @@ def image_absent(name):
             if __opts__['test']:
                 ret['result'] = True
             else:
-                __salt__['imgadm.delete'](name)
-                ret['result'] = name not in __salt__['imgadm.list']()
-            ret['comment'] = 'image {0} deleted'.format(name)
+                image = __salt__['imgadm.get'](uuid)
+                image_count = 0
+                if image['manifest']['name'] == 'docker-layer':
+                    # NOTE: docker images are made of multiple layers, loop over them
+                    while image:
+                        image_count += 1
+                        __salt__['imgadm.delete'](image['manifest']['uuid'])
+                        if 'origin' in image['manifest']:
+                            image = __salt__['imgadm.get'](image['manifest']['origin'])
+                        else:
+                            image = None
+                else:
+                    # NOTE: normal images can just be delete
+                    __salt__['imgadm.delete'](uuid)
+
+            ret['result'] = uuid not in __salt__['imgadm.list']()
+            if image_count:
+                ret['comment'] = 'image {0} and {1} children deleted'.format(name, image_count)
+            else:
+                ret['comment'] = 'image {0} deleted'.format(name)
             ret['changes'][name] = None
 
     return ret
@@ -368,6 +608,11 @@ def image_absent(name):
 def image_vacuum(name):
     '''
     Delete images not in use or installed via image_present
+
+    .. warning::
+
+        Only image_present states that are included via the
+        top file will be detected.
     '''
     name = name.lower()
     ret = {'name': name,
@@ -378,7 +623,7 @@ def image_vacuum(name):
     # list of images to keep
     images = []
 
-    # retreive image_present state data for host
+    # retrieve image_present state data for host
     for state in __salt__['state.show_lowstate']():
         # don't throw exceptions when not highstate run
         if 'state' not in state:
@@ -392,28 +637,50 @@ def image_vacuum(name):
             continue
         # keep images installed via image_present
         if 'name' in state:
-            images.append(state['name'])
+            if _is_uuid(state['name']):
+                images.append(state['name'])
+            elif _is_docker_uuid(state['name']):
+                state['name'] = __salt__['imgadm.docker_to_uuid'](state['name'])
+                if not state['name']:
+                    continue
+                images.append(state['name'])
 
     # retrieve images in use by vms
     for image_uuid in __salt__['vmadm.list'](order='image_uuid'):
-        if image_uuid in images:
-            continue
-        images.append(image_uuid)
+        if image_uuid not in images:
+            images.append(image_uuid)
 
     # purge unused images
     ret['result'] = True
     for image_uuid in __salt__['imgadm.list']():
         if image_uuid in images:
             continue
-        if image_uuid in __salt__['imgadm.delete'](image_uuid):
-            ret['changes'][image_uuid] = None
-        else:
-            ret['result'] = False
-            ret['comment'] = 'failed to delete images'
 
-    if ret['result'] and len(ret['changes']) == 0:
+        image = __salt__['imgadm.get'](image_uuid)
+        if image['manifest']['name'] == 'docker-layer':
+            # NOTE: docker images are made of multiple layers, loop over them
+            while image:
+                image_uuid = image['manifest']['uuid']
+                if image_uuid in __salt__['imgadm.delete'](image_uuid):
+                    ret['changes'][image_uuid] = None
+                else:
+                    ret['result'] = False
+                    ret['comment'] = 'failed to delete images'
+                if 'origin' in image['manifest']:
+                    image = __salt__['imgadm.get'](image['manifest']['origin'])
+                else:
+                    image = None
+        else:
+            # NOTE: normal images can just be delete
+            if image_uuid in __salt__['imgadm.delete'](image_uuid):
+                ret['changes'][image_uuid] = None
+            else:
+                ret['result'] = False
+                ret['comment'] = 'failed to delete images'
+
+    if ret['result'] and not ret['changes']:
         ret['comment'] = 'no images deleted'
-    elif ret['result'] and len(ret['changes']) > 0:
+    elif ret['result'] and ret['changes']:
         ret['comment'] = 'images deleted'
 
     return ret
@@ -433,11 +700,14 @@ def vm_present(name, vmconfig, config=None):
     .. note::
 
         The following configuration properties can be toggled in the config parameter.
-          - kvm_reboot (true) - reboots of kvm zones if needed for a config update
-          - auto_import (false) - automatic importing of missing images
-          - reprovision (false) - reprovision on image_uuid changes
+          - kvm_reboot (true)                - reboots of kvm zones if needed for a config update
+          - auto_import (false)              - automatic importing of missing images
+          - auto_lx_vars (true)              - copy kernel_version and docker:* variables from image
+          - reprovision (false)              - reprovision on image_uuid changes
+          - enforce_tags (true)              - false = add tags only, true =  add, update, and remove tags
+          - enforce_routes (true)            - false = add tags only, true =  add, update, and remove routes
+          - enforce_internal_metadata (true) - false = add metadata only, true =  add, update, and remove metadata
           - enforce_customer_metadata (true) - false = add metadata only, true =  add, update, and remove metadata
-          - enforce_tags (true) - false = add tags only, true =  add, update, and remove tags
 
     .. note::
 
@@ -457,6 +727,10 @@ def vm_present(name, vmconfig, config=None):
 
         e.g. disk0 will be the first disk added, disk1 the 2nd,...
 
+    .. versionchanged:: 2019.2.0
+
+        Added support for docker image uuids, added auto_lx_vars configuration, documented some missing configuration options.
+
     '''
     name = name.lower()
     ret = {'name': name,
@@ -469,7 +743,12 @@ def vm_present(name, vmconfig, config=None):
     config = {
         'kvm_reboot': True,
         'auto_import': False,
+        'auto_lx_vars': True,
         'reprovision': False,
+        'enforce_tags': True,
+        'enforce_routes': True,
+        'enforce_internal_metadata': True,
+        'enforce_customer_metadata': True,
     }
     config.update(state_config)
     log.debug('smartos.vm_present::%s::config - %s', name, config)
@@ -493,6 +772,15 @@ def vm_present(name, vmconfig, config=None):
             'filesystems'
         ]
     }
+    vmconfig_docker_keep = [
+        'docker:id',
+        'docker:restartcount',
+    ]
+    vmconfig_docker_array = [
+        'docker:env',
+        'docker:cmd',
+        'docker:entrypoint',
+    ]
 
     # parse vmconfig
     vmconfig = _parse_vmconfig(vmconfig, vmconfig_type['instance'])
@@ -502,6 +790,46 @@ def vm_present(name, vmconfig, config=None):
     if 'hostname' not in vmconfig:
         vmconfig['hostname'] = name
 
+    # prepare image_uuid
+    if 'image_uuid' in vmconfig:
+        # NOTE: lookup uuid from docker uuid (normal uuid's are passed throuhg unmodified)
+        #       we must do this again if we end up importing a missing image later!
+        docker_uuid = __salt__['imgadm.docker_to_uuid'](vmconfig['image_uuid'])
+        vmconfig['image_uuid'] = docker_uuid if docker_uuid else vmconfig['image_uuid']
+
+        # NOTE: import image (if missing and allowed)
+        if vmconfig['image_uuid'] not in __salt__['imgadm.list']():
+            if config['auto_import']:
+                if not __opts__['test']:
+                    res = __salt__['imgadm.import'](vmconfig['image_uuid'])
+                    vmconfig['image_uuid'] = __salt__['imgadm.docker_to_uuid'](vmconfig['image_uuid'])
+                    if vmconfig['image_uuid'] not in res:
+                        ret['result'] = False
+                        ret['comment'] = 'failed to import image {0}'.format(vmconfig['image_uuid'])
+            else:
+                ret['result'] = False
+                ret['comment'] = 'image {0} not installed'.format(vmconfig['image_uuid'])
+
+    # docker json-array handling
+    if 'internal_metadata' in vmconfig:
+        for var in vmconfig_docker_array:
+            if var not in vmconfig['internal_metadata']:
+                continue
+            if isinstance(vmconfig['internal_metadata'][var], list):
+                vmconfig['internal_metadata'][var] = json.dumps(
+                    vmconfig['internal_metadata'][var]
+                )
+
+    # copy lx variables
+    if vmconfig['brand'] == 'lx' and config['auto_lx_vars']:
+        # NOTE: we can only copy the lx vars after the image has bene imported
+        vmconfig = _copy_lx_vars(vmconfig)
+
+    # quick abort if things look wrong
+    # NOTE: use explicit check for false, otherwise None also matches!
+    if ret['result'] is False:
+        return ret
+
     # check if vm exists
     if vmconfig['hostname'] in __salt__['vmadm.list'](order='hostname'):
         # update vm
@@ -564,11 +892,20 @@ def vm_present(name, vmconfig, config=None):
                 continue
 
             # enforcement
-            enforce = True
-            if 'enforce_{0}'.format(collection) in config:
-                enforce = config['enforce_{0}'.format(collection)]
+            enforce = config['enforce_{0}'.format(collection)]
             log.debug('smartos.vm_present::enforce_%s = %s', collection, enforce)
 
+            # dockerinit handling
+            if collection == 'internal_metadata' and vmconfig['state'].get('docker', False):
+                if 'internal_metadata' not in vmconfig['state']:
+                    vmconfig['state']['internal_metadata'] = {}
+
+                # preserve some docker specific metadata (added and needed by dockerinit)
+                for var in vmconfig_docker_keep:
+                    val = vmconfig['current'].get(collection, {}).get(var, None)
+                    if val is not None:
+                        vmconfig['state']['internal_metadata'][var] = val
+
             # process add and update for collection
             if collection in vmconfig['state'] and vmconfig['state'][collection] is not None:
                 for prop in vmconfig['state'][collection]:
@@ -627,21 +964,20 @@ def vm_present(name, vmconfig, config=None):
                             update_cfg = {}
 
                             # handle changes
-                            if len(changed) > 0:
-                                for prop in changed:
-                                    update_cfg[prop] = state_cfg[prop]
+                            for prop in changed:
+                                update_cfg[prop] = state_cfg[prop]
 
                             # handle new properties
                             for prop in state_cfg:
                                 # skip empty props like ips, options,..
-                                if isinstance(state_cfg[prop], (list)) and len(state_cfg[prop]) == 0:
+                                if isinstance(state_cfg[prop], (list)) and not state_cfg[prop]:
                                     continue
 
                                 if prop not in current_cfg:
                                     update_cfg[prop] = state_cfg[prop]
 
                             # update instance
-                            if len(update_cfg) > 0:
+                            if update_cfg:
                                 # create update_ array
                                 if 'update_{0}'.format(instance) not in vmconfig['changed']:
                                     vmconfig['changed']['update_{0}'.format(instance)] = []
@@ -684,7 +1020,7 @@ def vm_present(name, vmconfig, config=None):
 
         # update vm if we have pending changes
         kvm_needs_start = False
-        if not __opts__['test'] and len(vmconfig['changed']) > 0:
+        if not __opts__['test'] and vmconfig['changed']:
             # stop kvm if disk updates and kvm_reboot
             if vmconfig['current']['brand'] == 'kvm' and config['kvm_reboot']:
                 if 'add_disks' in vmconfig['changed'] or \
@@ -707,7 +1043,7 @@ def vm_present(name, vmconfig, config=None):
             if __opts__['test']:
                 ret['changes'][vmconfig['state']['hostname']] = vmconfig['changed']
 
-            if vmconfig['state']['hostname'] in ret['changes'] and len(ret['changes'][vmconfig['state']['hostname']]) > 0:
+            if vmconfig['state']['hostname'] in ret['changes'] and ret['changes'][vmconfig['state']['hostname']]:
                 ret['comment'] = 'vm {0} updated'.format(vmconfig['state']['hostname'])
                 if config['kvm_reboot'] and vmconfig['current']['brand'] == 'kvm' and not __opts__['test']:
                     if vmconfig['state']['hostname'] in __salt__['vmadm.list'](order='hostname', search='state=running'):
@@ -718,18 +1054,9 @@ def vm_present(name, vmconfig, config=None):
                 ret['changes'] = {}
                 ret['comment'] = 'vm {0} is up to date'.format(vmconfig['state']['hostname'])
 
+            # reprovision (if required and allowed)
             if 'image_uuid' in vmconfig['current'] and vmconfig['reprovision_uuid'] != vmconfig['current']['image_uuid']:
                 if config['reprovision']:
-                    # check required image installed
-                    if vmconfig['reprovision_uuid'] not in __salt__['imgadm.list']():
-                        if config['auto_import']:
-                            # check if image is available
-                            available_images = __salt__['imgadm.avail']()
-                            if vmconfig['reprovision_uuid'] in available_images and not __opts__['test']:
-                                # import image
-                                __salt__['imgadm.import'](vmconfig['reprovision_uuid'])
-
-                    # reprovision
                     rret = __salt__['vmadm.reprovision'](
                         vm=vmconfig['state']['hostname'],
                         key='hostname',
@@ -737,15 +1064,9 @@ def vm_present(name, vmconfig, config=None):
                     )
                     if not isinstance(rret, (bool)) and 'Error' in rret:
                         ret['result'] = False
-                        if vmconfig['reprovision_uuid'] not in __salt__['imgadm.list']():
-                            ret['comment'] = 'vm {0} updated, reprovision failed because images {1} not installed'.format(
-                                vmconfig['state']['hostname'],
-                                vmconfig['reprovision_uuid']
-                            )
-                        else:
-                            ret['comment'] = 'vm {0} updated, reprovision failed'.format(
-                                vmconfig['state']['hostname']
-                            )
+                        ret['comment'] = 'vm {0} updated, reprovision failed'.format(
+                            vmconfig['state']['hostname']
+                        )
                     else:
                         ret['comment'] = 'vm {0} updated and reprovisioned'.format(vmconfig['state']['hostname'])
                         if vmconfig['state']['hostname'] not in ret['changes']:
@@ -763,20 +1084,6 @@ def vm_present(name, vmconfig, config=None):
     else:
         # check required image installed
         ret['result'] = True
-        if 'image_uuid' in vmconfig and vmconfig['image_uuid'] not in __salt__['imgadm.list']():
-            if config['auto_import']:
-                # check if image is available
-                available_images = __salt__['imgadm.avail']()
-                if vmconfig['image_uuid'] not in available_images:
-                    ret['result'] = False
-                    ret['comment'] = 'image {0} not available'.format(vmconfig['image_uuid'])
-                elif not __opts__['test']:
-                    if vmconfig['image_uuid'] not in __salt__['imgadm.import'](vmconfig['image_uuid']):
-                        ret['result'] = False
-                        ret['comment'] = 'failed to import image {0}'.format(vmconfig['image_uuid'])
-            else:
-                ret['result'] = False
-                ret['comment'] = 'image {0} not installed'.format(vmconfig['image_uuid'])
 
         # disks need some special care
         if 'disks' in vmconfig:
