@@ -247,7 +247,7 @@ def create(name,
     api_url
         Custom API URL in case the user has a StatusPage service running in a custom environment.
 
-    **kwargs
+    kwargs
         Other params.
 
     SLS Example:
