@@ -481,7 +481,7 @@ def container_setting(name, container, settings=None):
     :param str container: The type of IIS container. The container types are:
         AppPools, Sites, SslBindings
     :param str settings: A dictionary of the setting names and their values.
-    Example of usage for the ``AppPools`` container:
+        Example of usage for the ``AppPools`` container:
 
     .. code-block:: yaml
 
@@ -779,19 +779,26 @@ def remove_vdir(name, site, app='/'):
 def set_app(name, site, settings=None):
     # pylint: disable=anomalous-backslash-in-string
     '''
+    .. versionadded:: 2017.7.0
+
     Set the value of the setting for an IIS web application.
+
     .. note::
-        This function only configures existing app.
-        Params are case sensitive.
+        This function only configures existing app. Params are case sensitive.
+
     :param str name: The IIS application.
     :param str site: The IIS site name.
     :param str settings: A dictionary of the setting names and their values.
-    :available settings:    physicalPath: The physical path of the webapp.
-    :                       applicationPool: The application pool for the webapp.
-    :                       userName: "connectAs" user
-    :                       password: "connectAs" password for user
+
+    Available settings:
+
+    - ``physicalPath`` - The physical path of the webapp
+    - ``applicationPool`` - The application pool for the webapp
+    - ``userName`` "connectAs" user
+    - ``password`` "connectAs" password for user
+
     :rtype: bool
-    .. versionadded:: 2017.7.0
+
     Example of usage:
 
     .. code-block:: yaml
