@@ -5,10 +5,12 @@ IPC transport classes
 
 # Import Python libs
 from __future__ import absolute_import, print_function, unicode_literals
+import errno
 import logging
 import socket
 import weakref
 import time
+import threading
 
 # Import 3rd-party libs
 import msgpack
@@ -130,7 +132,7 @@ class IPCServer(object):
         else:
             self.sock = tornado.netutil.bind_unix_socket(self.socket_path)
 
-        with salt.utils.async.current_ioloop(self.io_loop):
+        with salt.utils.asynchronous.current_ioloop(self.io_loop):
             tornado.netutil.add_accept_handler(
                 self.sock,
                 self.handle_connection,
@@ -196,7 +198,7 @@ class IPCServer(object):
         log.trace('IPCServer: Handling connection '
                   'to address: %s', address)
         try:
-            with salt.utils.async.current_ioloop(self.io_loop):
+            with salt.utils.asynchronous.current_ioloop(self.io_loop):
                 stream = IOStream(
                     connection,
                 )
@@ -217,7 +219,12 @@ class IPCServer(object):
             self.sock.close()
 
     def __del__(self):
-        self.close()
+        try:
+            self.close()
+        except TypeError:
+            # This is raised when Python's GC has collected objects which
+            # would be needed when calling self.close()
+            pass
 
 
 class IPCClient(object):
@@ -255,9 +262,14 @@ class IPCClient(object):
             client = object.__new__(cls)
             # FIXME
             client.__singleton_init__(io_loop=io_loop, socket_path=socket_path)
+            client._instance_key = key
             loop_instance_map[key] = client
+            client._refcount = 1
+            client._refcount_lock = threading.RLock()
         else:
             log.debug('Re-using IPCClient for %s', key)
+            with client._refcount_lock:
+                client._refcount += 1
         return client
 
     def __singleton_init__(self, socket_path, io_loop=None):
@@ -329,7 +341,7 @@ class IPCClient(object):
                 break
 
             if self.stream is None:
-                with salt.utils.async.current_ioloop(self.io_loop):
+                with salt.utils.asynchronous.current_ioloop(self.io_loop):
                     self.stream = IOStream(
                         socket.socket(sock_type, socket.SOCK_STREAM),
                     )
@@ -353,7 +365,21 @@ class IPCClient(object):
                 yield tornado.gen.sleep(1)
 
     def __del__(self):
-        self.close()
+        try:
+            with self._refcount_lock:
+                # Make sure we actually close no matter if something
+                # went wrong with our ref counting
+                self._refcount = 1
+            try:
+                self.close()
+            except socket.error as exc:
+                if exc.errno != errno.EBADF:
+                    # If its not a bad file descriptor error, raise
+                    raise
+        except TypeError:
+            # This is raised when Python's GC has collected objects which
+            # would be needed when calling self.close()
+            pass
 
     def close(self):
         '''
@@ -363,7 +389,21 @@ class IPCClient(object):
         '''
         if self._closing:
             return
+
+        if self._refcount > 1:
+            # Decrease refcount
+            with self._refcount_lock:
+                self._refcount -= 1
+            log.debug(
+                'This is not the last %s instance. Not closing yet.',
+                self.__class__.__name__
+            )
+            return
+
         self._closing = True
+
+        log.debug('Closing %s instance', self.__class__.__name__)
+
         if self.stream is not None and not self.stream.closed():
             self.stream.close()
 
@@ -371,11 +411,12 @@ class IPCClient(object):
         # that a closed entry may not be reused.
         # This forces this operation even if the reference
         # count of the entry has not yet gone to zero.
-        if self.io_loop in IPCClient.instance_map:
-            loop_instance_map = IPCClient.instance_map[self.io_loop]
-            key = six.text_type(self.socket_path)
-            if key in loop_instance_map:
-                del loop_instance_map[key]
+        if self.io_loop in self.__class__.instance_map:
+            loop_instance_map = self.__class__.instance_map[self.io_loop]
+            if self._instance_key in loop_instance_map:
+                del loop_instance_map[self._instance_key]
+            if not loop_instance_map:
+                del self.__class__.instance_map[self.io_loop]
 
 
 class IPCMessageClient(IPCClient):
@@ -510,7 +551,7 @@ class IPCMessagePublisher(object):
         else:
             self.sock = tornado.netutil.bind_unix_socket(self.socket_path)
 
-        with salt.utils.async.current_ioloop(self.io_loop):
+        with salt.utils.asynchronous.current_ioloop(self.io_loop):
             tornado.netutil.add_accept_handler(
                 self.sock,
                 self.handle_connection,
@@ -549,7 +590,7 @@ class IPCMessagePublisher(object):
             if self.opts['ipc_write_buffer'] > 0:
                 kwargs['max_write_buffer_size'] = self.opts['ipc_write_buffer']
                 log.trace('Setting IPC connection write buffer: %s', (self.opts['ipc_write_buffer']))
-            with salt.utils.async.current_ioloop(self.io_loop):
+            with salt.utils.asynchronous.current_ioloop(self.io_loop):
                 stream = IOStream(
                     connection,
                     **kwargs
@@ -579,7 +620,12 @@ class IPCMessagePublisher(object):
             self.sock.close()
 
     def __del__(self):
-        self.close()
+        try:
+            self.close()
+        except TypeError:
+            # This is raised when Python's GC has collected objects which
+            # would be needed when calling self.close()
+            pass
 
 
 class IPCMessageSubscriber(IPCClient):
@@ -748,14 +794,11 @@ class IPCMessageSubscriber(IPCClient):
         '''
         if not self._closing:
             IPCClient.close(self)
-            # This will prevent this message from showing up:
-            # '[ERROR   ] Future exception was never retrieved:
-            # StreamClosedError'
-            if self._read_sync_future is not None:
-                self._read_sync_future.exception()
-            if self._read_stream_future is not None:
-                self._read_stream_future.exception()
-
-    def __del__(self):
-        if IPCMessageSubscriber in globals():
-            self.close()
+            if self._closing:
+                # This will prevent this message from showing up:
+                # '[ERROR   ] Future exception was never retrieved:
+                # StreamClosedError'
+                if self._read_sync_future is not None and self._read_sync_future.done():
+                    self._read_sync_future.exception()
+                if self._read_stream_future is not None and self._read_stream_future.done():
+                    self._read_stream_future.exception()
