@@ -19,6 +19,7 @@ import hashlib
 import itertools
 import logging
 import random
+import re
 import shlex
 import socket
 import ssl
@@ -35,8 +36,7 @@ from salt.utils.odict import OrderedDict
 
 # Import 3rd-party libs
 from salt.ext import six
-from salt.ext.six.moves import map, zip  # pylint: disable=redefined-builtin
-
+from salt.ext.six.moves import zip  # pylint: disable=redefined-builtin
 
 # Integrations
 try:
@@ -44,6 +44,11 @@ try:
     HAS_DNSPYTHON = True
 except ImportError:
     HAS_DNSPYTHON = False
+try:
+    import tldextract
+    HAS_TLDEXTRACT = True
+except ImportError:
+    HAS_TLDEXTRACT = False
 HAS_DIG = salt.utils.path.which('dig') is not None
 HAS_DRILL = salt.utils.path.which('drill') is not None
 HAS_HOST = salt.utils.path.which('host') is not None
@@ -60,7 +65,7 @@ class RFC(object):
     Simple holding class for all RFC/IANA registered lists & standards
     '''
     # https://tools.ietf.org/html/rfc6844#section-3
-    COO_TAGS = (
+    CAA_TAGS = (
         'issue',
         'issuewild',
         'iodef'
@@ -88,10 +93,8 @@ class RFC(object):
     ))
 
     TLSA_SELECT = OrderedDict((
-        (0, 'pkixta'),
-        (1, 'pkixee'),
-        (2, 'daneta'),
-        (3, 'daneee'),
+        (0, 'cert'),
+        (1, 'spki'),
     ))
 
     TLSA_MATCHING = OrderedDict((
@@ -129,12 +132,23 @@ def _to_port(port):
 def _tree(domain, tld=False):
     '''
     Split out a domain in its parents
+
+    Leverages tldextract to take the TLDs from publicsuffix.org
+    or makes a valiant approximation of that
+
     :param domain: dc2.ams2.example.com
     :param tld: Include TLD in list
     :return: [ 'dc2.ams2.example.com', 'ams2.example.com', 'example.com']
     '''
-    if '.' not in domain:
-        raise ValueError('Provide a decent domain')
+    domain = domain.rstrip('.')
+    assert '.' in domain, 'Provide a decent domain'
+
+    if not tld:
+        if HAS_TLDEXTRACT:
+            tld = tldextract.extract(domain).suffix
+        else:
+            tld = re.search(r'((?:(?:ac|biz|com?|info|edu|gov|mil|name|net|n[oi]m|org)\.)?[^.]+)$', domain).group()
+            log.info('Without tldextract, dns.util resolves the TLD of {0} to {1}'.format(domain, tld))
 
     res = [domain]
     while True:
@@ -142,12 +156,10 @@ def _tree(domain, tld=False):
         if idx < 0:
             break
         domain = domain[idx + 1:]
+        if domain == tld:
+            break
         res.append(domain)
 
-    # properly validating the tld is impractical
-    if not tld:
-        res = res[:-1]
-
     return res
 
 
@@ -166,6 +178,17 @@ def _weighted_order(recs):
     return res
 
 
+def _cast(rec_data, rec_cast):
+    if isinstance(rec_cast, dict):
+        rec_data = type(rec_cast.keys()[0])(rec_data)
+        res = rec_cast[rec_data]
+        return res
+    elif isinstance(rec_cast, (list, tuple)):
+        return RFC.validate(rec_data, rec_cast)
+    else:
+        return rec_cast(rec_data)
+
+
 def _data2rec(schema, rec_data):
     '''
     schema = OrderedDict({
@@ -180,11 +203,20 @@ def _data2rec(schema, rec_data):
     '''
     try:
         rec_fields = rec_data.split(' ')
-        assert len(rec_fields) == len(schema)
-        return dict((
-            (field_name, rec_cast(rec_field))
-            for (field_name, rec_cast), rec_field in zip(schema.items(), rec_fields)
-        ))
+        # spaces in digest fields are allowed
+        assert len(rec_fields) >= len(schema)
+        if len(rec_fields) > len(schema):
+            cutoff = len(schema) - 1
+            rec_fields = rec_fields[0:cutoff] + [''.join(rec_fields[cutoff:])]
+
+        if len(schema) == 1:
+            res = _cast(rec_fields[0], next(iter(schema.values())))
+        else:
+            res = dict((
+                (field_name, _cast(rec_field, rec_cast))
+                for (field_name, rec_cast), rec_field in zip(schema.items(), rec_fields)
+            ))
+        return res
     except (AssertionError, AttributeError, TypeError, ValueError) as e:
         raise ValueError('Unable to cast "{0}" as "{2}": {1}'.format(
             rec_data,
@@ -203,9 +235,14 @@ def _data2rec_group(schema, recs_data, group_key):
         for rdata in recs_data:
             rdata = _data2rec(schema, rdata)
             assert rdata and group_key in rdata
+
             idx = rdata.pop(group_key)
             if idx not in res:
                 res[idx] = []
+
+            if len(rdata) == 1:
+                rdata = next(iter(rdata.values()))
+
             res[idx].append(rdata)
         return res
     except (AssertionError, ValueError) as e:
@@ -220,6 +257,14 @@ def _rec2data(*rdata):
     return ' '.join(rdata)
 
 
+def _data_clean(data):
+    data = data.strip(string.whitespace)
+    if data.startswith(('"', '\'')) and data.endswith(('"', '\'')):
+        return data[1:-1]
+    else:
+        return data
+
+
 def _lookup_dig(name, rdtype, timeout=None, servers=None, secure=None):
     '''
     Use dig to lookup addresses
@@ -229,7 +274,7 @@ def _lookup_dig(name, rdtype, timeout=None, servers=None, secure=None):
     :param servers: [] of servers to use
     :return: [] of records or False if error
     '''
-    cmd = 'dig +search +fail +noall +answer +noclass +nottl -t {0} '.format(rdtype)
+    cmd = 'dig +search +fail +noall +answer +noclass +nosplit +nottl -t {0} '.format(rdtype)
     if servers:
         cmd += ''.join(['@{0} '.format(srv) for srv in servers])
     if timeout is not None:
@@ -241,7 +286,7 @@ def _lookup_dig(name, rdtype, timeout=None, servers=None, secure=None):
     if secure:
         cmd += '+dnssec +adflag '
 
-    cmd = __salt__['cmd.run_all'](cmd + six.text_type(name), python_shell=False, output_loglevel='quiet')
+    cmd = __salt__['cmd.run_all']('{0} {1}'.format(cmd, name), python_shell=False, output_loglevel='quiet')
 
     if 'ignoring invalid type' in cmd['stderr']:
         raise ValueError('Invalid DNS type {}'.format(rdtype))
@@ -263,7 +308,7 @@ def _lookup_dig(name, rdtype, timeout=None, servers=None, secure=None):
         elif rtype == 'RRSIG':
             validated = True
             continue
-        res.append(rdata.strip(string.whitespace + '"'))
+        res.append(_data_clean(rdata))
 
     if res and secure and not validated:
         return False
@@ -316,7 +361,7 @@ def _lookup_drill(name, rdtype, timeout=None, servers=None, secure=None):
             elif l_type != rdtype:
                 raise ValueError('Invalid DNS type {}'.format(rdtype))
 
-            res.append(l_rec.strip(string.whitespace + '"'))
+            res.append(_data_clean(l_rec))
 
     except StopIteration:
         pass
@@ -344,7 +389,7 @@ def _lookup_gai(name, rdtype, timeout=None):
         raise ValueError('Invalid DNS type {} for gai lookup'.format(rdtype))
 
     if timeout:
-        log.warn('Ignoring timeout on gai resolver; fix resolv.conf to do that')
+        log.info('Ignoring timeout on gai resolver; fix resolv.conf to do that')
 
     try:
         addresses = [sock[4][0] for sock in socket.getaddrinfo(name, None, sock_t, 0, socket.SOCK_RAW)]
@@ -369,7 +414,7 @@ def _lookup_host(name, rdtype, timeout=None, server=None):
     if timeout:
         cmd += '-W {0} '.format(int(timeout))
 
-    cmd = __salt__['cmd.run_all'](cmd + name, python_shell=False, output_loglevel='quiet')
+    cmd = __salt__['cmd.run_all']('{0} {1}'.format(cmd, name), python_shell=False, output_loglevel='quiet')
 
     if 'invalid type' in cmd['stderr']:
         raise ValueError('Invalid DNS type {}'.format(rdtype))
@@ -388,7 +433,7 @@ def _lookup_host(name, rdtype, timeout=None, server=None):
             if line.startswith(prefix):
                 line = line[len(prefix) + 1:]
                 break
-        res.append(line.strip(string.whitespace + '"'))
+        res.append(_data_clean(line))
 
     return res
 
@@ -412,7 +457,7 @@ def _lookup_dnspython(name, rdtype, timeout=None, servers=None, secure=None):
         resolver.ednsflags += dns.flags.DO
 
     try:
-        res = [six.text_type(rr.to_text().strip(string.whitespace + '"'))
+        res = [_data_clean(rr.to_text())
                for rr in resolver.query(name, rdtype, raise_on_no_answer=False)]
         return res
     except dns.rdatatype.UnknownRdatatype:
@@ -481,7 +526,7 @@ def _lookup_nslookup(name, rdtype, timeout=None, server=None):
                 else:
                     line = line.split(' ')
 
-            res.append(line[-1].strip(string.whitespace + '"'))
+            res.append(_data_clean(line[-1]))
             line = next(lookup_res)
 
     except StopIteration:
@@ -504,13 +549,14 @@ def lookup(
     secure=None
 ):
     '''
-    Lookup DNS record data
+    Lookup DNS records and return their data
+
     :param name: name to lookup
     :param rdtype: DNS record type
     :param method: gai (getaddrinfo()), dnspython, dig, drill, host, nslookup or auto (default)
     :param servers: (list of) server(s) to try in-order
     :param timeout: query timeout or a valiant approximation of that
-    :param walk: Find records in parents if they don't exist
+    :param walk: Walk the DNS upwards looking for the record type or name/recordtype if walk='name'.
     :param walk_tld: Include the final domain in the walk
     :param secure: return only DNSSEC secured responses
     :return: [] of record data
@@ -574,13 +620,16 @@ def lookup(
         name = [name]
     else:
         idx = 0
-        if rdtype == 'SRV':  # The only rr I know that has 2 name components
+        if rdtype in ('SRV', 'TLSA'):  # The only RRs I know that have 2 name components
             idx = name.find('.') + 1
         idx = name.find('.', idx) + 1
         domain = name[idx:]
-        name = name[0:idx]
+        rname = name[0:idx]
+
+        name = _tree(domain, walk_tld)
+        if walk == 'name':
+            name = [rname + domain for domain in name]
 
-        name = [name + domain for domain in _tree(domain, walk_tld)]
         if timeout:
             timeout /= len(name)
 
@@ -594,6 +643,8 @@ def lookup(
         if res:
             return res
 
+    return res
+
 
 def query(
     name,
@@ -606,14 +657,16 @@ def query(
     secure=None
 ):
     '''
-    Query DNS for information
+    Query DNS for information.
+    Where `lookup()` returns record data, `query()` tries to interpret the data and return it's results
+
     :param name: name to lookup
     :param rdtype: DNS record type
     :param method: gai (getaddrinfo()), pydns, dig, drill, host, nslookup or auto (default)
     :param servers: (list of) server(s) to try in-order
     :param timeout: query timeout or a valiant approximation of that
     :param secure: return only DNSSEC secured response
-    :param walk: Find records in parents if they don't exist
+    :param walk: Walk the DNS upwards looking for the record type or name/recordtype if walk='name'.
     :param walk_tld: Include the top-level domain in the walk
     :return: [] of records
     '''
@@ -630,32 +683,58 @@ def query(
     if rdtype == 'PTR' and not name.endswith('arpa'):
         name = ptr_name(name)
 
-    qres = lookup(name, rdtype, **qargs)
-    if rdtype == 'SPF' and not qres:
+    if rdtype == 'SPF':
         # 'SPF' has become a regular 'TXT' again
         qres = [answer for answer in lookup(name, 'TXT', **qargs) if answer.startswith('v=spf')]
+        if not qres:
+            qres = lookup(name, rdtype, **qargs)
+    else:
+        qres = lookup(name, rdtype, **qargs)
 
     rec_map = {
-        'A':    a_rec,
-        'AAAA': aaaa_rec,
-        'CAA':  caa_rec,
-        'MX':   mx_rec,
-        'SOA':  soa_rec,
-        'SPF':  spf_rec,
-        'SRV':  srv_rec,
+        'A':     a_rec,
+        'AAAA':  aaaa_rec,
+        'CAA':   caa_rec,
+        'MX':    mx_rec,
+        'SOA':   soa_rec,
+        'SPF':   spf_rec,
+        'SRV':   srv_rec,
+        'SSHFP': sshfp_rec,
+        'TLSA':  tlsa_rec,
     }
 
-    if rdtype not in rec_map:
+    if not qres or rdtype not in rec_map:
         return qres
-
-    caster = rec_map[rdtype]
-
-    if rdtype in ('MX', 'SRV'):
-        # Grouped returns
-        res = caster(qres)
+    elif rdtype in ('A', 'AAAA', 'SSHFP', 'TLSA'):
+        res = [rec_map[rdtype](res) for res in qres]
+    elif rdtype in ('SOA', 'SPF'):
+        res = rec_map[rdtype](qres[0])
     else:
-        # List of results
-        res = list(map(caster, qres))
+        res = rec_map[rdtype](qres)
+
+    return res
+
+
+def host(name, ip4=True, ip6=True, **kwargs):
+    '''
+    Return a list of addresses for name
+
+    ip6:
+        Return IPv6 addresses
+    ip4:
+        Return IPv4 addresses
+
+    the rest is passed on to lookup()
+    '''
+    res = {}
+    if ip6:
+        ip6 = lookup(name, 'AAAA', **kwargs)
+        if ip6:
+            res['ip6'] = ip6
+    if ip4:
+        ip4 = lookup(name, 'A', **kwargs)
+        if ip4:
+            res['ip4'] = ip4
 
     return res
 
@@ -692,8 +771,8 @@ def caa_rec(rdatas):
     '''
     rschema = OrderedDict((
         ('flags', lambda flag: ['critical'] if int(flag) > 0 else []),
-        ('tag', lambda tag: RFC.validate(tag, RFC.COO_TAGS)),
-        ('value', lambda val: six.text_type(val).strip('"'))
+        ('tag', RFC.CAA_TAGS),
+        ('value', lambda val: val.strip('\',"'))
     ))
 
     res = _data2rec_group(rschema, rdatas, 'tag')
@@ -845,7 +924,7 @@ def srv_name(svc, proto='tcp', domain=None):
     :return:
     '''
     proto = RFC.validate(proto, RFC.SRV_PROTO)
-    if svc.isdigit():
+    if isinstance(svc, int) or svc.isdigit():
         svc = _to_port(svc)
 
     if domain:
@@ -887,6 +966,21 @@ def sshfp_data(key_t, hash_t, pub):
     return _rec2data(key_t, hash_t, ssh_fp)
 
 
+def sshfp_rec(rdata):
+    '''
+    Validate and parse DNS record data for TLSA record(s)
+    :param rdata: DNS record data
+    :return: dict w/fields
+    '''
+    rschema = OrderedDict((
+        ('algorithm', RFC.SSHFP_ALGO),
+        ('fp_hash', RFC.SSHFP_HASH),
+        ('fingerprint', lambda val: val.lower())  # resolvers are inconsistent on this one
+    ))
+
+    return _data2rec(rschema, rdata)
+
+
 def tlsa_data(pub, usage, selector, matching):
     '''
     Generate a TLSA rec
@@ -913,6 +1007,22 @@ def tlsa_data(pub, usage, selector, matching):
     return _rec2data(usage, selector, matching, cert_fp)
 
 
+def tlsa_rec(rdata):
+    '''
+    Validate and parse DNS record data for TLSA record(s)
+    :param rdata: DNS record data
+    :return: dict w/fields
+    '''
+    rschema = OrderedDict((
+        ('usage', RFC.TLSA_USAGE),
+        ('selector', RFC.TLSA_SELECT),
+        ('matching', RFC.TLSA_MATCHING),
+        ('pub', str)
+    ))
+
+    return _data2rec(rschema, rdata)
+
+
 def service(
     svc,
     proto='tcp',
@@ -1029,18 +1139,13 @@ def parse_resolv(src='/etc/resolv.conf'):
                 try:
                     (directive, arg) = (line[0].lower(), line[1:])
                     # Drop everything after # or ; (comments)
-                    arg = list(itertools.takewhile(
-                        lambda x: x[0] not in ('#', ';'), arg))
-
+                    arg = list(itertools.takewhile(lambda x: x[0] not in ('#', ';'), arg))
                     if directive == 'nameserver':
-                        # Split the scope (interface) if it is present
-                        addr, scope = arg[0].split('%', 1) if '%' in arg[0] else (arg[0], '')
+                        addr = arg[0]
                         try:
                             ip_addr = ipaddress.ip_address(addr)
                             version = ip_addr.version
-                            # Rejoin scope after address validation
-                            if scope:
-                                ip_addr = '%'.join((str(ip_addr), scope))
+                            ip_addr = str(ip_addr)
                             if ip_addr not in nameservers:
                                 nameservers.append(ip_addr)
                             if version == 4 and ip_addr not in ip4_nameservers:
