@@ -61,7 +61,7 @@ import salt.ext.six.moves.http_client
 import salt.ext.six.moves.http_cookiejar
 import salt.ext.six.moves.urllib.request as urllib_request
 from salt.ext.six.moves.urllib.error import URLError
-from salt.ext.six.moves.urllib.parse import splitquery
+from salt.ext.six.moves.urllib.parse import splitquery, urlparse
 from salt.ext.six.moves.urllib.parse import urlencode as _urlencode
 # pylint: enable=import-error,no-name-in-module
 
@@ -488,13 +488,10 @@ def query(url,
             data = _urlencode(data)
 
         if verify_ssl:
-            # tornado requires a str, cannot be unicode str in py2
-            if ca_bundle is None:
-                req_kwargs['ca_certs'] = ca_bundle
-            else:
-                req_kwargs['ca_certs'] = salt.utils.stringutils.to_str(ca_bundle)
+            req_kwargs['ca_certs'] = ca_bundle
 
         max_body = opts.get('http_max_body', salt.config.DEFAULT_MINION_OPTS['http_max_body'])
+        connect_timeout = opts.get('http_connect_timeout', salt.config.DEFAULT_MINION_OPTS['http_connect_timeout'])
         timeout = opts.get('http_request_timeout', salt.config.DEFAULT_MINION_OPTS['http_request_timeout'])
 
         client_argspec = None
@@ -512,6 +509,13 @@ def query(url,
         if proxy_password:
             # tornado requires a str, cannot be unicode str in py2
             proxy_password = salt.utils.stringutils.to_str(proxy_password)
+        no_proxy = opts.get('no_proxy', [])
+
+        # Since tornado doesnt support no_proxy, we'll always hand it empty proxies or valid ones
+        # except we remove the valid ones if a url has a no_proxy hostname in it
+        if urlparse(url_full).hostname in no_proxy:
+            proxy_host = None
+            proxy_port = None
 
         # We want to use curl_http if we have a proxy defined
         if proxy_host and proxy_port:
@@ -530,30 +534,36 @@ def query(url,
 
         supports_max_body_size = 'max_body_size' in client_argspec.args
 
+        req_kwargs.update({
+            'method': method,
+            'headers': header_dict,
+            'auth_username': username,
+            'auth_password': password,
+            'body': data,
+            'validate_cert': verify_ssl,
+            'allow_nonstandard_methods': True,
+            'streaming_callback': streaming_callback,
+            'header_callback': header_callback,
+            'connect_timeout': connect_timeout,
+            'request_timeout': timeout,
+            'proxy_host': proxy_host,
+            'proxy_port': proxy_port,
+            'proxy_username': proxy_username,
+            'proxy_password': proxy_password,
+            'raise_error': raise_error,
+            'decompress_response': False,
+        })
+
+        # Unicode types will cause a TypeError when Tornado's curl HTTPClient
+        # invokes setopt. Therefore, make sure all arguments we pass which
+        # contain strings are str types.
+        req_kwargs = salt.utils.data.decode(req_kwargs, to_str=True)
+
         try:
             download_client = HTTPClient(max_body_size=max_body) \
                 if supports_max_body_size \
                 else HTTPClient()
-            result = download_client.fetch(
-                url_full,
-                method=method,
-                headers=header_dict,
-                auth_username=username,
-                auth_password=password,
-                body=data,
-                validate_cert=verify_ssl,
-                allow_nonstandard_methods=True,
-                streaming_callback=streaming_callback,
-                header_callback=header_callback,
-                request_timeout=timeout,
-                proxy_host=proxy_host,
-                proxy_port=proxy_port,
-                proxy_username=proxy_username,
-                proxy_password=proxy_password,
-                raise_error=raise_error,
-                decompress_response=False,
-                **req_kwargs
-            )
+            result = download_client.fetch(url_full, **req_kwargs)
         except tornado.httpclient.HTTPError as exc:
             ret['status'] = exc.code
             ret['error'] = six.text_type(exc)
@@ -831,7 +841,7 @@ def _render(template, render, renderer, template_dict, opts):
         if template_dict is None:
             template_dict = {}
         if not renderer:
-            renderer = opts.get('renderer', 'yaml_jinja')
+            renderer = opts.get('renderer', 'jinja|yaml')
         rend = salt.loader.render(opts, {})
         blacklist = opts.get('renderer_blacklist')
         whitelist = opts.get('renderer_whitelist')
@@ -915,6 +925,7 @@ def parse_cookie_header(header):
         for item in list(cookie):
             if item in attribs:
                 continue
+            name = item
             value = cookie.pop(item)
 
         # cookielib.Cookie() requires an epoch
