@@ -16,6 +16,11 @@ import time
 # Import salt libs
 import salt.utils.data
 from salt.utils.timeout import wait_for
+import salt.ext.six as six
+
+# Workaround for 'reload' builtin of py2.7
+if six.PY3:
+    from importlib import reload  # pylint: disable=no-name-in-module
 
 log = logging.getLogger(__name__)
 
@@ -135,7 +140,6 @@ def vb_get_manager():
     '''
     global _virtualboxManager
     if _virtualboxManager is None and HAS_LIBS:
-        # Reloading the API extends sys.paths for subprocesses of multiprocessing, since they seem to share contexts
         reload(vboxapi)
         _virtualboxManager = vboxapi.VirtualBoxManager(None, None)
 
@@ -149,7 +153,13 @@ def vb_get_box():
     @rtype: IVirtualBox
     '''
     vb_get_manager()
-    vbox = _virtualboxManager.vbox
+
+    try:
+        # This works in older versions of the SDK, but does not seem to work anymore.
+        vbox = _virtualboxManager.vbox
+    except AttributeError:
+        vbox = _virtualboxManager.getVirtualBox()
+
     return vbox
 
 
