@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
 
     salt.utils.vt
@@ -19,7 +19,7 @@
     .. __: https://github.com/pexpect/pexpect
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import os
@@ -52,7 +52,9 @@ except ImportError:
     import resource
 
 # Import salt libs
-import salt.utils
+import salt.utils.crypt
+import salt.utils.data
+import salt.utils.stringutils
 from salt.ext.six import string_types
 from salt.log.setup import LOG_LEVELS
 
@@ -212,7 +214,7 @@ class Terminal(object):
             # A lot can go wrong, so that's why we're catching the most general
             # exception type
             log.warning(
-                'Failed to spawn the VT: {0}'.format(err),
+                'Failed to spawn the VT: %s', err,
                  exc_info_on_loglevel=logging.DEBUG
             )
             raise TerminalException(
@@ -220,15 +222,15 @@ class Terminal(object):
             )
 
         log.debug(
-            'Child Forked! PID: {0}  STDOUT_FD: {1}  STDERR_FD: '
-            '{2}'.format(self.pid, self.child_fd, self.child_fde)
+            'Child Forked! PID: %s  STDOUT_FD: %s  STDERR_FD: %s',
+            self.pid, self.child_fd, self.child_fde
         )
         terminal_command = ' '.join(self.args)
         if 'decode("base64")' in terminal_command or 'base64.b64decode(' in terminal_command:
             log.debug('VT: Salt-SSH SHIM Terminal Command executed. Logged to TRACE')
-            log.trace('Terminal Command: {0}'.format(terminal_command))
+            log.trace('Terminal Command: %s', terminal_command)
         else:
-            log.debug('Terminal Command: {0}'.format(terminal_command))
+            log.debug('Terminal Command: %s', terminal_command)
         # <---- Spawn our terminal -------------------------------------------
 
         # ----- Setup Logging ----------------------------------------------->
@@ -440,10 +442,8 @@ class Terminal(object):
                         self.setwinsize(self.rows, self.cols)
                     except IOError as err:
                         log.warning(
-                            'Failed to set the VT terminal size: {0}'.format(
-                                err
-                            ),
-                            exc_info_on_loglevel=logging.DEBUG
+                            'Failed to set the VT terminal size: %s',
+                            err, exc_info_on_loglevel=logging.DEBUG
                         )
 
                 # Do not allow child to inherit open file descriptors from
@@ -492,7 +492,7 @@ class Terminal(object):
                 # Close parent FDs
                 os.close(stdout_parent_fd)
                 os.close(stderr_parent_fd)
-                salt.utils.reinit_crypto()
+                salt.utils.crypt.reinit_crypto()
 
                 # ----- Make STDOUT the controlling PTY --------------------->
                 child_name = os.ttyname(stdout_child_fd)
@@ -503,7 +503,7 @@ class Terminal(object):
                     if tty_fd >= 0:
                         os.close(tty_fd)
                 # which exception, shouldn't we catch explicitly .. ?
-                except:  # pylint: disable=W0702
+                except Exception:
                     # Already disconnected. This happens if running inside cron
                     pass
 
@@ -521,7 +521,7 @@ class Terminal(object):
                             'still possible to open /dev/tty.'
                         )
                 # which exception, shouldn't we catch explicitly .. ?
-                except:  # pylint: disable=W0702
+                except Exception:
                     # Good! We are disconnected from a controlling tty.
                     pass
 
@@ -553,7 +553,7 @@ class Terminal(object):
                 # <---- Duplicate Descriptors --------------------------------
             else:
                 # Parent. Close Child PTY's
-                salt.utils.reinit_crypto()
+                salt.utils.crypt.reinit_crypto()
                 os.close(stdout_child_fd)
                 os.close(stderr_child_fd)
 
@@ -643,7 +643,7 @@ class Terminal(object):
             if self.child_fde in rlist:
                 try:
                     stderr = self._translate_newlines(
-                        salt.utils.to_str(
+                        salt.utils.stringutils.to_unicode(
                             os.read(self.child_fde, maxsize)
                         )
                     )
@@ -676,7 +676,7 @@ class Terminal(object):
             if self.child_fd in rlist:
                 try:
                     stdout = self._translate_newlines(
-                        salt.utils.to_str(
+                        salt.utils.stringutils.to_unicode(
                             os.read(self.child_fd, maxsize)
                         )
                     )
@@ -686,7 +686,7 @@ class Terminal(object):
                         stdout = None
                     else:
                         if self.stream_stdout:
-                            self.stream_stdout.write(stdout)
+                            self.stream_stdout.write(salt.utils.stringutils.to_str(stdout))
                             self.stream_stdout.flush()
 
                         if self.stdout_logger:
@@ -709,9 +709,9 @@ class Terminal(object):
         def __detect_parent_terminal_size(self):
             try:
                 TIOCGWINSZ = getattr(termios, 'TIOCGWINSZ', 1074295912)
-                packed = struct.pack('HHHH', 0, 0, 0, 0)
+                packed = struct.pack(b'HHHH', 0, 0, 0, 0)
                 ioctl = fcntl.ioctl(sys.stdin.fileno(), TIOCGWINSZ, packed)
-                return struct.unpack('HHHH', ioctl)[0:2]
+                return struct.unpack(b'HHHH', ioctl)[0:2]
             except IOError:
                 # Return a default value of 24x80
                 return 24, 80
@@ -732,9 +732,9 @@ class Terminal(object):
                 )
 
             TIOCGWINSZ = getattr(termios, 'TIOCGWINSZ', 1074295912)
-            packed = struct.pack('HHHH', 0, 0, 0, 0)
+            packed = struct.pack(b'HHHH', 0, 0, 0, 0)
             ioctl = fcntl.ioctl(self.child_fd, TIOCGWINSZ, packed)
-            return struct.unpack('HHHH', ioctl)[0:2]
+            return struct.unpack(b'HHHH', ioctl)[0:2]
 
         def setwinsize(self, rows, cols):
             '''
@@ -760,7 +760,7 @@ class Terminal(object):
                 # Same bits, but with sign.
                 TIOCSWINSZ = -2146929561
             # Note, assume ws_xpixel and ws_ypixel are zero.
-            packed = struct.pack('HHHH', rows, cols, 0, 0)
+            packed = struct.pack(b'HHHH', rows, cols, 0, 0)
             fcntl.ioctl(self.child_fd, TIOCSWINSZ, packed)
 
         def isalive(self,
