@@ -4,16 +4,19 @@ Modules used to control the master itself
 '''
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import collections
 
 # Import salt libs
 import salt.client.mixins
 import salt.config
 import salt.loader
-import salt.transport
-import salt.utils
 import salt.utils.error
+import salt.utils.zeromq
+import salt.transport.client
+
+# Import 3rd-party libs
+from salt.ext import six
 
 
 class WheelClient(salt.client.mixins.SyncClientMixin,
@@ -43,7 +46,8 @@ class WheelClient(salt.client.mixins.SyncClientMixin,
 
     def __init__(self, opts=None):
         self.opts = opts
-        self.functions = salt.loader.wheels(opts)
+        self.context = {}
+        self.functions = salt.loader.wheels(opts, context=self.context)
 
     # TODO: remove/deprecate
     def call_func(self, fun, **kwargs):
@@ -53,7 +57,7 @@ class WheelClient(salt.client.mixins.SyncClientMixin,
         return self.low(fun, kwargs, print_event=kwargs.get('print_event', True), full_return=kwargs.get('full_return', False))
 
     # TODO: Inconsistent with runner client-- the runner client's master_call gives
-    # an async return, unlike this
+    # an asynchronous return, unlike this
     def master_call(self, **kwargs):
         '''
         Execute a wheel function through the master network interface (eauth).
@@ -63,13 +67,18 @@ class WheelClient(salt.client.mixins.SyncClientMixin,
         interface = self.opts['interface']
         if interface == '0.0.0.0':
             interface = '127.0.0.1'
-        master_uri = 'tcp://' + salt.utils.ip_bracket(interface) + \
-                                                      ':' + str(self.opts['ret_port'])
-        channel = salt.transport.Channel.factory(self.opts,
-                                                 crypt='clear',
-                                                 master_uri=master_uri,
-                                                 usage='master_call')
-        ret = channel.send(load)
+        master_uri = 'tcp://{}:{}'.format(
+            salt.utils.zeromq.ip_bracket(interface),
+            six.text_type(self.opts['ret_port'])
+        )
+        channel = salt.transport.client.ReqChannel.factory(self.opts,
+                                                           crypt='clear',
+                                                           master_uri=master_uri,
+                                                           usage='master_call')
+        try:
+            ret = channel.send(load)
+        finally:
+            channel.close()
         if isinstance(ret, collections.Mapping):
             if 'error' in ret:
                 salt.utils.error.raise_error(**ret['error'])
@@ -116,7 +125,7 @@ class WheelClient(salt.client.mixins.SyncClientMixin,
             {'jid': '20131219224744416681', 'tag': 'salt/wheel/20131219224744416681'}
         '''
         fun = low.pop('fun')
-        return self.async(fun, low)
+        return self.asynchronous(fun, low)
 
     def cmd(self, fun, arg=None, pub_data=None, kwarg=None, print_event=True, full_return=False):
         '''
